// |$Id: bnez.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $            |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: gennez.c                                    |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: generate a Northing, Easting, Elevation (*.nez)   |
   |          survey file based on regular geometry input       |
   |          parameters.                                       |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>


//...parameter definitions

//...function prototypes

// Compute exbar parameters and draw progress bar
      void bargrid(int ntraces, float *bar, int *ibar, int *invbar);

// Write a progress bar step for jrec into ntraces
      void exbar(int ntraces, float bar, int ibar, int invbar, int jrec);

// Print Copyright label, time, date to screen
      void cr_labl(char pid[]);

// Print Help Panel to screen
      void hlp_labl(char msg1[],  char  msg2[]);

// Print Copyright label, time, date to file h3
      void fcr_labl(char pid[], FILE *h3);


      int  getparm(int argc,char *argv[],
         char ofile[], int *npoints, char tag[], int *so, float *yo,
	 float *xo, float *zo, float *dy, float *dx, float *dz, int *ido,
	 int *did);

      void outlst(int npoints, char tag[], int so, float yo, float xo, 
	float zo, float dy, float dx, float dz, int ido, int did,
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h2, *h3;
//    int               jt,
      int               jrec,
//                      npts,
//                      ntraces,
                        ibar,
                        invbar;
//      float           fsamin,
	float           bar,
                        *s1,
                        *s2;
      char              pid[5]="bnez",   //process id name (4char)
                        ofile[40],
                        lstfile[13]="bnez.lst";

//...specific to this application
      int               npoints,
   	 		s,
			so,
			id,
			ido,
			did;
      char		tag[3]="VP";
      float             yo,
			xo,
			zo,
			dy,
			dx,
			dz,
			y,
			x,
			z;

//...get parameters
      if(getparm(argc,argv,
         ofile,&npoints,tag,&so,&yo,&xo,&zo,&dy,&dx,&dz,&ido,&did)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(npoints,tag,so,yo,xo,zo,dy,dx,dz,ido,did,pid,ofile,lstfile,h3);

//...open output file
        if( (h2=fopen(ofile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(npoints,&bar,&ibar,&invbar);

//...points loop
	s=so;
	y=yo;
	x=xo;
	z=zo;
	id=ido;

      for (jrec=0;jrec<npoints;jrec++)
     {
//...output a record
	fprintf(h2,"%11d %11f %11f %11f %2s%3.3d\n",s,y,x,z,tag,id);	
//...increment survey parameters
	s=s+1;
	y=y+dy;
	x=x+dx;
	z=z+dz;
	id=id+did;

//...display progress bar
      exbar(npoints,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d points",npoints);
        return(0);

        puts(" ");
        free(s2);
        free(s1);
        fclose(h2);
exit_1:
        fclose(h3);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         char ofile[], int *npoints, char *tag, int *so, float *yo,
	 float *xo, float *zo, float *dy, float *dx, float *dz, int *ido,
	 int *did)
     {
      char		help[3]="-h";
      char              msg1[46],msg2[46];
      char		pid[5]="bnez";
      int		itag;	

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Generate Northing, Easting, Elevation *.nez  ");
      sprintf(msg2,"file from regular geometry input parameters  ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  outfile  n-points tag so yo xo zo ido dy dx dz did \n",pid);
      puts(" ");
      puts("     outfile  = output file name (ex. aaaa0001.nez");
      puts(" ");
      puts("     n-points = number of survey points to generate ");
      puts("     tag      = 1 tag=VP");
      puts("              = 2 tag=SP");
      puts("     so       = first value of sequence number ");
      puts("     yo       = northing of first point ");
      puts("     xo       = easting of first point  ");
      puts("     zo       = elevation of first point ");
      puts("     ido      = initial ID number        ");
      puts("     dy       = spacing between points in north direction ");
      puts("     dx       = spacing between points in east direction  ");
      puts("     dz       = spacing in elevation between points  ");
      puts("     did      = interval in ID between points  ");
      puts("      ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strcpy(ofile,argv[1]);
       else
      {
       puts(" Enter OUTPUT file name ");
       scanf(" %13s",ofile);
      }
	if (namecheck(12,ofile) != 0 ) return (1);

//...convert string to int
      if(argc>2)
       *npoints=atoi(argv[2]);
       else
      {
       puts(" Enter number of points, (int)");
       scanf(" %d",npoints);
      }

//... string
      if(argc>3)
       itag=atoi(argv[3]);
       else
      {
       puts(" Enter tag: 1=VP 2=SP");
       scanf(" %d",&itag);
      }
	if (itag == 1 ) 
	sprintf(tag,"VP");
	else
	sprintf(tag,"SP");

//...convert string to int
      if(argc>4)
       *so=atoi(argv[4]);
       else
      {
       puts(" Enter so (initial sequence number, int)");
       scanf(" %d",so);
      }

//...convert string to float
      if(argc>5)
       *yo=atof(argv[5]);
       else
      {
       puts(" Enter yo, initial northing point (float)");
       scanf(" %f",yo);
      }

//...convert string to float
      if(argc>6)
       *xo=atof(argv[6]);
       else
      {
       puts(" Enter xo, initial easting point (float)");
       scanf(" %f",xo);
      }

//...convert string to float
      if(argc>7)
       *zo=atof(argv[7]);
       else
      {
       puts(" Enter zo, initial elevation point (float)");
       scanf(" %f",zo);
      }

//...convert string to int
      if(argc>8)
       *ido=atoi(argv[8]);
       else
      {
       puts(" Enter ido, initial ID name for first point (int)");
       scanf(" %d",ido);
      }

//...convert string to float
      if(argc>9)
       *dy=atof(argv[9]);
       else
      {
       puts(" Enter dy, spacing between north points (float)");
       scanf(" %f",dy);
      }

//...convert string to float
      if(argc>10)
       *dx=atof(argv[10]);
       else
      {
       puts(" Enter dx, spacing between east points (float)");
       scanf(" %f",dx);
      }

//...convert string to float
      if(argc>11)
       *dz=atof(argv[11]);
       else
      {
       puts(" Enter dz, spacing between elevation points (float)");
       scanf(" %f",dz);
      }

//...convert string to int 
      if(argc>12)
       *did=atoi(argv[12]);
       else
      {
       puts(" Enter did, ID interval points (int)");
       scanf(" %d",did);
      }

       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int npoints, char tag[], int so, float yo, float xo,
        float zo, float dy, float dx, float dz, int ido, int did,
	char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     npoints= %d ",npoints);
      fprintf(h3,"\n     tag    = %s ",tag);
      fprintf(h3,"\n     so     = %d ",npoints);
      fprintf(h3,"\n     yo     = %9.4f ",yo);
      fprintf(h3,"\n     xo     = %9.4f ",xo);
      fprintf(h3,"\n     zo     = %9.4f ",zo);
      fprintf(h3,"\n     dy     = %9.4f ",dy);
      fprintf(h3,"\n     dx     = %9.4f ",dx);
      fprintf(h3,"\n     dz     = %9.4f ",dz);
      fprintf(h3,"\n     ido    = %d ",ido);
      fprintf(h3,"\n     did    = %d ",did);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
