// |$Id: gensetg.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: gensetg.c                                   |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: Interactive program to create shot and receiver   |
   |          files for program setgeom.c to apply shot and     |
   |          receiver locations (from an *.nez file)           |
   |          to *.seg data sets                                |
   |   This program should be run BEFORE setgeom.c              |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
*/

# include <stdio.h>
# include <string.h>
# include <stdlib.h>
# include "../include/c_bsegy.h"
# include "../include/sub4.h"

  int  main(int argc, char *argv[])
{
	char	nfile[13];
	FILE	*fp1;
//	int	count,
	int	n1s,n2s,
		n1sp,incsp,
		nshots,
		j,k,
		spcount;
	int	nblocks,
		n1b,n2b,
		n1vp,
		incvp;
	int	npts,
		vpcount;
        char    help[3]="-h";
	char    pid[5]="gset";
	
//set channel counters
//	count=1;
	n1b=1;
	
//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);
      puts("INTERACTIVE");
      puts("Generates shots.txt and phones.txt files which");
      puts("direct processing of BSEGY (*.seg) files in");
      puts("concert with a survey (*.nez) file.");
      puts("  ");
      puts("This program should be run BEFORE setgeom  ");
      puts("since setgeom requires the shots.txt and phones.txt files.  ");
      return(0);
     } //endif

//display copyright
      cr_labl(pid);

//open file
	if( (fp1=fopen("shots.txt","w")) == NULL )
	{
	  fprintf(stderr, "Error opening file");
	  exit(1);
	}	//endif

//shots file
	puts("SHOTS: ---------------------------");
	puts(" Enter first shot file NAME number ");
	scanf("%d",&n1s);

	puts(" Enter last shot file NAME number ");
	scanf("%d",&n2s);

	puts(" Enter first SP label NUMBER  ");
	scanf("%d",&n1sp);

	puts(" Enter increment for SP label NUMBER ");
	scanf("%d",&incsp);

//shots loop
	nshots=n2s-n1s+1;
	spcount=n1sp;
	for (j=0; j<nshots; j++)
       {
        sprintf(nfile,"%4d.seg",(j+n1s));
/*	if (spcount < 100) fprintf(fp1,"%s  SP%02d\n",nfile,spcount);
	if (spcount > 99) fprintf(fp1,"%s  SP%3d\n",nfile,spcount);
*/
	fprintf(fp1,"%s  SP%03d\n",nfile,spcount);
	spcount=spcount+incsp;
       }  // next j

	fclose(fp1);

//open file
	if( (fp1=fopen("phones.txt","w")) == NULL )
	{
	  fprintf(stderr, "Error opening file");
	  exit(1);
	}	//endif
//phones file
	puts("PHONES: ---------------------------");

	puts(" Enter number of BLOCKS to define channels ");
	scanf("%d",&nblocks);

	for (j=0; j<nblocks; j++)
       {
	fprintf(stdout,"BLOCK Number------- %d\n",(j+1));
	fprintf(stdout,"Channels (%d) through (?) \n",n1b);
	puts(" Enter last channel for this block ");
	scanf("%d",&n2b);

	puts(" Enter first label VP NUMBER for this block");
	scanf("%d",&n1vp);

	puts(" Enter label VP increment for this block");
	scanf("%d",&incvp);

	npts=n2b-n1b+1;
	vpcount=n1vp;
	for (k=0; k<npts; k++)
       {
/*	if (vpcount < 100) fprintf(fp1,"%02d VP%02d\n",(k+n1b),vpcount);
	if (vpcount > 99) fprintf(fp1,"%02d  VP%3d\n",(k+n1b),vpcount);
*/
	fprintf(fp1,"%02d  VP%03d\n",(k+n1b),vpcount);
	vpcount=vpcount+incvp;
       } // next k

	n1b=n2b+1;

       }  // next j

	fclose(fp1);
	return(0);
}  //endmain

