c |$Id: bkil.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bkil.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |              Removes or Zeros selected traces              |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc   (modified segy header definitions) |
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      integer*2         itrk(500)
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bkil'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +iopt1,iopt2,ntrc,itrk,iftr,iltr)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iopt1,iopt2,ntrc)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      krec=0
      ntr=0
c
c...data loop=======================================....

c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      iskip=0
      do 101 jtst=1,ntrc
      if(jrec.eq.itrk(jtst)) then
      iskip=1
      endif
  101 continue

c
c...zero a trace
      if(iopt1.eq.1 .and. iskip.eq.1 ) then
      do 150 jt=1,npts
 150  s1(jt)=0.0
      endif
c
c...output trace
c
      if(iskip.eq.0 .or. iopt1.eq.1) then
      krec=krec+1
      CALL BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,krec)
      endif
      if(iskip.eq.1) then
        if(iopt1.eq.0) then
        write(io11,'(''   Trace killed  ='',i4)') jrec
        else
        write(io11,'(''   Trace zeroed ='',i4)') jrec
        endif
      endif
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +iopt1,iopt2,ntrc,itrk,iftr,iltr)
c
      character*40      infil
      integer*2         itrk(*)
      character*40      arg1,arg2,arg3,arg4,arg5
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bkil: Kills or zeros traces (specified   |'
      write(*,*) '| individually or by range).               |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)' INDIVIDUAL OPTION-------------'
      write(*,*)
     +'bkil infil iopt1 iopt2 ntrc itr itr ...itr '
      write(*,*)' '
      write(*,*)'infile:    =input file name '
      write(*,*)'iopt1:     =option  0=kill 1=zero traces '
      write(*,*)'iopt2:     =specify traces 0=individual 1=by range'
      write(*,*)'ntr        =number of traces to kill or zero '
      write(*,*)'itr......  =trace numbers to kill or zero '
      write(*,*)' '
      write(*,*)' RANGE OPTION------------------'
      write(*,*)
     +'bkil infil iopt1 iopt2 iftr iltr  '
      write(*,*)' '
      write(*,*)'infile:    =input file name '
      write(*,*)'iopt1:     =option  0=kill 1=zero traces '
      write(*,*)'iopt2:     =specify traces 0=individual 1=by range'
      write(*,*)'iftr       =first trace'
      write(*,*)'iltr       =last trace'
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...input parameters
      if(nargsx.lt.(3+karg)) then
      write(*,*)' enter option:  0=kill trace  1=zero trace'
      read(*,*) iopt1
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') iopt1
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)' enter option:  Specify by 0=individual 1=range'
      read(*,*) iopt2
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') iopt2
      endif
c
      if(iopt2.eq.0) then
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)' enter number of traces to kill or zero'
      read(*,*) ntrc
      else
      call getarg(4,arg4)
      read(arg4,'(i2)') ntrc
      endif
      if(ntrc.gt.500) then
      write(*,*)'  ABORT!!! too many kill or zero traces'
      stop
      endif
c
      do 37 j=1,ntrc
      if(nargsx.lt.(4+karg+j)) then
      write(*,'('' enter trace number ('',i2.2,'') '')') j
      read(*,*) itrk(j)
      else
      call getarg((j+4),arg4)
      read(arg4,'(i4)') itrk(j)
      endif
   37 continue
c
      else
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)' enter first trace'
      read(*,*) iftr
      else
      call getarg(4,arg4)
      read(arg4,'(i5)') iftr
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,*)' enter last trace'
      read(*,*) iltr
      else
      call getarg(5,arg5)
      read(arg5,'(i5)') iltr
      endif
c
      ntrc=iabs(iltr-iftr)+1
      if(ntrc.gt.500) then
      write(*,*)'  ABORT!!! too many kill or zero traces'
      stop
      endif
c
      do 38 j=1,ntrc
   38 itrk(j)=iftr+j-1
      endif
c...///end of iopt2=0 block
      return
      end

c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iopt1,iopt2,ntrc)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bkil  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      if(iopt1.eq.0) then
      write(io11,*)'   Kill Traces'
      else
      write(io11,*)'   Zero Traces'
      endif
      write(io11,'(''   Number of traces to kill or zero='',i4)') ntrc
      write(io11,'(''  IOPT1='',i5)') iopt1
      write(io11,'(''  IOPT2='',i5)') iopt2
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
