c |$Id: bmed.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bmed.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |      Computes median mix of a seismic file                 |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     FMED    Function to compute median value of an array   |
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim),work(ndim*21),x(21)
      character*4       aplc
      character*40      infil, outfil, outlst
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bmed'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +mix)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,mix)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
c
c...data loop=======================================....
c
c...load mix width of traces into memory buffer
      do 50 jrec=1,mix
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      krec=(jrec-1)*npts
      do 45 jt=1,npts
   45 work(krec+jt)=s1(jt)
   50 continue
c
c...median mix buffer
      do 60 jt=1,npts
      do 55 jrec=1,mix
      krec=(jrec-1)*npts
      x(jrec)=work(krec+jt)
   55 continue
      s2(jt)=FMED(x,mix)
   60 continue
c
c...taper in
      ntr=0
      mix2=mix/2+1
      do 100 jrec=1,mix2
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...output median mix
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  100 continue
c
c
      mix2p1=mix2+1
      ntrmix=ntrace-mix2+1
      jkrec=-1
      mixm1=mix-1
c
      do 105 jrec=mix2p1,ntrmix
      jkrec=jkrec+1
      if(jkrec.gt.mixm1) jkrec=0
      kkrec=jkrec*npts
c
c...input a trace
      jlead=jrec+mix2-1
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jlead,iexit)
c
      ntr = ntr + 1
c
c...replace old trace
      do 65 jt=1,npts
  65  work(kkrec+jt)=s1(jt)
c
c...median mix buffer
      do 70 jt=1,npts
      do 75 jrec2=1,mix
      krec=(jrec2-1)*npts
      x(jrec2)=work(krec+jt)
   75 continue
      s2(jt)=Fmed(x,mix)
   70 continue
c
c...get correct headers
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
c...output trace
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  105 continue
c
c...taper out
      ntrmix=ntrmix+1
      do 110 jrec=ntrmix,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...output median mix
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  110 continue
c
      ntrmix=ntrmix-1
      write(io11,900) mix2,ntrmix,ntrace
  900 format(5x,'Traces 1 to ',i5,' are same for taper in'/
     +5x,'Traces ',i5,' to',i5,' are same for taper out')
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +mix)
c
      character*40      infil
      character*40      arg1,arg2
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bmed: Median mix of seismic traces       |'
      write(*,*) '| across the trace direction.              |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bmed infile mix'
      write(*,*) ' '
      write(*,*) 'infile  =input file name '
      write(*,*) 'mix     =mix width <21 '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  enter: mix (mix<=21 and odd)'
      read(*,*) mix
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') mix
      endif
c
      mix=(mix/2)*2+1
      if(mix.gt.21) mix=21
      if(mix.lt.1) mix=1
c
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,mix)                        
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program bmed  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   mix= '',i5)') mix
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
