c |$Id: brev.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: brev.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |              Reverses Polarity or Order of seismic traces  |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      integer*2         ipol(1000)
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''brev'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,ndim,s1,
     +iop1,nflip,ipol)           
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iop1,nflip)           
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c---------option data polarity reversal----------------
      if(iop1.eq.1) then
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...test to flip polarity
      if(ipol(jrec).gt.0) then
      write(io11,'(''     Channel'',i5,'' pol. rev.'')') jrec
      do 150 jt=1,npts
 150  s2(jt)=-(s1(jt))
c...flip header by 180 for geophone
c....horizontal
      if(geover.eq.90) then
      geoazi=geoazi+180
      if(geoazi.gt.360) geoazi=geoazi-360
      endif
c....vertical
      if(geover.ne.90) then
      geover=geover+180
      if(geover.gt.360) geover=geover-360
      endif
c
      else
      do 175 jt=1,npts
 175  s2(jt)=(s1(jt))
      endif
c
c...output trace
      iunit2=2
      call BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c------------iopt-----option block ELSE---------------------
      else  
c
c...read traces, compute------------------------------               
c
c...BEGIN LOOP====================================....
      do 200,jrec=1,ntrace
      krec=(ntrace+1)-jrec
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,krec,iexit)
c
      ntr = ntr + 1
c
c
c...output trace
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
  200 continue
c
c...END LOOP========================================....
c
      endif
c----------iopt----------end option block--------------------
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,ndim,s1,
     +iop1,nflip,ipol)           
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4
      integer*2         ipol(1000)
      real*4            s1(*)
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| brev: Reverses channel order or sample   |'
      write(*,*) '| polarity (specify range or individual)   |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'brev infil iop1 nflip ch1 ch2 ...    '
      write(*,*)' '
      write(*,*)'infil:  =name of input file '
      write(*,*)'iop1            0=reverse channel order'
      write(*,*)'       or           '
      write(*,*)'                1=reverse data polarity'
      write(*,*)'nflip   =number of channels to be polarity reversed'
      write(*,*)'ch1     =number of channel reverse data polarity  '
      write(*,*)'ch2     =number of channel reverse data polarity  '
      write(*,*)'ch3     =number of channel reverse data polarity  '
      write(*,*)'... ch_nflip.  NOTE: if nflip=number of channels,   '
      write(*,*)'     then all channels will be reversed in pol.  )'
      write(*,*)'     (no need to input ch1, ch2,....)'
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...count traces and check dimensions------------------------
      iunit1=1
      icall1=0
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
      close(iunit1)
c
c...input parameters
      if(nargsx.lt.(3+karg)) then
      write(*,*)' Enter option:  0=reverse channel order'
      write(*,*)'                1=reverse data polarity'
      read(*,*) iop1
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') iop1
      endif
c
c...//////////////BEGIN OPTION BLOCK////////////////////////
      if(iop1.eq.1) then
c
      if(nargsx.lt.(4+karg)) then
      write(*,'('' number of channels in input='',i5)') ntrace
      write(*,*)' Enter number of channels to reverse polarity'
      read(*,*) nflip
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') nflip
      endif
      do 43 j=1,ntrace
   43 ipol(j)=0
c
      if(nflip.eq.ntrace) then
      do 44 j=1,ntrace
   44 ipol(j)=j
      else
      do 47 jflip=1,nflip
      if(nargsx.lt.(jflip+4+karg)) then
      write(*,*)'  enter a channel to flip'
      read(*,*) ichan
      ipol(ichan)=ichan
      else
      call getarg((jflip+3+karg),arg4)
      read(arg4,'(i5)') ichan
      ipol(ichan)=ichan
      endif
   47 continue
      endif
c
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iop1,nflip)           
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program brev  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Parameters: '')')
      write(io11,'(''   iop1='',i5)') iop1
      write(io11,'(''   nflip='',i5)') nflip
      if(iop1.eq.1) write(io11,*)'     Reverse Data POLARITY'
      if(iop1.eq.0) write(io11,*)'     Reverse Data ORDER'
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

