c |$Id: bscl.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bscl.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Scales seismic data amplitudes based on a selected option. |
c |                                                            |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
      real*8 sum,datapt
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, ampfil
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bscl'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +itr1,itrn,isw1,scaler,ampfil)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...check for limits on traces
      itrlst=itr1+itrn-1
      if(itrlst.gt.ntrace) itrlst=ntrace
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itr1,itrn,isw1)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      jrec = 0
      ntr = 0
      fmax=-1.0E+20
      fmin=+1.0E+20
      sum=0.0D+00

c
c...data loop=======================================....
      do 500 jrec=itr1,itrlst
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      if(iexit.eq.1) go to 600
      ntr = ntr + 1
c
c...find scale factor
      do 75 jt=1,npts
      datapt=s1(jt)
      sum1=datapt*datapt + sum1
      if(s1(jt).gt.fmax) then
      fmax=s1(jt)
      itrmx=jrec
      ismx=jt
      endif
      if(s1(jt).lt.fmin) then 
      fmin=s1(jt) 
      itrmn=jrec
      ismn=jt 
      endif
  75  continue
500   continue
600   continue
c
c...echo check on scale factor to listing file
      if(isw1.eq.1) then
      scalef=dsqrt(sum1/(dfloat(npts)*dfloat(ntr)))
      write(io11,'(''     L2 Norm of Data Set='',e14.7)') scalef
      endif
      if(isw1.eq.0) then
      write(io11,'(''  User Supplied scaler='',e14.7)') scaler
      if (scaler .ne. 0. ) then  
      scalef=1.0/scaler
      else
      write(*,*) 'REALLY? Zero is a bad idea. '
      stop
      endif
      endif
      if(isw1.eq.3) then
      scalef=max(abs(fmax),abs(fmin))
      write(io11,'(''  Peak Absolute Value='',f24.10)') scalef
      endif

c
      jrec = 0
      ntr = 0
c
c...BEGIN LOOP====================================....
      if(isw1.eq.2) open(12,file=ampfil,access='sequential')
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      if(isw1.eq.2) then
      read(12,*) jjj,scaler
      write(io11,'(''trace='',i5,'' factor='',e14.7)') jjj,scaler
      if(scaler.ne.0.0) then
      scalef=1.0/scaler
      else
      write(*,*) 'REALLY? Zero is a bad idea. '
      stop
      endif
      endif
c
c...apply factor
      do 80 jt=1,length
  80  s2(jt)=(s1(jt)/scalef)
c
c...output trace
      iunit2=2
      call BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c...output statistics
      write(io11,'(2x,65(1h-))')
      write(io11,250) fmax,itrmx,ismx,fmin,itrmn,ismn
250   format(5x,'Maximum Value= ',e10.4,5x,'Trace #=',i5/
     +35x,'Sample #=',i5/
     +5x,'Minimum Value= ',e10.4,5x,'Trace #=',i5/ 
     +35x,'Sample #=',i5/2x,65(1h-))
      rscale=1.0/scalef
      write(io11,251) rscale
251   format(5x,'Scale Factor= ',e14.7)
      fmax=fmax/scalef
      fmin=fmin/scalef
      write(io11,'(//'' =====New Maximum and Minimum===='')')
      write(io11,250) fmax,itrmx,ismx,fmin,itrmn,ismn
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(12)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +itr1,itrn,isw1,scaler,ampfil)  
c
      character*40      infil, ampfil
      character*40      arg1,arg2,arg3,arg4,arg5
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bscl: Scale seismic traces (with either  |'
      write(*,*) '| a user provided factor or data measured).|'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bscl infil itr1 itrn isw1 [ scaler | ampfil ] '
      write(*,*)' '
      write(*,*)'infil:   =input file name '
      write(*,*)'itr1:    =starting trace for determination window'
      write(*,*)'itrn:    =number of traces to include in window'
      write(*,*)' '
      write(*,*)'isw1:   0=user supplied scale factor  '
      write(*,*)'        1=scale factor from 1/L2 norm '
      write(*,*)'        2=input file with (trace,ampfactor)'
      write(*,*)'        3=scale factor from Max Abs Value '
      write(*,*)'scaler:  =user supplied scale factor (ONLY isw1=0)  '
      write(*,*)'ampfil:  file name for option (ONLY isw1=2)       '
      write(*,*)' -  - - - - - - - - - - - - - - - - - - - - - - - - '
      write(*,*)' | Converting microvolts to m/s particle velocity | '
      write(*,*)' |                                                | '
      write(*,*)' |                      VSP Ref.Phone 28Hz Oyo    | '
      write(*,*)' |     scalar=4.0978E-8 28Hz Oyo SMC 28-720       | '
      write(*,*)' |     scalar=5.6497E-8 14Hz Oyo Phone            | '
      write(*,*)' |     scalar=5.0761E-8 10Hz Oyo GS-20DM Phone    | '
      write(*,*)' |     scalar=3.2787E-8 10Hz Mark L10-A Phone     | '
      write(*,*)' |     scalar=3.2034E-8 08Hz Mark L10-A Phone     | '
      write(*,*)' |                                                | '
      write(*,*)' |     strain=(part.vel.)/(wave phase vel.)       | '
      write(*,*)' -  - - - - - - - - - - - - - - - - - - - - - - - - '
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...enter input parameter
      if(nargsx.lt.(3+karg)) then
      write(*,*)' scale factor window: enter starting trace #'
      read(*,*) itr1
      else
      call getarg(2,arg2)
      read(arg2,'(i1)') itr1
      endif
      if(itr1.lt.1) itr1=1
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)' scale factor window: enter number of traces'
      read(*,*) itrn
      else
      call getarg(3,arg3)
      read(arg3,'(i2)') itrn
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)' enter option:0=user supplied scale factor'
      write(*,*)'              1=1/L2 norm of data ==> factor'
      write(*,*)'              2=file with (trace,factor) pairs'
      write(*,*)'              3=Max Abs Value ==> factor'
      read(*,*) isw1
      else
      call getarg(4,arg4)
      read(arg4,'(i1)') isw1
      endif
c
      if(isw1.eq.0) then
      if(nargsx.lt.(6+karg)) then
      write(*,*)' Enter scale factor to multipy data by.  Note:'
      write(*,*)' -  - - - - - - - - - - - - - - - - - - - - - - - - '
      write(*,*)' | Converting microvolts to m/s particle velocity | '
      write(*,*)' |                                                | '
      write(*,*)' |     scalar=5.6497E-8 14Hz Oyo Phone            | '
      write(*,*)' |     scalar=5.0761E-8 10Hz Oyo GS-20DM Phone    | '
      write(*,*)' |     scalar=3.2787E-8 10Hz Mark L10-A Phone     | '
      write(*,*)' |                                                | '
      write(*,*)' |     strain=(part.vel.)/(wave phase vel.)       | '
      write(*,*)' -  - - - - - - - - - - - - - - - - - - - - - - - - '
      write(*,*)' '
      read(*,*) scaler
      else
      call getarg(5,arg5)
      read(arg5,'(f10.4)') scaler
      endif
      endif
c
      if(isw1.eq.2) then
      if(nargsx.lt.(6+karg)) then
      write(*,*)'  Enter file name'
      read(*,'(a13)') ampfil
      else
      call getarg(5,ampfil)
      endif
      call namechk(12,ampfil)
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itr1,itrn,isw1)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bscl'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''     Scale Factor Determination Window'')')
      write(io11,'(''     Starting Trace #= '',i5)') itr1
      write(io11,'(''     Number of Traces=  '',i5)') itrn
      if(isw1.eq.1) write(io11,'(''     1/L2 norm scale factor '')')
      if(isw1.eq.0) write(io11,'(''     User Supplied factor '')')
      if(isw1.eq.3) write(io11,'(''     Peak Amp. scale factor'')')
      write(io11,'(''  '')')
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
