c |$Id: bwin.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bwin.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |       Windows b-segy data traces with raised cosine window |
c |        t1=start time for taper on                          |
c |        t2=window at maximum=1.0                            |
c |        t3=end time for maximum, begin taper off            |
c |        t4=end of window, amp=0.0                           |
c |                                                            |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim),windo(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bwin'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +tw1,tw2,tw3,tw4)         
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,tw1,tw2,tw3,tw4)            
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
c
c...generate window
      jw1=int(tw1/fsamin)+1
      jw2=int(tw2/fsamin)+1
      jw3=int(tw3/fsamin)+1
      jw4=int(tw4/fsamin)+1
      if(jw1.lt.1)    jw1=1
      if(jw2.lt.jw1)  jw2=jw1
      if(jw3.lt.jw2)  jw3=jw2
      if(jw4.lt.jw3)  jw4=jw3
      if(jw4.gt.npts) jw4=npts
c
      if(jw1.gt.1) then
      do 75 j=1,jw1
      windo(j)=0.
   75 continue
      endif
c
      if(jw2.gt.jw1) then
      dangle=3.1415926/float(jw2-jw1)
      dingle=3.1415926
      do 77 j=jw1,jw2
      windo(j)=(1.0+cos(dingle))/2.0
   77 dingle=dingle+dangle
      endif
c
      if(jw3.gt.jw2) then
      do 79 j=jw2,jw3
      windo(j)=1.0
   79 continue
      endif
c
      if(jw4.gt.jw3) then
      dangle=3.1415926/float(jw4-jw3)
      dingle=0.0
      do 80 j=jw3,jw4
      windo(j)=(1.0+cos(dingle))/2.0
   80 dingle=dingle+dangle
      endif
c
      if(npts.gt.jw4) then
      do 82 j=jw4,npts
   82 windo(j)=0.0
      endif

      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c
c...window samples                  
      do 150 jt=1,npts
 150  s2(jt)=s1(jt)*windo(jt)
c
c
c...output trace
      iunit2=2
      call BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +tw1,tw2,tw3,tw4)         
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bwin: Applies a window in time to seismic|'
      write(*,*) '| traces (use to taper in / out )          |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bwin infile tw1 tw2 tw3 tw4 '
      write(*,*)' '
      write(*,*)'infile  =input file name '
      write(*,*)'tw1      =time of start taper on, amp=0 '
      write(*,*)'tw2      =time of taper off,      amp=1 '
      write(*,*)'tw3      =time of start taper off amp=1 '
      write(*,*)'tw4      =time of taper off,      amp=0 '
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...read parameters
      if(nargsx.lt.(3+karg)) then
      write(*,*)' enter window start time, tw1'
      read(*,*)  tw1
      else
      call getarg(2,arg2)
      read(arg2,'(f8.4)') tw1
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)' enter window end-taper time, tw2'
      read(*,*)  tw2
      else
      call getarg(3,arg3)
      read(arg3,'(f8.4)') tw2
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)' enter window start-taper time, tw3'
      read(*,*)  tw3
      else
      call getarg(4,arg4)
      read(arg4,'(f8.4)') tw3
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,*)' enter window off time, tw4'
      read(*,*)  tw4
      else
      call getarg(5,arg5)
      read(arg5,'(f8.4)') tw4
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,tw1,tw2,tw3,tw4)            
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program master'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''    Window Start Time, t1='',f10.4)') tw1
      write(io11,'(''    Window Taper Off Time, t2='',f10.4)') tw2
      write(io11,'(''    Window Taper On  Time, t3='',f10.4)') tw3
      write(io11,'(''    Window End   Time, t4='',f10.4)') tw4
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

