! |$Id: caplot.F90,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $      |
! --------------------------------------------------------------
! | Basic Seismic Utilities (BSU)                              |
! | LANGUAGE: Fortran77                                        |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: caplot.f                                     |
! | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
! | MODIFICATION: Double precision for plplot                  |
! |------------------------------------------------------------|
! | Plots bvas.his and bamp.his files from runs of bvas.f and  |
! | bamp.f (for joint inversion of dispersion and decay).      |
! | Files bvas.his and bamp.his should be in the directory     |
! | and are the resulting measurements of dispersion and decay.|
! | Also shown are a plot of semblance values at each frequency|
! |                                                            |
! |------------------------------------------------------------|
! |  NOTICE:                                                   |
! |  Copyright (C) 2017 Paul Michaels                          |
! |  <pm@cgiss.boisestate.edu>                                 |
! |  This program is free software; you can                    |
! |  redistribute it and/or modify it under the terms          |
! |  of the GNU General Public License as published            |
! |  by the Free Software Foundation; either version           |
! |  3 of the License, or (at your option) any later           |
! |  version.  This program is distributed in the              |
! |  hope that it will be useful, but                          |
! |  WITHOUT ANY WARRANTY; without even the implied            |
! |  warranty of MERCHANTABILITY or FITNESS FOR A              |
! |  PARTICULAR PURPOSE.  See the GNU General Public           |
! |  License for more details.                                 |
! |  You should have received a copy of the GNU                |
! |  General Public License along with this program;           |
! |  if not, write to the Free Software Foundation,            |
! |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
! |------------------------------------------------------------|
! |  LIBRARY REQUIRED:                                         |
! |                                                            |
! |PLPLOT is a GNU plot library which supports both C and      |
! |Fortran calls.  The distribution may be downloaded from     |
! |http://plplot.sourceforge.net                               |
! |                                                            |
! |------------------------------------------------------------|
! |  SUBROUTINES CALLED:                                       |
! | In-line===============                                     |
! |     GETPRM  get input parameters from command line or      |
! |             interactive query.  Should be inline with      |
! |             main program, specific to application.         |
! |     VPLOT3  Plots velocity vs frequency (top of page)      |
! |             Plots semblance vs frequency (middle of page)  |
! |     APLOT3  Plots amplitude decay vs frequency (bottom)    |
! |                                                            |
! |------------------------------------------------------------|
!
#ifdef PLPLOT
      use plplot
#endif
      parameter (mdim=256)
!
      real*8            frq(mdim),cvel(mdim),semblx(mdim),sigmam(mdim)
      real*8            svfrq(mdim),svamp(mdim),svvar(mdim)
      character*4       lineid
      character*40      infil,infil2,outfil
      integer*2         tyear,tday
      character*1024    cmd
      integer*1         bz
#ifdef PLPLOT
      real*8            fmargid
#endif

!
!...get parameters-------------------------------------------
      call GETPRM(nargsx,infil, &
      infil2,emin,emax,lineid,tyear,tday,idev)
      fmargin=.02
!
!...bvas history file
      open(unit=3,file=infil,access='sequential')
!
!...read bvas.his measurements
      ncfilt=0
      do 50, j=1,mdim
      read(3,FMT=*,end=55,err=55) &
      frq(j),cvel(j),sigmam(j),semblx(j),tbar,tvar
      ncfilt=ncfilt+1
   50 continue
   55 continue
      close(3)
!
!...bamp history file
      open(unit=4,file=infil2,access='sequential')
!
!...read bvas.his measurements
      bar=float(mdim)/50.
      ibar=int(bar+0.5)
      invbar=int(1./bar+0.5)
!
      nafilt=0
      do 60, j=1,mdim
      read(4,FMT=*,end=65,err=65) svfrq(j),svamp(j),svvar(j)
      nafilt=nafilt+1
      call pltbar(bar,invbar,ibar,j,ncfilt)
   60 continue
   65 continue
      close(4)
!
      SELECT CASE (idev)
      CASE (0)
#ifdef PLPLOT
      write(outfil,'("X window plot, press enter to end")')
#else
      write(outfil,'("Press q (focus in plot) to end")')
#endif 
      CASE (1)
      write(outfil,'("caplot.ps")')
      CASE (2)
      write(outfil,'("caplot.fig")')
      CASE (3)
      write(outfil,'("caplot.jpeg")')
      CASE (4)
      write(outfil,'("caplot.pdf")')
      END SELECT
!
      write(*,'(" ")')
      write(6,240) ncfilt,outfil
240   format(' Samples Processed = ',i5/ &
      5x,'====> output in ',a)
!
!...summary plot
!...copy to double
#ifdef PLPLOT
      fmargid=fmargin
      call vplot3(ncfilt,frq,cvel,sigmam,semblx,emin,emax, &
      lineid,tyear,tday,idev,fmargid)
!
!...generate summary plot
      call aplot3(nafilt,svfrq,svamp,svvar,emin,emax, &
      lineid,tyear,tday)
#else
      call gvplot3(ncfilt,frq,cvel,sigmam,semblx,emin,emax, &
      lineid,tyear,tday,idev)
!
!...generate summary plot
      call gaplot3(nafilt,svfrq,svamp,svvar,emin,emax, &
      lineid,tyear,tday)
#endif
!
#ifndef PLPLOT
      bz=0
      write(cmd,'("gnuplot caplot.gp",a)') bz
      call runcmd(cmd)
#endif

      stop
      end
!
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: getprm.f                                     |
! |------------------------------------------------------------|
! | get input parameters or help                               |
! --------------------------------------------------------------
!
      subroutine GETPRM(nargsx,infil, &
      infil2,emin,emax,lineid,tyear,tday,idev)
!
      character*40      infil,infil2
      character*20      arg1,arg2
      character*4       lineid
      integer*2         tyear,tday
!
!...get input/output file names...
!   |--------------------------------------------|
!   | file naming convention                     |
!   | application prefix 4 characters "bxxx"     |
!   | data set suffix 4 characters "dddd"        |
!   | file name = bxxxdddd.seg                   |
!   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
!
!cc---LINUX VERSION
      nargsx=iargc()
      karg=-1
!...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| caplot:  Plot results of bvas and bamp   |'
      write(*,*) '| runs (velocity dispersion and decay)     |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*) &
      'caplot  bvas_file bamp_file emin emax well year date idev '
      write(*,*) ' '
      write(*,*) 'bvas_file =input file ( bvas.his) '
      write(*,*) 'bamp_file =input file ( bamp.his) '
      write(*,*) ' '
      write(*,*) 'Title info from bvas and bamp runs: '
      write(*,*) ' emin     =minimum elevation (real) '
      write(*,*) ' emax     =maximum elevation (real) '
      write(*,*) ' well     =well name (char 4) '
      write(*,*) ' year     = year of survey (integer, 4 digits)'
      write(*,*) ' date     = 4 digit integer mmdd'
      write(*,*) ' idev     =device for plotting '
      write(*,*) '         0=X window display '
      write(*,*) '         1=Post Script *.ps file'
      write(*,*) '         2=Xfig *.fig file '
      write(*,*) '         3=JPEG *.jpeg file '
      write(*,*) '         4=PDF  *.pdf  file '
!     write(*,*) ' marg     =normalized margin around plot '
!     write(*,*) '          (keep it small, like .02 or zero)'
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of bvas file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
!
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  enter: name of bamp file'
      read(*,'(a13)') infil2
      else
      call getarg(2,infil2)
      endif
      call namechk(12,infil2)
!
      if(nargsx.lt.(4+karg)) then
      write(*,*) '  enter: emin (minimum elevation surveyed) '
      read(*,*) emin
      else
      call getarg(3,arg2)
      read(arg2,'(f8.0)') emin
      endif
!
      if(nargsx.lt.(5+karg)) then
      write(*,*) '  enter: emax (maximum elevation surveyed) '
      read(*,*) emax
      else
      call getarg(4,arg2)
      read(arg2,'(f8.0)') emax
      endif
!
      if(nargsx.lt.(6+karg)) then
      write(*,*) '  enter: well ID (4char)  '
      read(*,'(a)') lineid
      else
      call getarg(5,lineid)
      endif
!
      if(nargsx.lt.(7+karg)) then
      write(*,*) '  enter: year (4digit int, 2002 for example) '
      read(*,*) tyear
      else
      call getarg(6,arg2)
      read(arg2,'(i4)') tyear
      endif
!
      if(nargsx.lt.(8+karg)) then
      write(*,*) '  enter: date (4digit int, mmdd, 0325 for example)'
      read(*,*) tday
      else
      call getarg(7,arg2)
      read(arg2,'(i4)') tday
      endif
!
      if(nargsx.lt.(9+karg)) then
      write(*,*) '  enter: device for plotting '
      write(*,*) '         0=X window display '
      write(*,*) '         1=Post Script *.ps file'
      write(*,*) '         2=Xfig *.fig file '
      write(*,*) '         3=JPEG *.jpeg file '
      write(*,*) '         4=PDF *.pdf file '
      read(*,*) idev
      else
      call getarg(8,arg2)
      read(arg2,'(i1)') idev
      endif
!
!     if(nargsx.lt.(10+karg)) then
!     write(*,*) '  enter: margin around plot  '
!     write(*,*) '  (keep it small, like .02 or zero)'
!     read(*,*) fmargin
!     else
!     call getarg(9,arg2)
!     read(arg2,'(f8.0)') fmargin
!     endif
!
      return
      end

#ifdef PLPLOT
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: vplot3.f                                     |
! |------------------------------------------------------------|
! | velocity  vs. frequency (top plot on the page)             |
! | semblance vs. frequency (middle plot on  page)             |
! |   x = frequency                                            |
! |   y = phase velocity                                       |
! |   sigmam = 1 standard deviation, phase velocity            |
! |   semblx = semblance values for velocity determination     |
! --------------------------------------------------------------
!
      subroutine vplot3(npts,x,y,sigmam,semblx,emin,emax, &
      lineid,tyear,tday,idev,fmargin)
      use plplot 
      real*8 x(npts),y(npts),sigmam(npts),semblx(npts)
      real*8 fdef,fscale
      real*8 fmargin,aspect,yjust,xjust
      real*8 xmin,xmax,ymin,ymax,ymin2,ymax2
      real*8 xx(npts),y1(npts),y2(npts)
      real*8  fwidth
!
      character*14 pltfil
      character*80 titl
      character*40 xlbl
      character*40 ylbl
      character*4 lineid
      integer*2 tyear,tday
      data ifirst /1/

!
      izero=0
      ione=1
!
!...open device
!      if (ifirst.eq.1) then
!  Portrait
      call plscolor(izero)
      call plsori(ione)
      SELECT CASE (idev)
!  Landscape for XWIN device
      CASE (0)
        call plsori(izero)
        call plsdev('xwin')
        aspect=0.00
        yjust=0.0
        xjust=0.0
      CASE (1)
!...aspect ratio needed for portrait, unless you don't mind
!   plots which are squished in the vertical dimension.
!   aspect=8.5/11.0
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('ps')
          write(pltfil,'("caplot.ps")')
        CASE (2)
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('xfig')
          write(pltfil,'("caplot.fig")')
        CASE (3)
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('jpgqt')
          write(pltfil,'("caplot.jpeg")')
        CASE (4)
          aspect=0.77
          yjust=0.5
          xjust=0.0
          call plsdev('pdf')
          write(pltfil,'("caplot.pdf")')
       ENDSELECT
!
      call plsdidev(fmargin,aspect,yjust,xjust)
!
      if(idev.gt.0) then
      call plsfnam(pltfil)
      endif
!
!...initialize plot
      nx=1
      ny=3
      call plinit()
      call plssub(nx,ny)
      call plfontld(izero)
      ifirst=0
!
!-------end ifirst test-----------------
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      ymin2=semblx(1)
      ymax2=semblx(1)
      do 10, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
      ymin2=min(semblx(j),ymin2)
      ymax2=max(semblx(j),ymax2)
! set standard scales
      xmin=min(xmin,0.0)
!     ymax=max(ymax,1000)
      ymax=1000.0
      ymin=min(ymin,0.0)
      ymin2=min(ymin2,0.0)
!     ymax2=max(ymax2,1.0)
      ymax2=1.0
  10  continue
!
!...set environment, window and viewport
      izero=0
      ione=1
      fwidth=1.0
      iwidth=1
      fdef=0.
      fscale=0.9
      call plschr(fdef,fscale)
      fscale=0.6
      call plssym(fdef,fscale)
      call plenv(xmin,xmax,ymin,ymax,izero,izero)
      write(titl,'(''Velocity (95%conf)'', &
      '' Elevations: '',f6.2,'' to '', &
      f6.2,'' m ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       emin,emax,lineid,tyear,tday
      write(xlbl,'(''Frequency (Hz) '')')
      write(ylbl,'(''Phase Velocity (m/s)'')')
      call pllab(xlbl,ylbl,titl)
      icode=9
      call plpoin(x,y,icode)
!
!...plot error bars for velocity, 95% confidence
      do 50, j=1,npts
      xx(j)=x(j)
      y1(j)=y(j)+sigmam(j)*1.96
      y2(j)=y(j)-sigmam(j)*1.96
!      call plerry(xx,y1,y2)
   50 continue
      call plerry(xx(1:npts),y1(1:npts),y2(1:npts))
!
!...plot semblance values
      fwidth=1.0
      iwidth=1
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      call plenv(xmin,xmax,ymin2,ymax2,0,0)
      write(titl,'(''Semblance Values'', &
      '' Elevations: '',f6.2,'' to '', &
      f6.2,'' m ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       emin,emax,lineid,tyear,tday
      write(xlbl,'(''Frequency (Hz) '')')
      write(ylbl,'(''Semblance (unitless)'')')
      call pllab(xlbl,ylbl,titl)
      fwidth=2.0
      iwidth=2
#ifdef PLPNEW
       call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      call plline(x,semblx)
      return
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: aplot3.f                                     |
! |------------------------------------------------------------|
! | amplitude vs. frequency (3rd plot, at bottom of page)      |
! |   x = frequency in hz                                      |
! |   y = amplitude decay after sph.div. correction nepers/m   |
! |  yvar=one standard deviation                               |
! --------------------------------------------------------------
!
      subroutine aplot3(npts,x,y,yvar,emin,emax, &
      lineid,tyear,tday)
      use plplot
      real*8 x(npts),y(npts),yvar(npts)
      real*8 xmin,xmax,ymin,ymax
      real*8 xx(npts),y1(npts),y2(npts)
      real*8 fwidth
      character*80 titl
      character*40 xlbl
      character*40 ylbl
      character*4 lineid
      integer*2 tyear,tday
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 10, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
  10  continue
!...set standard scales
      xmin=min(xmin,0.)
      ymin=min(ymin,0.)
!     ymax=max(ymax,0.5)
      if(ymax.lt.0.5) then
      ymax=0.5
      else
      if(ymax.lt.1.0) then
      ymax=1.0
      else
      if(ymax.lt.1.5) then
      ymax=1.5
      else
      ymax=2.0
      endif
      endif
      endif
!
!...set environment, window and viewport
      fwidth=1.0
      iwidth=1
      izero=0
      fzero=0.
#ifdef PLPNEW
      call plwidth(fwidth)
#else
      call plwid(iwidth)
#endif
      call plenv(xmin,xmax,ymin,ymax,izero,izero)
      write(titl,'(''Decay (95%conf) '', &
      ''Elev. '',f6.2,'' to '', &
      f6.2,'' m ['',a,1x,i4.4,''.'',i4.4,'']'')') &
       emin,emax,lineid,tyear,tday
      write(xlbl,'(''Frequency Hz '')')
      write(ylbl,'(''Decay (1/m)'')')
      call pllab(xlbl,ylbl,titl)
      icode=9
      call plpoin(x,y,icode)
!
!...errorbars for 95% confidence
      ione=1
      do 50, j=1,npts
      xx(j)=x(j)
      y1(j)=y(j)+yvar(j)*1.96
      y2(j)=y(j)-yvar(j)*1.96
!     call plerry(xx,y1,y2)
   50 continue
      call plerry(xx(1:npts),y1(1:npts),y2(1:npts))
!
!...close plot
      call plend()
      return
      end
#else
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: gvplot3.f                                    |
! |------------------------------------------------------------|
! | velocity  vs. frequency (top plot on the page)             |
! | semblance vs. frequency (middle plot on  page)             |
! |   x = frequency                                            |
! |   y = phase velocity                                       |
! |   sigmam = 1 standard deviation, phase velocity            |
! |   semblx = semblance values for velocity determination     |
! --------------------------------------------------------------
!
      subroutine gvplot3(npts,x,y,sigmam,semblx,emin,emax, &
      lineid,tyear,tday,idev)
      real*8 x(npts),y(npts),sigmam(npts),semblx(npts)
      real*8 xmin,xmax,ymin,ymax,ymin2,ymax2
      real*8 testy
!
      character*4 lineid
      character*80 sxrange,syrange,syrange2
      integer*2 tyear,tday
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      ymin2=semblx(1)
      ymax2=semblx(1)
      do 5, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
      ymin2=min(semblx(j),ymin2)
      ymax2=max(semblx(j),ymax2)
! set standard scales
      xmin=min(xmin,0.0)
!     ymax=max(ymax,1000)
      ymax=1000.0
      ymin=min(ymin,0.0)
      ymin2=min(ymin2,0.0)
!     ymax2=max(ymax2,1.0)
      ymax2=1.0
   5  continue

! overides
      testy=ymax
      xmin=0.0
      ymin=0.0
      if (testy.LT.2000.0) ymax=2000.0
      if (testy.LT.1000.0) ymax=1000.0
      if (testy.LT.500.0) ymax=500.0
      ymin2=0.0
      ymax2=1.0
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax
      write(syrange2,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax2
!
      open(unit=12,file='caplot.gp',access='sequential')
      write(12,*) "# Generated by caplot.F90  Subroutine gvplot3()   "
      if (idev.eq.0 ) then
#ifdef WIN
      write(12,*) "set terminal wxt persist "
#else
      write(12,*) "set terminal x11 persist "
#endif
      endif
      if (idev.eq.1 ) then
      write(12,*) "set terminal postscript size 7.0,10. in \"
      write(12,*) "portrait enhanced font 'Helvetica,10' "
      write(12,*) "set output 'caplot.ps'                            "
      endif
      if (idev.eq.2 ) then
      write(12,*) "set terminal fig monochrome size 7.0,10.0 in \"
      write(12,*) "portrait  "
      write(12,*) "set output 'caplot.fig'                           "
      endif
      if (idev.eq.3 ) then
      write(12,*) "set terminal jpeg enhanced nocrop size 700,1000 "
      write(12,*) "set output 'caplot.jpeg'                           "
      endif
      if (idev.eq.4 ) then
      write(12,*) "set terminal pdf color enhanced size 7.0,10.0 in "
      write(12,*) "set output 'caplot.pdf'                           "
      endif
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Frequency (Hz)'                       "
      write(12,*) "set ylabel 'Phase Velocity (m/s)                  "
      write(12,*) "set title 'Velocity (95% Conf) Elevations: \"
      write(12,'(F6.2," to ",F6.2,"\")') emin,emax 
      write(12,*) "meters \"
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " '                                                "
      write(12,*) "set multiplot layout 3,1 columnsfirst downwards \"
      write(12,*) "scale .9,.8  offset -.03,-.02                     "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set tmargin 1                                     "
      write(12,*) "set bmargin 0                                     "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2):($3*1.96) w yerrorbars \"
      write(12,*) "pt 6 ps 1                                         "
      do 10, j=1,npts
   10 write(12,'(3(E14.7,1x))') x(j),y(j),sigmam(j)
      write(12,*) "e "
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Frequency (Hz)'                       "
      write(12,*) "set ylabel 'Semblance (unitless)                  "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange2 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set bmargin 1                                     "
      write(12,*) "set title 'Semblance Values Elevations: \"
      write(12,'(F6.2," to ",F6.2,"\")') emin,emax 
      write(12,*) "meters \"
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " ' "  
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2) ls 1 w l                     "
      do 20, j=1,npts
   20 write(12,'(3(E14.7,1x))') x(j),semblx(j)
      write(12,*) "e "
      return
      end
! --------------------------------------------------------------
! | Basic Seismic Utilities                                    |
! | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       |
! | PROGRAM NAME: gaplot3.f                                    |
! |------------------------------------------------------------|
! | amplitude vs. frequency (3rd plot, at bottom of page)      |
! |   x = frequency in hz                                      |
! |   y = amplitude decay after sph.div. correction nepers/m   |
! |  yvar=one standard deviation                               |
! --------------------------------------------------------------
!
      subroutine gaplot3(npts,x,y,yvar,emin,emax, &
      lineid,tyear,tday)
      real*8 x(npts),y(npts),yvar(npts)
      real*8 xmin,xmax,ymin,ymax,testy
      character*4 lineid
      integer*2 tyear,tday
      character*80 sxrange,syrange
!
!...determine maximum and minimums
      xmin=x(1)
      ymin=y(1)
      xmax=x(1)
      ymax=y(1)
      do 5, j=2,npts
      xmin=min(x(j),xmin)
      xmax=max(x(j),xmax)
      ymin=min(y(j),ymin)
      ymax=max(y(j),ymax)
   5  continue
!...set overides         
      testy=ymax
      xmin=0.0
      ymin=0.0
      if (testy.LT.1.0) ymax=1.0
      if (testy.LT.0.5) ymax=0.5
      write(sxrange,'("set xrange [",F10.2,":",F10.2,"]")') xmin,xmax
      write(syrange,'("set yrange [",F10.2,":",F10.2,"]")') ymin,ymax
      write(12,*) "set grid                                          "
      write(12,*) "set nokey                                         "
      write(12,*) "set xlabel 'Frequency (Hz)'                       "
      write(12,*) "set ylabel 'Decay (1/meter)                       "
      write(12,*) "set title 'Decay (95% Conf) Elevations: \"
      write(12,'(F6.2," to ",F6.2,"\")') emin,emax 
      write(12,*) "meters \"
      write(12,'(" [",a,1x,i4.4,".",i4.4,"]\")') lineid,tyear,tday
      write(12,*) " '                                                "
      write(12,'(a)') sxrange 
      write(12,'(a)') syrange 
      write(12,*) "set mxtics 5                                      "
      write(12,*) "set mytics 5                                      "
      write(12,*) "set tmargin 0                                     "
      write(12,*) "set bmargin 4                                     "
      write(12,*) "set style line 1 lc rgb 'black' lt 1 lw 2         "
      write(12,*) "plot '-' u ($1):($2):($3*1.96) w yerrorbars \"
      write(12,*) "pt 6 ps 1                                         "
      do 10, j=1,npts
   10 write(12,'(3(E14.7,1x))') x(j),y(j),yvar(j)
      write(12,*) "e "
      write(12,*) "unset multiplot"
      close(12)
      return
      end
#endif
