c |$Id: picrestore.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $    |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: picrestore.f                                 |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |         RESTORES PICK TIMES FROM BSU Octave Program        |
c |         segpic.m (which requires bsegin.m bsegout.m        | 
c |                   segyinfo.m)                              | 
c |         OUTPUT PERFORMED ON DATA WITH A REDUCTION VELOCITY |
c |         (bred.f program output contains time shifts)       |
c |                                                            | 
c | bred.f ---> bredxxxx.dat file has (tr, statics)            |
c | segpic.m  ---> (tr, pics ) in pick file bredxxxx.pic       |
c |                                                            |
c |read bredxxxx.dat (trace, static) and bredxxxx.pic(tr, pics)|
c |and subtract bred to get restored picks, output in order    |
c |needed by bpic.f program                                    | 
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (c) 2004 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of picks (ie traces)                |
c |------------------------------------------------------------|
c |------------------------------------------------------------|

      parameter (NDIM=500)
      real*4 btr(NDIM),bstat(NDIM),str(NDIM),sstat(NDIM)
      real*4 pic(NDIM),bpic(NDIM)
      character*80 infil1,infil2
      integer bcount,scount
      nargsx=iargc()
      if (nargsx.lt.2) then
      write(*,*) 'usage: picrestore bredxxxx.dat bredxxxx.pic >out.pic'
      write(*,*) '  FLOW '
      write(*,*) ' 1)bred flattens data, saved: bredxxxx.dat    '
      write(*,*) ' 2)OCTAVE: segpic.m pics 1st breaks'
      write(*,*) ' segpic.m  outputs bredxxxx.pic (tr,pic)   '
      write(*,*) ' 3)picrestore stdout stream:   '
      write(*,*) ' adjusted pics, removing the flattening  '
      write(*,*) ' 4)bpic xxxx.seg 1 out.pic '
      write(*,*) ' (pics inserted into original file before bred)   '
      write(*,*) '   '
      stop
      endif
      call getarg(1,infil1)
      call namechk(12,infil1)
      call getarg(2,infil2)
      call namechk(12,infil2)
c      write(*,'(a)') infil1,infil2  
      open(2,file=infil1,access='sequential')
      open(3,file=infil2,access='sequential')
      bcount=0
      do 10, j=1,NDIM
      read(2,*,end=500) btr(j),bstat(j)
      bcount=bcount+1
c      write(*,'(2x,e14.7,2x,e14.7)') btr(j),bstat(j)
   10 continue
  500 continue
      scount=0
      do 20, j=1,NDIM
      read(3,*,end=600) str(j),sstat(j)
      scount=scount+1
c      write(*,'(2x,e14.7,2x,e14.7)') str(j),sstat(j)
   20 continue
  600 continue    
      close(2)
      close(3) 
c      write(*,'(''bcount='',i5)') bcount
c      write(*,'(''scount='',i5)') scount
      do 30, j=1,scount
      pic(j)=0.
      do 40, k=1,bcount
      if(btr(k).eq.str(j)) then
      pic(j)=sstat(j)-bstat(k)
      endif
   40 continue 
   30 continue     
c
      do 50, j=1,bcount
      bpic(j)=0.0
      do 60, k=1,scount
      if(str(k).eq.btr(j)) then
      bpic(j)=pic(k)
      goto 50
      endif
   60 continue
   50 continue
c
      do 70, j=1,bcount
      write(*,'(1x,i5,1x,f10.6)') int(btr(j)),bpic(j)
   70 continue
      end
