c |$Id: top2dxf.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: top2dxf.f                                    |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Reads a Topcon Total Station *.nez file, and writes out    |
c | survey locations to a drawing exchange file (*.dxf)        |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      parameter (ndim=10000)
      character*40      infil,outfil,outlst
      character*12      labl(ndim)
      real*4            x(ndim),y(ndim),z(ndim)
c
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +isw1,ilabel,txtsiz,outfil,outlst)
c
c
c...open output file *.dxf format
      open(unit=4,file=outfil,access='sequential')
c
c...open input file
      open(unit=3,file=infil,access='sequential')
c
c...output listing-------------------------------------------
      call LSTPRM(io11,outlst,outfil,
     +infil)
c
c...find min and max
      fxmin=+1.E+10
      fxmax=-1.E+10
      fymin=+1.E+10
      fymax=-1.E+10
      jrec=0
   30 jrec=jrec+1
      read(3,200,end=40,err=40) y(jrec),x(jrec),z(jrec),labl(jrec)
  200 format(12x,f12.0,f12.0,f12.0,a)
      if(x(jrec).gt.fxmax) fxmax=x(jrec)
      if(x(jrec).lt.fxmin) fxmin=x(jrec)
      if(y(jrec).gt.fymax) fymax=y(jrec)
      if(y(jrec).lt.fymin) fymin=y(jrec)
      go to 30
   40 continue
      npts=jrec-1
      bar=float(npts)/50.
      ibar=int(bar+0.5)
      invbar=int(1./bar+0.5)

c
c...write header with max and min limits
      if(isw1.eq.1) then
      write(4,900) fxmin,fymin,fxmax,fymax
  900 format('  0'/'SECTION'/'  2'/'HEADER'/'  9'/'$LIMMIN'/
     1' 10'/f16.6/' 20'/f16.6/'  9'/'$LIMMAX'/
     2' 10'/f16.6/' 20'/f16.6/'  0'/'ENDSEC'/
     3'  0'/'SECTION'/'  2'/'ENTITIES')
      else
      write(4,903)
  903 format('  0'/'SECTION'/'  2'/'ENTITIES')
      endif
c
      write(io11,800) fxmin,fymin,fxmax,fymax
  800 format(5x,'MINIMUM X,Y'/5x,2(f16.6,1x)/
     15x,'MAXIMUM X,Y'/5x,2(f16.6,1x)//)
c
c...output point locations
      do 50 jrec=1,npts
      write(4,901) x(jrec),y(jrec),z(jrec)
  901 format('  0'/'POINT'/'  8'/'0'/
     1' 10'/f16.6/' 20'/f16.6/' 30'/f16.6)
c...listing
      write(io11,802) x(jrec),y(jrec),z(jrec),labl(jrec)
  802 format(5x,3(f16.6,1x),a)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
  50  continue
      write(*,*) '  '
c
c...output labels
      if(ilabel.eq.1) then
      do 55 jrec=1,npts
      write(4,902) x(jrec),y(jrec),z(jrec),txtsiz,labl(jrec)
  902 format('  0'/'TEXT'/'  8'/'RED'/
     1' 10'/f16.6/' 20'/f16.6/' 30'/f16.6/' 40'/f10.3/
     2'  1'/a/'  7'/'ROMANS')
   55 continue
      endif
c
c...end section
      write(4,904)
  904 format('  0'/'ENDSEC'/'  0'/'EOF')
c
      close(io11)
      close(4)
      close(3)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +isw1,ilabel,txtsiz,outfil,outlst)
c
      character*40      infil
      character*40      outfil,outlst
      character*40      arg1,arg2,arg3,arg4
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| top2dxf:  Reads a *.nez survey file      |'
      write(*,*) '| and outputs as *.dxf CAD file.           |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*) '*.nez file format(12x,f12.0,f12.0,f12.0,a) '
      write(*,*) ' '
      write(*,*)
     +'top2dxf    infile   isw1   ilabel    txtsiz'
      write(*,*) ' '
      write(*,*) 'infile  =*.nez input file name '
      write(*,*) 'isw1    =switch to control limits '
      write(*,*) '       0=no limits header '
      write(*,*) '       1=limits based on min and max values '
      write(*,*) 'ilabel 0=no printing of point labels  '
      write(*,*) '       1=print labels '
      write(*,*) 'txtsiz  =size of text in coord. units (float) '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  enter: isw1'
      write(*,*) '       0=no limits header '
      write(*,*) '       1=limits based on min and max values '
      read(*,*) isw1
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') isw1
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*) 'ilabel 0=no printing of point labels  '
      write(*,*) '       1=print labels '
      read(*,*) ilabel
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') ilabel
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)'txtsiz  =size of text in coord. units (float) '
      read(*,*) txtsiz
      else
      call getarg(4,arg4)
      read(arg4,'(f10.3)') txtsiz
      endif
c
c...find input file prefix name
      do 10 j=1,40
      if(infil(j:j).eq.'.') then
      jcnt=j-1
      go to 20
      endif
      if(infil(j:j).eq.' ') then
      jcnt=j-1
      go to 20
      endif
   10 continue
   20 continue
      write(outfil,'(a,''.dxf'')') infil(1:jcnt)
      write(outlst,'(a,''.lst'')') infil(1:jcnt)
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,outlst,outfil,
     +infil)
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program master'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

