c  | $Id: disper.f,v 1.4 2018/02/05 23:52:35 pm Exp $       |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  |  disper:  main program for computing dispersion            |
c  |           curve for rayleigh waves  (propagation matrix)   |
c  |------------------------------------------------------------|
c  | &disper                                                    |
c  |    nlay=number of layers                                   |
c  |    rho=vector of densities (one for each layer)            |
c  |    mu=vector of shear moduli                               |
c  |    lame=vector of lame's moduli                            |
c  |    zi=vector of layer tops                                 |
c  |    zstart=depth, to start integration                      |
c  |    deltz=depth increment for integration                   |
c  |                                                            |
c  |    nfreq=number of frequencies                             |
c  |    flo=lowest frequency                                    |
c  |    delf=frequency increment for dispersion                 |
c  |                                                            |
c  |   ------if pvlcty.ne.0.0  Compute Motion-Stress Functions  |
c  |    pvlcty = phase velocity for eigenvector listing         |
c  |    pfreq = frequency for pvlcty                            |
c  |    zend = depth to compute eigenvector to                  |
c  |   NOTE: You have to run disper first with pvlcty=0 to      |
c  |         compute dispersion curves, then pick a phase       |
c  |         velocity from disper.tmp listing at a desired      |
c  |         frequency, pfreq.  Re-run disper with these new    |
c  |         values of pvlcty,pfreq to compute motion-stress    |
c  |         vectors down to a depth=zend.  octave files        |
c  |         are generated for plotting, disper.tmp is listing. |
c  |------------------------------------------------------------|
c  |  NOTICE:                                                   |
c  |  Copyright (C) 2017 Paul Michaels                          |
c  |  <pm@cgiss.boisestate.edu>                                 |
c  |  This program is free software; you can                    |
c  |  redistribute it and/or modify it under the terms          |
c  |  of the GNU General Public License as published            |
c  |  by the Free Software Foundation; either version           |
c  |  3 of the License, or (at your option) any later           |
c  |  version.  This program is distributed in the              |
c  |  hope that it will be useful, but                          |
c  |  WITHOUT ANY WARRANTY; without even the implied            |
c  |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c  |  PARTICULAR PURPOSE.  See the GNU General Public           |
c  |  License for more details.                                 |
c  |  You should have received a copy of the GNU                |
c  |  General Public License along with this program;           |
c  |  if not, write to the Free Software Foundation,            |
c  |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c  |------------------------------------------------------------|
c  |  dimensions:                                               |
c  |    ndim= number of control points in the model             |
c  |    nfmax= maximum number of frequencies                    |
c  |    maxmod= maximum number of modes                         |
c  |------------------------------------------------------------|
c  |  subroutines in line:                                      |
c  |    matmlt....cvel....objfun....msvctr....                  |
c  |    gparms....halfsp....dmodel....intgr4....pmatr2....      |
c  |    propgt....                                              |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      program main
      parameter (ndim=1000, i5=5, i4=4, maxmod=10)
      parameter (nfmax=1000)
      implicit real*8 (a-h,o-z)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
      real*8 pv(maxmod)
      real*8 plt(nfmax,maxmod)
      character*40 infile
      character*74 card
      character*3 mode(maxmod)
      character*40 ofile
      character*40 octav1,octav2
      character*40 curve
      character*20 label
      integer*2 istat
c
c
      namelist /disper/ rho,mu,lame,nlay,nfreq,flo,delf,itmax,
     +zi,deltz,ofile,jsmax,modemx,
     +pvlcty,pfreq,zend,iskip,ksw,curve,epsk,
     +octav1,octav2
c
c...read input file name
c...test if input file on command line
      istat=0
c
c...test if input file in argument list on command line
      nargs=iargc()
      if(nargs.eq.0) then
      write(*,*)'  enter: name of input file'
      read(*,'(a)') infile
      else
      call getarg(1,infile)
      endif
c
c
ccc      write(infile,'(''disper.d'')')
      write(ofile,'(''disper.lst'')')
      write(octav1,'(''phase.m'')')
      write(octav2,'(''model.m'')')
      write(curve,'(''earth.crv'')')
c
c
c...set defaults
      zero=0.0000000000000000
      epsk=.00000000000001
      jsmax=500
      ksw=0
      iskip=1
      pvlcty=0.0
      pfreq=1.0
      modemx=9
      itmax=10
      flo=5.
      nfreq=10.
      delf=5.
      nlay=2
      rho(1)=1500.
      rho(2)=2000.
      mu(1)=135.e+06
      mu(2)=1280.e+06
      lame(1)=465.e+06
      lame(2)=1940.e+06
      zi(1)=0.
      zi(2)=40.
      zend=-1.0
      pi=3.14159265358979311599796346854
      pi2=2.00000000*pi
      deltz=1.
      do 1 j=1,nfmax
      do 1 k=1,maxmod
    1 plt(j,k)=0.0
c
c...open files
      open(09, file=infile, access='sequential')
c
c...read input data
      read(09,nml=disper)
c
c setup octave output variables
      write(mode(2),'('' c1'')')
      write(mode(3),'('' c2'')')
      write(mode(4),'('' c3'')')
      write(mode(5),'('' c4'')')
      write(mode(6),'('' c5'')')
      write(mode(7),'('' c6'')')
      write(mode(8),'('' c7'')')
      write(mode(9),'('' c8'')')
      write(mode(10),'('' c9'')')
c
c
      call dmodel(rho,mu,lame,zi,nlay,ndim,deltz)
c
      open(10, file=ofile, access='sequential')
      if(zend.lt.0.0) zend=deltz+.00000000001
      rewind 9
      write(10,'(//)')
    7 read(09,'(a)',end=8) card
      write(10,'(5x,a)') card
      go to 7
    8 continue
      close(09)
c
c...trim deltz
      nz=int(zi(nlay)/deltz + 0.50)
      deltz=zi(nlay)/dfloat(nz)
      write(*,'('' trimmed deltz='',f10.5)') deltz
c
c...if pvlcty not zero, list eigenvector
      if(pvlcty.ne.zero) then
      wfreq=pfreq*2.0*pi
      fkn1=wfreq/pvlcty
c
      open(file=octav2,unit=14,access='sequential')
      open(unit=15,access='sequential')
      open(unit=16,access='sequential')
      open(unit=17,access='sequential')
c
      betahf=dsqrt(mu(nlay)/rho(nlay))
c
c...find max and min beta
      betamx=-1.e+10
      betamn=+1.e+10
      do 50 j=1,nlay
      beta=dsqrt(mu(j)/rho(j))
      if(betamx.lt.beta) then
      betamx=beta
      endif
   50 if(betamn.gt.beta) betamn=beta
ccc      write(*,'(''  wfreq,betamx,betamn'',3(e14.7,2x))')
ccc     +wfreq,betamx,betamn
c
cc      testbm=betahf-betamx
cc      if(testbm.lt.-0.1) then
cc      write(10,'(''  !!! abort  half space may be too slow'')')
cc      write(*,'(''   !!! abort half space may be too slow'')')
cc      write(10,'('' beta max='',f10.4)') betamx
cc      write(10,'(''  beta half space='',f10.4)') betahf
cc      write(*,'('' beta max='',f10.4)') betamx
cc      write(*,'(''  beta half space='',f10.4)') betahf
cc      close(10)
cc      stop
cc      endif
c
c
      call msvctr(fkn1,zend,iskip,
     +lame,mu,rho,zi,nlay,rv1,rv2,wfreq,
     +deltz,rtotal,ndim,i5)
c
      close(14)
      close(15,status='delete')
      close(16,status='delete')
      close(17,status='delete')
c
      go to 999
      endif
c
c...save dispersion curve as wavenumbers
      open(unit=08, file=curve, access='sequential')
c
c...output model to curve file
      write(8,700) nlay
  700 format(' &model'/'  nlay=',i5,',')
      write(8,'("rho=",$)')

      write(8,701) (rho(kk),kk=1,nlay)
  701 format(3(2x,D30.20,','))
      write(8,'("mu=",$)')
      write(8,701) (mu(kk),kk=1,nlay)
      write(8,'("lame=",$)')
      write(8,701) (lame(kk),kk=1,nlay)
      write(8,'("zi=",$)')
      write(8,701) (zi(kk),kk=1,nlay)
      write(8,'(''   /'')')
c
c...adjust flo to fall on a multiple of delf
      fdist=1.E+25
      do 34 jf=1,nfreq
      test=abs(float(jf)*delf-flo)
      if(test.lt.fdist) then
      fdist=test
      jfkeep=jf
      endif
   34 continue
      flo=float(jfkeep)*delf
      write(10,'(''       FLO RESET to flo='',f10.4)') flo
c
      lc=1
      write(10,'(/6x,
     +''| frequency |'',
     +'' phase velocity (fundamental ====> higher modes)    |'')')
c...open octave plot file
      open(file=octav1,unit=12,access='sequential')
      write(12,*) 'clear'
      write(12,*) '%  generated by disper.f <pm@cgiss.boisestate.edu>'
      if (ksw.eq.0) then
      write(12,*) '%  phase velocity plot'
      else
      write(12,*) '%  wavenumber plot ' 
      endif
c
c...start frequency loop
      jsmax2=jsmax
      do 500 jf=1,nfreq
      wfreq=2.0*pi*(flo+float(jf-1)*delf)
c
      if(lc.gt.55) then
      write(10,'(a)') char(12)
      lc=1
      write(10,'(/6x,
     +''| frequency |'',
     +'' phase velocity (fundamental ====> higher modes)    |'')')
      endif
c
c...compute phase velocity
      do 11 jv=1,maxmod
   11 pv(jv)=0.0d+00
      jsmax=jsmax2
      call cvel(wfreq,pv,jsmax,epsk,modemx,
     +lame,mu,rho,zi,nlay,rv1,rv2,
     +deltz,rtotal,ndim,i5,maxmod)
c
c...number of modes found
      nmode=0
      do 501 jm=1,maxmod
  501 if(pv(jm).ne.zero) nmode=jm
c
c...output iteration
      if(lc.gt.55) then
      write(10,'(a)') char(12)
      lc=1
      write(10,'(/6x,
     +''| frequency |'',
     +'' phase velocity (fundamental ====> higher modes)    |'')')
      endif
      fq=wfreq/(2.*pi)
c
c...save wavenumbers
      write(8,799) fq,(wfreq/pv(jj),jj=1,nmode),
     +(pv(jj),jj=nmode+1,9)
  799 format(10(1x,e32.25))   
c phase velocity
      if(nmode.ge.6) then
      write(10,800) fq,(pv(jm),jm=1,5)
      write(10,800) fq,(pv(jm),jm=6,nmode)
      write(*,800) fq,(pv(jm),jm=1,5)
      write(*,800) fq,(pv(jm),jm=6,nmode)
      lc=lc+2
      else
      write(10,800) fq,(pv(jm),jm=1,nmode)
      write(*,800) fq,(pv(jm),jm=1,nmode)
      lc=lc+1
      endif
  800 format(4x,f13.7,' | ',5(f14.7),' |')
c
c...save data for plot
      if(jf.gt.nfmax) then
c     write(10,'(''  abort: more than nfmax frequencies'')')
c     go to 999
      endif
c
      do 483 kf=1,nmode
  483 plt(jf,kf+1)=pv(kf)
      plt(jf,1)=fq
c
  500 continue
c -----end frequency loop
c
c...output to maple file for plot
      do 484 kf=2,10
      iswp=0
      do 485 jf=1,nfreq
      if(plt(jf,kf).ne.zero.and.iswp.ge.0) iswp=1
c-------------%
      if(iswp.gt.0) then
c+++
      if(ksw.eq.0) then
      write(12,900) mode(kf),plt(jf,1),plt(jf,kf)
      else
      fk1=(pi2*plt(jf,1))/plt(jf,kf)
      write(12,900) mode(kf),plt(jf,1),fk1
      endif
c+++
      iswp=-1
      go to 485
      endif
c-------------%
      if(jf.lt.nfreq.and.plt(jf,kf).ne.zero) then
c+++
      if(ksw.eq.0) then
      write(12,901) plt(jf,1),plt(jf,kf)
      else
      fk1=(pi2*plt(jf,1))/plt(jf,kf)
      write(12,901) plt(jf,1),fk1
      endif
c+++
      endif
c--------------%
      if(jf.eq.nfreq.and.plt(jf,kf).ne.zero) then
c+++
      if(ksw.eq.0) then
      write(12,902) plt(jf,1),plt(jf,kf)
      else
      fk1=(pi2*plt(jf,1))/plt(jf,kf)
      write(12,902) plt(jf,1),fk1
      endif
c+++
      endif
c---------------%
  485 continue
  484 continue
  900 format(a,'=[',f10.4,', ',f10.3,';')
  901 format(f10.4,', ',f10.3,';')
  902 format(f10.4,', ',f10.3,'];')
c
c...select octave output format         
      write(12,*) 'figure(1)'
      write(12,*) 'clf'
      write(12,*) 'hold on'
      write(12,*) 'set(gca(),''fontsize'',12) '
      if(nmode.eq.1) write(12,914)
      if(nmode.eq.2) write(12,915)
      if(nmode.eq.3) write(12,916)
      if(nmode.eq.4) write(12,917)
      if(nmode.eq.5) write(12,918)
      if(nmode.eq.6) write(12,919)
      if(nmode.eq.7) write(12,920)
      if(nmode.eq.8) write(12,921)
      if(nmode.eq.9) write(12,922)
c
c modified 18 Oct 2014 for octave
  914 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)')
  915 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')')
  916 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')',/,
     +' plot(c3(:,1),c3(:,2),''r'')')
  917 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')',/,
     +' plot(c3(:,1),c3(:,2),''r'')',/,
     +' plot(c4(:,1),c4(:,2),''r'')')
  918 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')',/,
     +' plot(c3(:,1),c3(:,2),''r'')',/,
     +' plot(c4(:,1),c4(:,2),''r'')',/,
     +' plot(c5(:,1),c5(:,2),''r'')')
  919 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')',/,
     +' plot(c3(:,1),c3(:,2),''r'')',/,
     +' plot(c4(:,1),c4(:,2),''r'')',/,
     +' plot(c5(:,1),c5(:,2),''r'')',/,
     +' plot(c6(:,1),c6(:,2),''r'')')
  920 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')',/,
     +' plot(c3(:,1),c3(:,2),''r'')',/,
     +' plot(c4(:,1),c4(:,2),''r'')',/,
     +' plot(c5(:,1),c5(:,2),''r'')',/,
     +' plot(c6(:,1),c6(:,2),''r'')',/,
     +' plot(c7(:,1),c7(:,2),''r'')')
  921 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')',/,
     +' plot(c3(:,1),c3(:,2),''r'')',/,
     +' plot(c4(:,1),c4(:,2),''r'')',/,
     +' plot(c5(:,1),c5(:,2),''r'')',/,
     +' plot(c6(:,1),c6(:,2),''r'')',/,
     +' plot(c7(:,1),c7(:,2),''r'')',/,
     +' plot(c8(:,1),c8(:,2),''r'')')
  922 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)',/,
     +' plot(c2(:,1),c2(:,2),''r'')',/,
     +' plot(c3(:,1),c3(:,2),''r'')',/,
     +' plot(c4(:,1),c4(:,2),''r'')',/,
     +' plot(c5(:,1),c5(:,2),''r'')',/,
     +' plot(c6(:,1),c6(:,2),''r'')',/,
     +' plot(c7(:,1),c7(:,2),''r'')',/,
     +' plot(c8(:,1),c8(:,2),''r'')',/,
     +' plot(c9(:,1),c9(:,2),''r'')')
      if (ksw.eq.0) then
      write(12,924)
  924 format('title(''Dispersion: Phase Velocity'')'/
     +'xlabel(''Frequency  Hz'')'/
     +'ylabel(''Phase Velocity M/S'')')
      else
      write(12,926)
  926 format('title(''Dispersion: Wave Number'')'/
     +'xlabel(''Frequency  Hz'')'/
     +'ylabel(''Wave Number cycles/meter'')')
      endif
      write(12,*) 'grid on'
c
      close(12)
      close(8)
  999 continue
      close(10)
      stop
      end
c# |------------------------------------------------------------|
c  | matmlt.f                                                   |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | matmlt:  matrix multiply                                   |
c  |                                                            |
c  |------------------------------------------------------------|
      subroutine matmlt(a,x,y,n,m)
      implicit real*8 (a-h,o-z)
      real*8 a(n,m),x(m),y(n)
      do 5 j=1,n
      y(j)=0.00d+00
      do 6 k=1,m
    6 y(j)=y(j)+a(j,k)*x(k)
    5 continue
      return
      end
c# |------------------------------------------------------------|
c  | cvel.f                                                     |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | cvel: calculates phase velocity for full model             |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      subroutine cvel(wfreq,pv,jsmax,epsk,modemx,
     +lame,mu,rho,zi,nlay,rv1,rv2,
     +deltz,rtotal,ndim,i5,maxmod)
      implicit real*8 (a-h,o-z)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
      real*8 kstart
      real*8 pv(maxmod)
      real*8 unity
c
      pi=3.14159265358979311599796346854
      iskip=1
      unity=1.0
c
      iter=0
c
c
c...assign elastic parameters of halfspace
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
      betahf=dsqrt(gmu/grho)
c
c...find max and min beta
      betamx=-1.e+10
      betamn=+1.e+10
      do 50 j=1,nlay
      beta=dsqrt(mu(j)/rho(j))
      if(betamx.lt.beta) betamx=beta
   50 if(betamn.gt.beta) betamn=beta
c
      betak=betamn*0.80
c
ccc      write(*,'(''  wfreq,betamx,betamn'',3(e14.7,2x))')
ccc     +wfreq,betamx,betamn
c
      testbm=betahf-betamx
      if(testbm.lt.-0.1) then
ccc      write(10,'(''  !!! warning half space may be too slow'')')
ccc      write(*,'(''  !!! warning half space may be too slow'')')
ccc      write(10,'('' beta max='',f10.4)') betamx
ccc      write(10,'(''  beta half space='',f10.4)') betahf
ccc      write(*,'('' beta max='',f10.4)') betamx
ccc      write(*,'(''  beta half space='',f10.4)') betahf
ccc      close(10)
ccc      stop
      betamx=betahf
      endif
c
c...test modemx=maximum number of modes
      if(modemx.gt.9) then
      write(10,'(''  ****abort**** too many modes'')')
      write(*,'(''  ****abort**** too many modes'')')
      close(10)
      stop
      endif
c
c...save jsmax if adapted bracket search compiled version (iadapt=1)
      jsmax2=jsmax
c
c...loop up to 9 modes
      modect=0
      do 100 jv=1,modemx
c     return to 29 if no cvel found this mode, increased jsmax
c     (return to 29 only activated when iadapt=1)
      iadapt=1
      iacnt=0
   29 continue
c...bail out if adaptive bracket fails in 10 attempts
      if (iacnt .gt. 10) go to 31
c
c...set kstart, and endk
c...note: betak changes at end of loop
c...after each mode is found
      endk=wfreq/(betamx-.00001)
      kstart=wfreq/(betak)
c
      fkh=wfreq/(betak+.1)
      delk=(endk-kstart)/float(jsmax+1)
      fkl=kstart
      ccvl=wfreq/fkh
c
c
ccc   write(*,'('' test phasev,fkl,fkh'',3(e14.7,2x))') ccvl,fkl,fkh
c
c...test to avoid exceeding betamx
      if(fkl.le.endk) go to 99
c
      call objfun(phil,fkl,iskip,
     +lame,mu,rho,zi,nlay,rv1,rv2,wfreq,
     +deltz,rtotal,ndim,i5)
      jsgn=int(sign(unity,phil))
ccc   write(*,'(''  phil,fkl,jsgn'',2(e14.7,1x),i5)') phil,fkl,jsgn
c
      do 30 js=1,jsmax
c...test to avoid exceeding betamx
      if(fkh.le.endk) go to 99
c
      call objfun(phih,fkh,iskip,
     +lame,mu,rho,zi,nlay,rv1,rv2,wfreq,
     +deltz,rtotal,ndim,i5)
      jhsgn=int(sign(unity,phih))
      cphvel=wfreq/fkh
ccc Here it is, sign does not change, DEBUG jhsgn does not flip from -1
      if(jhsgn.ne.jsgn) go to 40
c
c...move low bracket
      fkl=fkh
c
      fkh=fkh+delk
   30 continue
c... only come here if bracket search failure
      if (iadapt .eq. 1) then
      jsmax=jsmax*2
      iacnt=iacnt+1
c     write(*,'(''jsmax, iacnt'',i5,2x,i5)') jsmax,iacnt
      go to 29
      endif
   31 continue
c... fatal failure to find a bracket 
      pv(jv)=0.0
      go to 105
   40 continue
c...bracket found, reset jsmax
      jsmax=jsmax2
c     if (iacnt.ne.0) then
c     write(*,'(''reset: jsmax='',i5)') jsmax
c     endif
c
c...capture sign, hi and low k
      isnh=int(sign(unity,phih))
      isnl=int(sign(unity,phil))
c
      if(isnl.eq.isnh) then
      write(10,'(''error in cvel'')')
      write(*,'(''error in cvel'')')
      stop
      endif
c
c...find wavenumber and phase velocity
   10 if(iter.gt.1000) then
      write(*,'(''  max out iterations in cvel '',
     +'' ( resolution not achieved)'')')
      write(10,'(''  max out iterations in cvel '',
     +'' ( resolution not achieved)'')')
      go to 20
      endif
      testk=(fkl+fkh)/2.0
c      write(*,'('' testk='',e14.7)') testk
c
c...avoid exceeding betamx
      if(testk.le.endk) go to 99
c
      if(abs(fkh-fkl).lt.epsk) go to 20
      call objfun(phit,testk,iskip,
     +lame,mu,rho,zi,nlay,rv1,rv2,wfreq,
     +deltz,rtotal,ndim,i5)
      isnt=int(sign(unity,phit))
c
      if(isnt.eq.isnl) then
      fkl=testk
      isnl=isnt
      else
      fkh=testk
      isnh=isnt
      endif
c
      iter=iter+1
      go to 10
   20 continue
      phasev=wfreq/testk
      pv(jv)=phasev
      betak=phasev + 1.00e+00
      modect=modect+1
      if(modect.eq.modemx) go to 99
c
  100 continue
   99 continue
c----end mode search
  105 continue
c
      return
      end
c# |------------------------------------------------------------|
c  | objfun.f                                                   |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | objfun: calculates objective function                      |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      subroutine objfun(phi,fkn1,iskip,
     +lame,mu,rho,zi,nlay,rv1,rv2,wfreq,
     +deltz,rtotal,ndim,i5)
      implicit real*8 (a-h,o-z)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
c
      zero=0.00d+00
c
      z1=zi(nlay)
c
c...get elastic parameters
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
c
c...use trial k to propagate
      phasev=wfreq/fkn1
c
c...set inital p and s wave balance
      aa1=1.0d+00
c
      call halfsp(grho,gmu,glame,wfreq,z1,phasev,
     +rz1,rz2,rz3,rz4,aa1,rv1,rv2)
c
      zstart=zi(nlay)
      rv1(5)=zi(nlay)
      rv1(5)=zi(nlay)
c
c...integrate upwards
      fsign=-1.0
      iprnt=0
      call intgr4(fkn1,zstart,deltz,nlay,wfreq,
     +rtotal,fsign,rho,mu,lame,zi,rv1,rv2,iskip,
     +ndim,aa1,i5,iprnt)
c
c...compute objective function
      phi=rv1(3)*rv2(4)-rv2(3)*rv1(4)
c
      return
      end
c# |------------------------------------------------------------|
c  |                                                            |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | msvctr: calculates motion stress vector                    |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      subroutine msvctr(fkn1,zend,iskip,
     +lame,mu,rho,zi,nlay,rv1,rv2,wfreq,
     +deltz,rtotal,ndim,i5)
      implicit real*8 (a-h,o-z)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
      real*8 s1(5),s2(5)
      real*8 tv1(5),tv2(5)
c
      zero=0.00d+00
c
c...use trial k to propagate
      phasev=wfreq/fkn1
c
c...get elastic parameters
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
c
c...set p and s wave balance
      aa1=1.0d+00
c
c...compute initial motion stress vector at base of model
c
      z1=zi(nlay)
c
      call halfsp(grho,gmu,glame,wfreq,z1,phasev,
     +rz1,rz2,rz3,rz4,aa1,rv1,rv2)
c
      write(10,'(/'' @zr1: '',5(e10.4,1x))') (rv1(jj),jj=1,5)
      write(10,'('' @zr2: '',5(e10.4,1x))') (rv2(jj),jj=1,5)
c
c...save initial motion stress vectors at base of model
      do 40 j=1,5
      s1(j)=rv1(j)
   40 s2(j)=rv2(j)
c
      zstart=zi(nlay)
c
      rv1(5)=zstart
      rv2(5)=zstart
c
c...integrate upwards with trial k
      fsign=-1.0
      iprnt=0
      call intgr4(fkn1,zstart,deltz,nlay,wfreq,
     +rtotal,fsign,rho,mu,lame,zi,rv1,rv2,iskip,
     +ndim,aa1,i5,iprnt)
      write(10,'(/'' @0 r1: '',5(e10.4,1x))') (rv1(jj),jj=1,5)
      write(10,'('' @0 r2: '',5(e10.4,1x))') (rv2(jj),jj=1,5)
c
c...motion stress vector at surface in rv1,rv2
c...determine total eigenvector
c
      aa1=-rv2(3)/rv1(3)
      aa2=-rv2(4)/rv1(4)
c
      write(10,'(/''   aa1'',e14.7)') aa1
      write(10,'(''   aa2'',e14.7)') aa2
      do 30 j=1,4
   30 rtotal(j)=aa1*rv1(j) + rv2(j)
      rtotal(5)=rv1(5)
      write(*,'('' total vector at surface:''/
     +5(1x,e10.4))') (rtotal(jj),jj=1,5)
      write(10,'(/6x,'' total vector at surface:''/
     +6x,5(1x,e10.4))') (rtotal(jj),jj=1,5)
c...............................................
      am1=aa1*s1(1) + s2(1)
      am2=aa1*s1(2) + s2(2)
      am3=aa1*s1(3) + s2(3)
      am4=aa1*s1(4) + s2(4)
      write(10,'(/''  am1,am2'',2(e14.7,2x))') am1,am2
      write(10,'(''  am3,am4'',2(e14.7,2x))') am3,am4
c
c...redetermine p and s wave with new weight aa1
      xz=zi(nlay)
c
c...recompute motion/stress vector using new balance, aa1
      call halfsp(grho,gmu,glame,wfreq,xz,phasev,
     +rz1,rz2,rz3,rz4,aa1,tv1,tv2)
c
c...tie test
      rm1=am1/rz1
      rm2=am2/rz2
      rm3=am3/rz3
      rm4=am4/rz4
      write(10,'(//''   rm1,rm2'',2(e14.7,2x))') rm1,rm2
      write(10,'(''   rm3,rm4'',2(e14.7,2x))') rm3,rm4
c
c...list eigenfunction for wavenumber fkn1
      fsign=-1.0
      zstart=zend
c
c...compute motion/stress vector at depth=zend
c   (initial value for upward integration)
      if(zend.ge.zi(nlay)) then
      call halfsp(grho,gmu,glame,wfreq,zend,phasev,
     +rz1,rz2,rz3,rz4,aa1,rv1,rv2)
      do 31 j=1,4
   31 rtotal(j)=aa1*rv1(j) + rv2(j)
      rtotal(5)=rv1(5)
      write(*,'('' total vector at zend:''/
     +5(1x,e10.4))') (rtotal(jj),jj=1,5)
      write(10,'(/6x,'' total vector at zend:''/
     +6x,5(1x,e10.4))') (rtotal(jj),jj=1,5)
      else
      zin=zi(nlay)
      call halfsp(grho,gmu,glame,wfreq,zin,phasev,
     +rz1,rz2,rz3,rz4,aa1,rv1,rv2)
      do 32 j=1,4
   32 rtotal(j)=aa1*rv1(j) + rv2(j)
      rtotal(5)=rv1(5)
      write(*,'('' total vector at zi(nlay):''/
     +5(1x,e10.4))') (rtotal(jj),jj=1,5)
      write(10,'(/6x,'' total vector at zi(nlay):''/
     +6x,5(1x,e10.4))') (rtotal(jj),jj=1,5)
      endif
c
      iprnt=1
      write(*,'(''  iskip='',i5)') iskip
      call intgr4(fkn1,zstart,deltz,nlay,wfreq,
     +rtotal,fsign,rho,mu,lame,zi,rv1,rv2,iskip,
     +ndim,aa1,i5,iprnt)
c
      return
      end
c# |------------------------------------------------------------|
c  |                                                            |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | gparms: gets elastic parameters                            |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      subroutine gparms(rho,mu,lame,zi,nlay,ndim,
     +depth,grho,gmu,glame)
      implicit real*8 (a-h,o-z)
      real*8 mu(ndim),lame(ndim),rho(ndim),zi(ndim)
c
c...determine current layer
      l1=1
      do 10 j=1,nlay
   10 if(depth.gt.zi(j)) l1=j
c
      if(l1.lt.nlay) then
      l2=l1+1
      dz=zi(l2) - zi(l1)
      sloper=(rho(l2)-rho(l1))/dz
      slopel=(lame(l2)-lame(l1))/dz
      slopem=(mu(l2)-mu(l1))/dz
      z=depth-zi(l1)
      grho=sloper*z + rho(l1)
      gmu=slopem*z + mu(l1)
      glame=slopel*z + lame(l1)
      else
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
      endif
      return
      end
c# |------------------------------------------------------------|
c  | halfsp.f                                                   |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | halfsp: calculates the theoretical phase velocity and      |
c  |         displacement,stress eigenfunctions at depth, z.    |
c  |                                                            |
c  | rtotal as follows:                                         |
c  | r1=horizontal displacement      (aki & richards sign       |
c  | r2=vertical displacement         convention)               |
c  | r3=horizontal stress                                       |
c  | r4=vertical stress                                         |
c  | aa1= p-wave/s-wave balance ===>  rtotal=aa1*rv1 + rv2      |
c  |       rv1=p-wave set   rv2=s-wave set                      |
c  |                                                            |
c  | c=phase velocity for rayleigh waves                        |
c  |                                                            |
c  | rho=half space density                                     |
c  | fmu=half space shear modulus                               |
c  | flame= half space lame's constant                          |
c  |------------------------------------------------------------|
c
      subroutine halfsp(rho,fmu,flame,wfreq,z,c,r1,r2,r3,r4,
     +aa1,rv1,rv2)
      implicit real*8 (a-h,o-z)
      real*8 rv1(5),rv2(5)
c
c...compute shear velocity
      b=dsqrt(fmu/rho)
      d=flame/fmu
      if(c.eq.0.0e+00) then
c
c   compute phase velocity
c...set up coef. for cubic equation
      a1=-8.0e+00
      a2=((8.0e+00)*(d*3.0e+00+4.0e+00))/(d+2.0e+00)
      a3=((-16.0e+00)*(d+1.0e+00))/(d+2.0e+00)
c
c...solve cubic equation
      pi=3.14159265358979311599796346854
      q=(a1*a1-3*a2)/9.0e+00
      r=(2.0e+00*a1*a1*a1-9.0e+00*a1*a2+27.0e+00*a3)/54.0e+00
      test=q*q*q - r*r
      if(test.ge.0.0e+00) then
      t1=acos(r/dsqrt(q*q*q))
      t2=-2.0e+00*dsqrt(q)
      t3=3.0e+00
      x1=t2*cos(t1/t3)-a1/t3
      x2=t2*cos((t1+2.0e+00*pi)/t3)-a1/t3
      x3=t2*cos((t1+4.0e+00*pi)/t3)-a1/t3
      else
      t1=(dsqrt(r*r-q*q*q)+abs(r))**.333333333
      t3=3.0e+00
      x1=-sign(unity,r)*(t1+q/t1)-a1/t3
      endif
      c=dsqrt(b*b*x1)
      endif
c
c...compute eigenfunctions
      fk=wfreq/c
      t1=(c*c)/(b*b)
      g1=dsqrt(1.0-t1)*fk
      g2=dsqrt(1.0-t1/(d+2.0e+00))*fk
      t2=t1-2.0e+00
      e1=exp(-g1*z)
      e2=exp(-g2*z)
c
c...p-wave set
      rv1(1)=fk*e2
      rv1(2)=g2*e2
      rv1(3)=fmu*fk*(-2.0e+00)*g2*e2
      rv1(4)=(fk*fk*fmu*t2)*e2
      rv1(5)=z
c
c...s-wave set
      rv2(1)=fk*(t2/2.0e+00)*e1
      rv2(2)=((fk*fk)/(2.0e+00*g1))*t2*e1
      rv2(3)=fmu*fk*((t2*t2*fk*fk)/(g1*2.0e+00))*e1
      rv2(4)=-(fk*fk*fmu*t2)*e1
      rv2(5)=z
c
      r1=aa1*fk*e2 + fk*(t2/2.0e+00)*e1
c
      r2=aa1*g2*e2 + ((fk*fk)/(2.0e+00*g1))*t2*e1
c
      r3=fmu*fk*(aa1*(-2.0e+00)*g2*e2  +
     +((t2*t2*fk*fk)/(g1*2.0e+00))*e1)
c
      r4=(fk*fk*fmu*t2)*(aa1*e2 - e1)
c
c
      return
      end
c# |------------------------------------------------------------|
c  |                                                            |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | dmodel: display earth model                                |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      subroutine dmodel(rho,mu,lame,zi,nlay,ndim,deltz)
      implicit real*8 (a-h,o-z)
      real*8 mu(ndim),lame(ndim),rho(ndim),zi(ndim)
      character*5 rvct(5)
      character*80 card
c
c...setup octave output
      write(rvct(1),'('' Alph'')')
      write(rvct(2),'('' Beta'')')
      write(rvct(3),'('' Rhoo'')')
      write(rvct(4),'('' muuu'')')
      write(rvct(5),'('' lame'')')
c
c...compute model at regularly space depths
      zo=zi(1)
      z1=deltz+zo
      nz=int(zi(nlay)/deltz+.5)
c
c...open files
      open(file='model.m',unit=14,access='sequential')
c
      open(unit=15,access='sequential')
      open(unit=16,access='sequential')
      open(unit=17,access='sequential')
      open(unit=18,access='sequential')
c
      depth=(zo+z1)/2.0D+00
      call gparms(rho,mu,lame,zi,nlay,ndim,
     +depth,grho,gmu,glame)
      alpha=dsqrt((glame+2.0d+00*gmu)/grho)
      beta=dsqrt(gmu/grho)
      write(14,'(''clear'')')
      write(14,*) '%  generated by disper.f <pm@cgiss.boisestate.edu>'
      write(14,800) rvct(1),zo,alpha
      write(15,800) rvct(2),zo,beta
      write(16,800) rvct(3),zo,grho
      write(17,800) rvct(4),zo,gmu
      write(18,800) rvct(5),zo,glame
      write(14,801) z1,alpha
      write(15,801) z1,beta
      write(16,801) z1,grho
      write(17,801) z1,gmu
      write(18,801) z1,glame
c
      do 10 jz=2,(nz*2),2
      zo=z1
      z1=zo+deltz
      depth=(zo+z1)/2.0D+00
      call gparms(rho,mu,lame,zi,nlay,ndim,
     +depth,grho,gmu,glame)
      alpha=dsqrt((glame+2.0d+00*gmu)/grho)
      beta=dsqrt(gmu/grho)
c
      write(14,801) zo,alpha
      write(15,801) zo,beta
      write(16,801) zo,grho
      write(17,801) zo,gmu
      write(18,801) zo,glame
      write(14,801) z1,alpha
      write(15,801) z1,beta
      write(16,801) z1,grho
      write(17,801) z1,gmu
      write(18,801) z1,glame
   10 continue
      depth=z1
      call gparms(rho,mu,lame,zi,nlay,ndim,
     +depth,grho,gmu,glame)
      alpha=dsqrt((glame+2.0d+00*gmu)/grho)
      beta=dsqrt(gmu/grho)
      write(14,801) depth,alpha
      write(15,801) depth,beta
      write(16,801) depth,grho
      write(17,801) depth,gmu
      write(18,801) depth,glame
c
      depth=2.0*zi(nlay)
      write(14,802) depth,alpha
      write(15,802) depth,beta
      write(16,802) depth,grho
      write(17,802) depth,gmu
      write(18,802) depth,glame
c
c...transfer scratch plots
      rewind(15)
      rewind(16)
      rewind(17)
      rewind(18)
      nz2=2.*nz+4
      do 23 jz=1,nz2
      read(15,'(a)',end=33) card
   23 write(14,'(a)') card
   33 continue
      do 24 jz=1,nz2
      read(16,'(a)',end=44) card
   24 write(14,'(a)') card
   44 continue
      do 25 jz=1,nz2
      read(17,'(a)',end=55) card
   25 write(14,'(a)') card
   55 continue
      do 26 jz=1,nz2
      read(18,'(a)',end=66) card
   26 write(14,'(a)') card
   66 continue
c
c...output to octave 
      write(14,*) 'set(gca(),''fontsize'',12)'
      write(14,*) 'xmin=0; xmax=1.5*max(Alph(:,2));'
      write(14,*) 'ymin=-max(Beta(:,1)); ymax=0;'
      write(14,*) 'xmax2=1.1*max(Rhoo(:,2))/1000;'
      write(14,*) 'xmin2=.9*min(Rhoo(:,2))/1000;'
      write(14,*) 'figure(1)'
      write(14,*) 'clf'
      write(14,*) 'subplot(121)'
      write(14,*) 'plot(Beta(:,2),-Beta(:,1),''-b'',''linewidth'',2)'
      write(14,*) 'axis([xmin,xmax,ymin,ymax]);'
      write(14,*) 'hold on '
      write(14,*) 'plot(Alph(:,2),-Alph(:,1),''-r'',''linewidth'',2)'
      write(14,*) 'grid on'
      write(14,*) 'title(''SV and P Velocities'')'
      write(14,*) 'xlabel(''m/s'')'
      write(14,*) 'ylabel(''depth (m)'');'
      write(14,*) 'subplot(122)'
      write(14,*) 'plot(Rhoo(:,2)/1000,-Rhoo(:,1),''-k'',''linewidth''
     +,2)'
      write(14,*) 'axis([xmin2,xmax2,ymin,ymax]);'
      write(14,*) 'title(''Mass Density'')'
      write(14,*) 'xlabel(''g/cm3'')'
      write(14,*) 'ylabel(''depth (m)'');'
      write(14,*) 'grid on'
c
c
  800 format(a,'=[',f24.7,',',f24.7,';')
  801 format(f24.7,',',f24.7,';')
  802 format(f24.7,',',f24.7,'];')
      close(14)
      close(15,status='delete')
      close(16,status='delete')
      close(17,status='delete')
      close(18,status='delete')
      return
      end
c# |------------------------------------------------------------|
c  |                                                            |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  | intgr4: propagator matrix version of intgrt                |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      subroutine intgr4(fkn1,zstart,deltz,nlay,wfreq,
     +rtotal,fsign,rho,mu,lame,zi,rv1,rv2,iskip,
     +ndim,aa1,i5,iprnt)
      implicit real*8 (a-h,o-z)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
      real*8 p(4,4)
      character*80 card
      character*3 rvct(4)
c
c
c...octave setup
      write(rvct(1),'('' r1'')')
      write(rvct(2),'('' r2'')')
      write(rvct(3),'('' r3'')')
      write(rvct(4),'('' r4'')')
c
      zero=0.0000000000000000
      jzsw=1
      itrig=1
c
c...get elastic parameters
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
c
c...use trial k for analytic formula
      phasev=wfreq/fkn1
c
c...upper and lower wavenumbers
c
c...depth integration loop
      if(zstart.ge.zi(nlay)) then
      nz=int(zstart/deltz + 0.50)
      rv1(5)=zstart
      else
      nz=int(zi(nlay)/deltz + 0.50)
      rv1(5)=zi(nlay)
      endif
c
c     write(*,'(''  zstart,deltz,nz'',2e10.4,i5)') zstart,deltz,nz
      lcount=61
      if(nz.lt.1) go to 21
      h=fsign*deltz
      do 20 jz=1,nz
c
c...integrate above zi(nlay), analytic below
c   for stability (as opposed to accuracy)
      test=zi(nlay)
      if(rv1(5).le.test) then
c
c...synchronize integration on zi(nlay)
      if(itrig.eq.1) then
      zin=zi(nlay)
      call halfsp(grho,gmu,glame,wfreq,zin,phasev,
     +rz1,rz2,rz3,rz4,aa1,rv1,rv2)
      itrig=0
      endif
c
c...propagate set 1 and set 2
      z0=rv1(5)
      z1=rv1(5) + h
      call propgt(zi,mu,lame,rho,wfreq,phasev,nlay,
     +p,rv1,rv2,ndim,i5,z0,z1)
c
c      write(10,'(''  rv1'',5(e10.4,1x))') (rv1(jjj),jjj=1,5)
c      write(10,'(''  rv2'',5(e10.4,1x))') (rv2(jjj),jjj=1,5)
c
c...analytic eigenfunctions ................................
      else
      z1=rv1(5)+deltz*fsign
      rv1(5)=z1
c
      if(mod(jz,iskip).eq.0 .or. jz.eq.1) then
      call halfsp(grho,gmu,glame,wfreq,z1,phasev,r1,r2,r3,r4,
     +aa1,rv1,rv2)
      endif
c
       endif
c
      if (iprnt.ne.zero) then
      if(mod(jz,iskip).eq.0 .or. jz.eq.1) then
      do 30 jjj=1,4
   30 rtotal(jjj)=aa1*rv1(jjj) + rv2(jjj)
      rtotal(5)=rv1(5)
c
      if (lcount.gt.55) then
      write(10,'(a)') char(12)
      write(10,'(6x,''eigenvector vs depth'')')
      write(10,'(12x,
     +''u-horiz|u-vert|stress-horiz|stress-vert|depth'')')
      lcount=1
      endif
c
c...only print output if depth < zstart
      if(rtotal(5).le.zstart) then
      write(10,'(6x,5(1x,e10.4))') (rtotal(jjj),jjj=1,5)
      lcount=lcount+1
      endif
c
      endif
c
c
c
c
c. . . . . . . . . . . . . . . . . . . . . .
c...only print output if depth < zstart
      if(rtotal(5).le.zstart) then
c
c...octave plot of eigenvector
      if(jzsw.eq.1) then
c...octave io
      write(14,700) rvct(1),rtotal(5),rtotal(1)
      write(15,700) rvct(2),rtotal(5),rtotal(2)
      write(16,700) rvct(3),rtotal(5),rtotal(3)
      write(17,700) rvct(4),rtotal(5),rtotal(4)
      jzsw=0
      endif
c
      if(mod(jz,iskip).eq.0) then
      if(jz.gt.1.and.jz.lt.nz) then
c
c...octave io
      write(14,701) rtotal(5),rtotal(1)
      write(15,701) rtotal(5),rtotal(2)
      write(16,701) rtotal(5),rtotal(3)
      write(17,701) rtotal(5),rtotal(4)
c
      endif
      endif
c
      if(jz.eq.nz) then
c-----
c...octave io
      write(14,702) rtotal(5),rtotal(1)
      write(15,702) rtotal(5),rtotal(2)
      write(16,702) rtotal(5),rtotal(3)
      write(17,702) rtotal(5),rtotal(4)
  700 format(a,'=[',f20.7,',',1x,E14.7,';')
  701 format(f22.7,',',1x,E14.7,';')
  702 format(f22.7,',',1x,E14.7,'];')
c
  800 format(a,'=[',f20.7,',',1x,f10.7,'*10**(',i4,'),')
  801 format(f22.7,',',1x,f10.7,'*10**(',i4,'),')
  802 format(f22.7,',',1x,f10.7,'*10**(',i4,')];')
c-----
      endif
      endif
c
c
c...end of block if, depth<zstart
      endif
c
   20 continue
c===(end of depth loop, jz=1,nz)===
c
c
c...transfer scratch plots
      if(iprnt.ne.zero) then
      rewind(15)
      rewind(16)
      rewind(17)
      do 23 jz=1,nz
      read(15,'(a)',end=33) card
   23 write(14,'(a)') card
   33 continue
      do 24 jz=1,nz
      read(16,'(a)',end=44) card
   24 write(14,'(a)') card
   44 continue
      do 25 jz=1,nz
      read(17,'(a)',end=55) card
   25 write(14,'(a)') card
   55 continue
c
      write(14,805)
c...octave version
  805 format(' '/
     +'figure(1)'/
     +'clf'/
     +'set(gca(),''fontsize'',12)'/
     +'subplot(121)'/
     +'plot(r1(:,2),-r1(:,1),''-b'',''linewidth'',2)'/
     +'hold on'/
     +'plot(r2(:,2),-r2(:,1),''-r'',''linewidth'',2)'/
     +'xlabel(''Displacement '');'/
     +'ylabel(''Depth (m)'');'/
     +'title(''Motion Stress Vector (H and V)'')'/
     +'grid on')
      write(14,806)
  806 format('subplot(122)'/
     +'plot(r3(:,2),-r3(:,1),''-b'',''linewidth'',2)'/
     +'set(gca(),''fontsize'',12)'/
     +'hold on'/
     +'plot(r4(:,2),-r4(:,1),''-r'',''linewidth'',2)'/
     +'xlabel(''Stress'');'/
     +'title(''Motion Stress Vector (H and V)'')'/
     +'grid on')
c
      endif
c
   21 continue
c
      return
      end
c  |------------------------------------------------------------|
c  | pmatr2.f                                                   |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  |                                                            |
c  | pmatrx: compute propagation matrix for a constant layer    |
c  |                                                            |
c  |                                                            |
c  |------------------------------------------------------------|
c
      subroutine pmatr2(p,gmu,glame,grho,wfreq,fkn,
     +z0,z1)
      implicit real*8 (a-h,o-z)
c
      real*8 p(4,4),m,lm,k,z1,z0,r,w
      zero=0.00d+00
      f2=2.00d+00
c
c...get elastic parameters
      m=gmu
      lm=glame
      r=grho
      w=wfreq
      k=fkn
      dz=z1-z0
      a2=(lm+f2*m)/r
      b2=m/r
c
c...test for harmonic solution
      arg1=k*k-(w*w)/(b2)
      arg2=k*k-(w*w)/(a2)
      if(arg1.lt.zero .or. arg2.lt.zero) then
c
c========start g1 complex, g2 real block
      if(arg1.lt.zero .and. arg2.ge.zero) then
      g1=dsqrt(dabs(arg1))
      if(arg2.eq.zero) then
      g2=zero
      else
      g2=dsqrt(arg2)
      endif
      sg1=sin(g1*dz)
      cg1=cos(g1*dz)
      sg2=sinh(g2*dz)
      cg2=cosh(g2*dz)
c
      a=(-k/g1)
      c=(m*(-g1*g1 + k*k))/g1
      e=-f2*k*m
      b=-g2/k
      d=f2*m*g2
      f=(k*k*lm - (lm+f2*m)*g2*g2)/k
      fme=f-e
      adcb=a*d-c*b
c
      p(1,1)=(f*cg1-e*cg2)/fme
      p(3,3)=p(1,1)
c
      p(1,2)=-(c*sg2+d*sg1)/adcb
      p(4,3)=-p(1,2)
c
      p(1,3)=(a*sg2+b*sg1)/adcb
c
      p(1,4)=-(cg1-cg2)/fme
      p(2,3)=-p(1,4)
c
      p(2,1)=(a*f*sg1-b*e*sg2)/fme
      p(3,4)=-p(2,1)
c
      p(2,2)=(a*d*cg1-b*c*cg2)/adcb
      p(4,4)=p(2,2)
c
      p(2,4)=-(a*sg1-b*sg2)/fme
c
      p(3,1)=(c*f*sg1-e*d*sg2)/fme
c
      p(3,2)=(c*d)*((cg1-cg2)/adcb)
      p(4,1)=-p(3,2)
c
      p(4,2)=-(c*f*sg2+e*d*sg1)/adcb
c
      endif
c...============>end complex g1, real g2 block
c
c
c...============>totaly harmonic block
c....both g1 and g2 complex
      if(arg1.lt.zero .and. arg2.lt.zero) then
      g2=dsqrt(dabs(arg2))
      g1=dsqrt(dabs(arg1))
      sg2=sin(g2*dz)
      sg1=sin(g1*dz)
      cg2=cos(g2*dz)
      cg1=cos(g1*dz)
c
      a=(-k/g1)
      c=(m*(-g1*g1 + k*k))/g1
      e=-f2*k*m
      b=-g2/k
      d=f2*m*g2
      f=(k*k*lm + (lm+f2*m)*g2*g2)/k
      fme=f-e
      adcb=a*d-c*b
c
      p(1,1)=(f*cg1-e*cg2)/fme
      p(3,3)=p(1,1)
c
      p(1,2)=-(c*sg2+d*sg1)/adcb
      p(4,3)=-p(1,2)
c
      p(1,3)=(a*sg2+b*sg1)/adcb
c
      p(1,4)=-(cg1-cg2)/fme
      p(2,3)=-p(1,4)
c
      p(2,1)=(a*f*sg1+b*e*sg2)/fme
      p(3,4)=-p(2,1)
c
      p(2,2)=(a*d*cg1-b*c*cg2)/adcb
      p(4,4)=p(2,2)
c
      p(2,4)=-(a*sg1+b*sg2)/fme
c
      p(3,1)=(c*f*sg1+e*d*sg2)/fme
c
      p(3,2)=(c*d)*((cg1-cg2)/adcb)
      p(4,1)=-p(3,2)
c
      p(4,2)=-(c*f*sg2+e*d*sg1)/adcb
c
      endif
c...==============>end totaly harmonic (g1 and g2 complex)

c
      else
c
c...============>totaly non harmonic block
      if(arg2.eq.zero) then
      g2=zero
      else
      g2=dsqrt(arg2)
      endif
      if(arg1.eq.zero) then
      g1=zero
      else
      g1=dsqrt(arg1)
      endif
      sg2=sinh(g2*dz)
      sg1=sinh(g1*dz)
      cg2=cosh(g2*dz)
      cg1=cosh(g1*dz)
c
c      write(10,'(/''  alpha,beta,w,g1,g2 '',5(e10.4,1x))')
c     +a2,b2,w,g1,g2
c
      a=(-k/g1)
      c=(m*(g1*g1 + k*k))/g1
      e=-f2*k*m
      b=-g2/k
      d=f2*m*g2
      f=(k*k*lm - (lm+f2*m)*g2*g2)/k
      fme=f-e
      adcb=a*d-c*b
c
      p(1,1)=(f*cg1-e*cg2)/fme
      p(3,3)=p(1,1)
c
      p(1,2)=-(c*sg2-d*sg1)/adcb
      p(4,3)=-p(1,2)
c
      p(1,3)=(a*sg2-b*sg1)/adcb
c
      p(1,4)=-(cg1-cg2)/fme
      p(2,3)=-p(1,4)
c
      p(2,1)=(a*f*sg1-b*e*sg2)/fme
      p(3,4)=-p(2,1)
c
      p(2,2)=(a*d*cg1-b*c*cg2)/adcb
      p(4,4)=p(2,2)
c
      p(2,4)=-(a*sg1-b*sg2)/fme
c
      p(3,1)=(c*f*sg1-e*d*sg2)/fme
c
      p(3,2)=(c*d)*((cg1-cg2)/adcb)
      p(4,1)=-p(3,2)
c
      p(4,2)=-(c*f*sg2-e*d*sg1)/adcb
c
      endif
c...==============>end totaly non harmonic block
      return
      end

c# |------------------------------------------------------------|
c  | propgt.f                                                   |
c  | author: p. michaels                                        |
c  |------------------------------------------------------------|
c  |                                                            |
c  | propgt: propagate the motion/stress vector                 |
c  |         from depth z0 to depth z1                          |
c  |                                                            |
c  |------------------------------------------------------------|
      subroutine propgt(zi,mu,lame,rho,wfreq,phasev,nlay,
     +p,rv1,rv2,ndim,i5,z0,z1)
      implicit real*8 (a-h,o-z)
      real*8 mu(ndim),lame(ndim),rho(ndim),zi(ndim)
      real*8 tmp(4),tmp2(4),rv1(i5),rv2(i5)
      real*8 p(4,4)
c
      fkn=wfreq/phasev
c
      depth=(z0+z1)/2.00d+00
c
c...get elastic parameters
      call gparms(rho,mu,lame,zi,nlay,ndim,
     +depth,grho,gmu,glame)
c
      call pmatr2(p,gmu,glame,grho,wfreq,fkn,
     +z0,z1)
c
c      write(10,'('' p'',4(e10.4,1x))') (p(1,j),j=1,4)
c      write(10,'('' p'',4(e10.4,1x))') (p(2,j),j=1,4)
c      write(10,'('' p'',4(e10.4,1x))') (p(3,j),j=1,4)
c      write(10,'('' p'',4(e10.4,1x))') (p(4,j),j=1,4)
      do 5 j=1,4
    5 tmp(j)=rv1(j)
      call matmlt(p,tmp,tmp2,4,4)
      do 10 j=1,4
   10 rv1(j)=tmp2(j)
      rv1(5)=z1
      do 15 j=1,4
   15 tmp(j)=rv2(j)
      call matmlt(p,tmp,tmp2,4,4)
      do 20 j=1,4
   20 rv2(j)=tmp2(j)
      rv2(5)=z1
      return
      end
