// |$Id: c_boxit.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: c_boxit.c                                   |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: box car filter                                    |
   |                                                            |
   | s1=array to filter  s2=output  s3=scratch array            |
   | isign  -1=low-pass  0=high-pass  1=high-pass               |
   |                  (1-sinc^ntimes) (1-sinc)^ntimes           |
   | iwide=width of box in samples                              |
   | ntimes=number of times to apply box car                    |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>

      int c_boxit(int npts,int iwide,int isign,int ntimes,
         float s1[], float s2[])
     {

//...declare variables
      int               j,
                        k,
                        jtime,
                        ihw,
                        jend,
                        jstart,
                        ihwm1;
      float             fact,
                        *s3,
                        sfact,
                        sum,
                        srfact;

//...test parameters
      if((npts<1) | (iwide<0) | (isign<-1) | (isign>1) | (ntimes<1) )
     {  puts("ABORT: function c_boxit");
        puts("invalid parameter(s)");
        return(1);
     }

//...dynamic allocation of seimic trace memory
        s3=calloc(npts,sizeof(float));
          if(s3==NULL)
         {
          puts("ABORT: function c_boxit");
          puts("ABORT:s3 can't allocate enough trace memory");
          printf(" npts = %d",npts);
          return(1);
         }

//...make iwide odd if it is even
      iwide=(iwide/2)*2+1;
      ihw=(iwide+1)/2;
      ihwm1=ihw-1;

//...normalization for unity gain
      sfact=1./(float)iwide;
      srfact=(float)iwide;
      fact=1.0;
      for (k=0;k<ntimes;k++) sfact=fact*sfact;

//...copy to scratch
      for (j=0;j<npts;j++)
      s3[j]=s1[j];

//...start number of applications loop
      for (jtime=0;jtime<ntimes;jtime++)
     {
       if(iwide==1)                      //small iwide
      {
        for (j=0;j<npts;j++) s2[j]=s1[j];
        goto L500;
      } //endif

       if(npts<iwide)                    //small trace
      {
       sum=0.0;
       for (j=0;j<npts;j++) sum=sum+s3[j];
       for (j=0;j<npts;j++) s2[j]=sum;
       goto L500;
      } //endif

//...taper in (set all values=sum for 1/2 operator)
      sum=0.;
      for (k=0;k<iwide;k++) sum=sum+s3[k];
      for (k=0;k<ihw;k++)   s2[k]=sum;

//...main application: add next, subtract first
      jend=npts-ihw+1;
      jstart=ihw;
      for (j=jstart;j<jend;j++)
     {
      sum=sum-s3[j-ihw];
      sum=sum+s3[j+ihwm1];
      s2[j]=sum;
     } // end j

//...taper off (all values=sum of 1/2 operator width)
      jstart=jend;
      for (j=jstart;j<npts;j++) s2[j]=sum;

L500: ; //continue

//...High (1) or Low Pass?
      if (isign>=1)
     {
      for (j=0;j<npts;j++) s2[j]=s3[j]*srfact-s2[j];
     } //endif

//...prepare for next iteration
      for (j=0;j<npts;j++) s3[j]=s2[j];

     } // end jtime

      free(s3);

//...rescale to gain=1
      for (j=0;j<npts;j++) s2[j]=s2[j]*sfact;

//...High or Low Pass?
      if(isign!=0) return(0);
      for (j=1;j<npts;j++) s2[j]=s1[j]-s2[j];
      return(0);
     } //  end c_boxit
