c |$Id: arma.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: arma.f                                       |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | Filters seismic data (module formed from bfil.f            |
c | Method: Bilinear Transform of Laplace Transform            |
c |                                                            |
c |          ARMA REALIZATION                                  |
c |                                                            |
c --------------------------------------------------------------
c
      subroutine ARMA(npts,s1,s2,itype,fcenter,bwidth,npoles,fsamin,
     1ifaz)
      real*4            s1(*),s2(*)
c
c...must be even number of poles
      npow=npoles/2
      npoles=2*npow
c//////////////////////////////Branch to Filter Type/////////////
c
      if(itype.eq.1) then
      call BPASS(fcenter,bwidth,npow,fsamin,C0,C1,C2,D0,D1,D2)
      endif
      if(itype.eq.0) then
      call LPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
      endif
      if(itype.eq.2) then
      call HPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
      endif
c////////////////////////////////////////////////////////////////
c
c...apply filter....TIME DOMAIN ARMA...........
c
      if(ifaz.eq.1) then
c
c...minimum phase
c
      do 301 jpow=1,npow
      s2(1)=(s1(1)*C0)/D0
      s2(2)=(s1(2)*C0+s1(1)*C1-s2(1)*D1)/D0
c
c
      do 351 jt=3,npts
      tt1=(s1(jt-2)*C2)/D0
      tt2=(s1(jt-1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt-2)*D2)/D0
      tt5=(s2(jt-1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  351 continue
c
      do 361 jt=1,npts
  361 s1(jt)=s2(jt)
  301 continue
      else
c
c...zero phase (forward/reverse trick)
c
      do 401 jpow=1,npow
      s2(1)=(s1(1)*C0)/D0
      s2(2)=(s1(2)*C0+s1(1)*C1-s2(1)*D1)/D0
c
c
      do 451 jt=3,npts
      tt1=(s1(jt-2)*C2)/D0
      tt2=(s1(jt-1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt-2)*D2)/D0
      tt5=(s2(jt-1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  451 continue
c
      do 461 jt=1,npts
  461 s1(jt)=s2(jt)
  401 continue
c
      do 402 jpow=1,npow
      s2(1)=(s1(npts)*C0)/D0
      s2(2)=(s1(npts-1)*C0+s1(npts)*C1-s2(npts)*D1)/D0
c
c
      npts3=npts-2
      do 452 jt=npts3,1,-1
c
      tt1=(s1(jt+2)*C2)/D0
      tt2=(s1(jt+1)*C1)/D0
      tt3=(s1(jt)*C0)/D0
c
      tt4=(s2(jt+2)*D2)/D0
      tt5=(s2(jt+1)*D1)/D0
      s2(jt)=(tt1+tt2+tt3-tt4-tt5)
  452 continue
c
      do 462 jt=1,npts
  462 s1(jt)=s2(jt)
  402 continue
      endif
c
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bpass.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |                                                            |
c |------------------------------------------------------------|
c | Computes bandpass filter Z-transform                       |
c | Method: Bilinear Transform of Laplace Transform            |
c |                                                            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c

      subroutine BPASS(fcenter,bwidth,npow,fsamin,C0,C1,C2,D0,D1,D2)
c
c...convert to radians/sec
      pi=3.1415926
      wcd=2.*pi*fcenter
      bwd=bwidth*2.*pi
c
c...map to Laplace S-Plane
      wc=tan((-wcd*fsamin)/2.0)
      bw2=tan(((-wcd*fsamin)-(bwd*fsamin)/2.0)/2.0)
      bw1=tan(((-wcd*fsamin)+(bwd*fsamin)/2.0)/2.0)
      bw=bw2-bw1
      bw=bw/sqrt(2.0**(1./float(npow))-1)
c
c...Bilinear Transform s=(z-1)/(z+1)
c
c...numerator (zeros)
      C0=-bw
      C1=0.0
      C2=bw
c
c...denominator (poles)
      D0=1.-bw+wc*wc
      D1=2.*(wc*wc-1.)
      D2=1.+bw+wc*wc
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lpass.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |                                                            |
c |------------------------------------------------------------|
c | Computes lowpass  filter Z-transform                       |
c | Method: Bilinear Transform of Laplace Transform            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c

      subroutine LPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
c
c...convert to radians/sec
      pi=3.1415926
      wcd=2.*pi*fcenter
c
c...map to Laplace S-Plane
      wc=tan((-wcd*fsamin)/2.0)
      wo=(1./(2.0**(1./float(npow))-1.0)**(.25))*wc
      bw=wo*sqrt(2.0)
c
c...Bilinear Transform s=(z-1)/(z+1)
c
c...numerator (zeros)
      C0=wo*wo
      C1=wo*wo*2.0
      C2=wo*wo
c
c...denominator (poles)
      D0=1.-bw+wo*wo
      D1=2.*(wo*wo-1.)
      D2=1.+bw+wo*wo
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: hpass.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |                                                            |
c |------------------------------------------------------------|
c | Computes highpass filter Z-transform                       |
c | Method: Bilinear Transform of Laplace Transform            |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c
      subroutine HPASS(fcenter,npow,fsamin,C0,C1,C2,D0,D1,D2)
c
c...convert to radians/sec
      pi=3.1415926
      wcd=2.*pi*fcenter
c
c...map to Laplace S-Plane
      wc=tan((-wcd*fsamin)/2.0)
      wo=((2.0**(1./float(npow))-1.0)**(.25))*wc
      bw=wo*sqrt(2.0)
c
c...Bilinear Transform s=(z-1)/(z+1)
c
c...numerator (zeros)
      C0=1.0
      C1=-2.0
      C2=1.0
c
c...denominator (poles)
      D0=1.-bw+wo*wo
      D1=2.*(wo*wo-1.)
      D2=1.+bw+wo*wo
c
      return
      end
