c |$Id: nlogn.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: E. A. Robinson (1967, p.63)                        |
c | PROGRAM NAME: nlogn.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c | Copyright (c) 1967 by Holden-Day Inc. 500 Sansome St.,     |
c | San Francisco, all rights reserved                         |
c |                                                            |
c | Holden-Day Inc., Liquidation Trust, 1259 S.W. 14th Street  |
c | Boca Raton, FL 33486 Phone: (561)750-9229 Fax: 561.394.6809|
c |                                                            |
c |    This subroutine is included for distribution with Basic |
c | Seismic Utilities (BSU) by joint permission of the author, |
c | Enders Robinson, and Holden-Day Inc. Liquidation Trust,    |
c | Frederick H. Murphy, Trustee.  Being part of the BSU       |
c | package, it is governed by the terms of the GPL license,   |
c | and is distributed in the hope that it will be useful, but |
c | WITHOUT ANY WARRANTY; without even the implied warranty    |
c | of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    |
c | See the GNU General Public License for more details.       |
c | Paul Michaels, February 2002 <pm@cgiss.boisestate.edu>     |
c |                                                            |
c |------------------------------------------------------------|
c |Minor modifications, Fortran IV to Fortran 77               |
c |and some other things which I needed once for use on a cray |
c |(pi variables, fnpts, fn )      P. Michaels                 |
c |------------------------------------------------------------|
c |                      REFERENCE                             |
c | Robinson, E.A., 1967, "Multichannel Time Series Analysis   |
c |   with Digital Computer Programs", Holden-Day, 298p.       |
c |                                                            |
c |------------------------------------------------------------|
c | nlogn: computes fast fourier transform, radix2             |
c |                                                            |
c | sign = -1.0 forward transform (sum exp(-2pi*i*j*k/npts))   |
c | sign = +1.0 inverse  (1/npts)*sum exp(+2pi*i*j*k/npts))    |
c |                                                            |
c |------------------------------------------------------------|

      subroutine nlogn(npts,x,sign)
      complex x(*)
      integer m(50),n
      complex wk,hold,q
      pi=3.14159265358979323846264338327950
      pi2=pi*2.000000000000000000000000
      fnpts=float(npts)
      fn=alog10(fnpts)/alog10(2.00000000000000)
      n=int(fn+0.5,4)
      lx=2**n
      do 1 i=1,n
    1 m(i)=2**(n-i)
      do 4 l=1,n
      nblock=2**(l-1)
      lblock=lx/nblock
      lbhalf=lblock/2
      k=0
      do 4 iblock=1,nblock
      fk=k
      flx=lx
      v=(sign*pi2*fk)/flx
      wk=cmplx(cos(v),sin(v))
      istart=lblock*(iblock-1)
      do 2 i=1,lbhalf
      j=istart+i
      jh=j+lbhalf
      q=x(jh)*wk
      x(jh)=x(j)-q
      x(j)=x(j)+q
    2 continue
      do 3 i=2,n
      ii=i
      if(k.lt.m(i)) go to 4
    3 k=k-m(i)
    4 k=k+m(ii)
      k=0
      do 7 j=1,lx
      if(k.lt.j) go to 5
      hold=x(j)
      x(j)=x(k+1)
      x(k+1)=hold
    5 do 6 i=1,n
      ii=i
      if(k.lt.m(i)) go to 7
    6 k=k-m(i)
    7 k=k+m(ii)
      if(sign.lt.0.0) go to 500
      do 8 i=1,lx
    8 x(i)=x(i)/flx
  500 continue
      return
      end
