c |$Id: xcor.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: xcor.f                                       |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |                                                            |
c | xcor:   computes cross correlation between two             |
c |         traces (normalized)                                |
c | c=moving trace, v=stationary trace, cc=output trace        |
c | t1=start time  t2=end time  tlagmx=total max. lag in sec.  |
c | tzero=time of zero lag on cc  dt=sample rate               |
c |                                                            |
c --------------------------------------------------------------
c
      subroutine xcor(npts,t1,t2,tlagmx,tzero,dt,c,v,cc)
      implicit real*4 (a-h,o-z)
      real*4 v(*),c(*),cc(*)
c
c...compute start and end samples
      is1=int(t1/dt + 0.5) + 1
      is2=int(t2/dt + 0.5) + 1
c
      if(is1.lt.1) then
      is1=1
      write(*,'('' warning--is1 reset in xcor'')')
      endif
c
      if(is2.gt.npts) then
      is2=npts
      write(*,'('' warning--is2 reset in xcor'')')
      endif
c
      izero=int(tzero/dt)
      ismx=int(tlagmx/dt)
      ibak=izero
      izero=izero+1
c
c...correlate--------------------------------------------
      do 100 kk=1,ismx
      sum=0.00e+00
      jj1=1-kk+is1+ibak
      ii=is1
c
      if(jj1.lt.is1) then
      ii=is1+(is1-jj1)
      jj1=is1
      endif
c
      do 150 jj=jj1,is2
      test=is2-ii
      if(test) 110,50,50
   50 sum=sum+v(ii)*c(jj)
      ii=ii+1
  150 continue
  110 cc(kk)=sum
  100 continue
c
c...normalize by sqrt of product of autocorrelations at zero lag
      ca=0.00e+00
      cv=0.00e+00
      do 200 kk=is1,is2
      ca=ca+c(kk)*c(kk)
  200 cv=cv+v(kk)*v(kk)
      cacv=sqrt(ca*cv)
      do 300 kk=1,ismx
      if(cacv) 250,260,250
  250 cc(kk)=cc(kk)/cacv
  260 continue
  300 continue
      return
      end
