! --------------------------------------------------------------
! | author: p. michaels                      date: 07 dec 92   |
! |                                                            |
! | genwav.f                                                   |
! |------------------------------------------------------------|
! | genwav:                                                    |
! | Creates namelists for waves.f which computes synthetic     |
! | seismograms for surface waves.                             |
! |                                                            |
! | MODIFICATION: 24 April 2014, 32bit to 64 bit port          |
! |               return to ry=nrec*0.0,                       |
! |                         rz=nrec*0.0,                       |
! |------------------------------------------------------------|
! |  NOTICE:                                                   |
! |  Copyright (C) 2017 Paul Michaels                          |
! |  <pm@cgiss.boisestate.edu>                                 |
! |  This program is free software; you can                    |
! |  redistribute it and/or modify it under the terms          |
! |  of the GNU General Public License as published            |
! |  by the Free Software Foundation; either version           |
! |  3 of the License, or (at your option) any later           |
! |  version.  This program is distributed in the              |
! |  hope that it will be useful, but                          |
! |  WITHOUT ANY WARRANTY; without even the implied            |
! |  warranty of MERCHANTABILITY or FITNESS FOR A              |
! |  PARTICULAR PURPOSE.  See the GNU General Public           |
! |  License for more details.                                 |
! |  You should have received a copy of the GNU                |
! |  General Public License along with this program;           |
! |  if not, write to the Free Software Foundation,            |
! |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
! |                                                            |
! --------------------------------------------------------------
!
      program main
      implicit real*4 (a-h,o-z)
      parameter (ngrp=100)
      real*4 xoffst(ngrp)
      character*40 ofile
      character*40 crvfil
!
!...Input parameters
!
      write(*,*)'  enter name of namelist file (40 char)'
      write(*,*)'  Example: waves.d'
      read(*,'(a)') ofile
!
      write(*,*)'  enter name of dispersion curve file'
      write(*,*)'  (this is file from disper.f)'
      write(*,*)'  Example: earth.crv'
      read(*,'(a)') crvfil
!
      write(*,*)'  enter near offset:  xnear '
      read(*,*) xnear
!
      write(*,*)'  enter group interval:  delx '
      read(*,*)  delx
!
      write(*,*)'  enter number of receivers:  nrec'
      read(*,*) nrec
!
      if(nrec.gt.ngrp) then
      write(*,*)'  !!! abort !!! number of offsets too large'
      write(*,'(''increase dimension ngrp='',i5)') ngrp
      stop
      endif
!
      xmax=xnear+float(nrec-1)*delx
!
      write(*,*)'  enter minimum group velocity expected'
      read(*,*)  betamn
!
      tmaxr=(xmax/betamn)*2.0
      write(*,'(''  RECOMMENDED minimum tmax='',f10.4)') tmaxr
!
      write(*,*)'  enter: maximum trace time, tmax'
      read(*,*) tmax
!
      write(*,*)'  enter: sample interval (seconds), fsamin'
      read(*,*) fsamin
!
      write(*,*)'  enter frequencies: fmin, fmax '
      read(*,*) fmin, fmax
!
      write(*,*)'  enter maximum mode to include '
      read(*,*) modemx
!
      write(*,*)'  enter ksw switch 0=c plot, 1=k plot'
      read(*,*) ksw
!
      write(*,*)'  enter type of plot format, mapmat'
      write(*,*)'  0=octave (Matlab)  1=scilab'
      read(*,*) mapmat
!
      write(*,*)'  enter Output option 0=Vertical  1=Radial '
      read(*,*) irvsel
!
      write(*,*)'  enter source depth'
      read(*,*) sz
!
      write(*,*)' enter (3) diagonal elements, moment tensor'
      read(*,*) t11,t22,t33
!
!
!...output a namelist%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
!
      call namelw(tmax,xnear,delx,nrec,fsamin,crvfil,mapmat, &
      fmin,fmax,modemx,ksw,sz,t11,t22,t33,xoffst,ofile,irvsel)
!
!
!
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!
      write(*,'(''  output in =====>'',a)') ofile
      close(8)
      stop
      end
!#--------------------------------------------------------------
! | author: p. michaels                      date: 07 dec 92   |
! |                                                            |
! | namelw: generates a waves namelist                         |
! |                                                            |
! |------------------------------------------------------------|
!
      subroutine namelw(tmax,xnear,delx,nrec,fsamin,crvfil,mapmat, &
      fmin,fmax,modemx,ksw,sz,t11,t22,t33,xoffst,ofile,irvsel)
      implicit real*4 (a-h,o-z)
      real*4 xoffst(*)
      integer*2 modes(10)
      character*40 ofile
      character*40 crvfil
      character*1 blank
!
      write(blank,'('' '')')
!
!...open output file
!
      open(12,file=ofile,access='sequential')
!
!...calculate parameters
      zero=0.0
      if(modemx.gt.9) modemx=9
      do 18 imode=1,10
   18 modes(imode)=0
      do 19 imode=1,modemx
      modes(imode)=imode
   19 continue
!
!...offsets
!
      xoffst(1)=xnear
      do 20 jx=2,nrec
      xoffst(jx)=xoffst(jx-1)+delx
   20 continue
      xmax=xoffst(nrec)
!
!...compute number of points, radix 2
      npts=int(tmax/fsamin)*2
      npr2=2
      do 5 j=1,50
      if(npr2.gt.npts) go to 10
      npr2=npr2*2
    5 continue
   10 continue
      tmax=npr2*fsamin
      delf=1.00/tmax
      nfreq=fmax/delf
!
      write(*,'(''  Padded Radix 2 tmax='',f10.4)') tmax
      write(*,'(''  Number of points in signal='',i5)') npr2
      write(*,'(50(1h-))')
      write(*,'(''  .......Frequency interval='',f12.8)') delf
      write(*,*)' NOTE: Frequency Interval MUST MATCH DISPER OUTPUT'
      write(*,*)' WAVES will output signal length = 1.0/delf'
      write(*,*)' IF MISMATCHED: CHANGE sample rate in WAVES'
      write(*,*)'                or RERUN DISPER'
      write(*,'(50(1h-))')
      write(*,'(''  Number of frequencies='',i5)') nfreq
!
!...determine length of curfil
      jlen=0
      do 33 jc=1,40
      jlen=jlen+1
      if(crvfil(jc:jc).eq.blank) then
      jlen=jlen-1
      go to 34
      endif
   33 continue
   34 continue
!
!...write namelist
      write(12,*)' &waves'
      write(12,'(''  ksw='',i2,'','','' stepz=20,'')')  ksw
      write(12,99) (modes(imode),imode=1,9)
   99 format('  modes=',9(i1,','))
      write(12,100) fmin,fmax
  100 format('  fmin=',f10.4,', fmax=',f10.4,',')
      write(12,110) fsamin
  110 format('  fsamin=',f10.5,',')
      write(12,120) crvfil(1:jlen)
  120 format('  curve=''',a,''', ')
      write(12,130) mapmat
  130 format('  mapmat=',i1,','/ &
      '  matlb1=''matc.m'', ', &
      '  scilb1=''matc.sci'', ')
      write(12,140)
  140 format('  matlb2=''matu.m'', ', &
      '  scilb2=''matu.sci'', ')
      write(12,142) irvsel
  142 format('  irvsel=',i1,',')
      write(12,150)
  150 format('  ofile=''waves.tmp'', /')
!
      write(12,*)' &source'
#ifdef WIN
      write(12,160) t11,t22,t33
  160 format('  tm=',f4.1,', 0.0, 0.0, 0.0,',f4.1,', 0.0,' &
      '0.0, 0.0,',f4.1,',')
#else
      write(12,160) t11,t22,t33
  160 format('  tm=',f4.1,', 0.0, 0.0,'/'     0.0,',f4.1,', 0.0,'/ &
      '     0.0, 0.0,',f4.1,', /')
#endif
      write(12,170) sz
  170 format('  sz=',f8.2,', sy=0.00, sx=0.00, /')
!
      write(12,*)' &recvr'
      write(12,180) nrec,nrec,nrec
  180 format('  nrec=',i2,','/'  rz=',i2,'*0.0, '/'  ry=',i2'*0.0, ')
#ifdef WIN
      write(12,182)
  182 format("rx= ",$)
      do 200, kk=1,nrec
      write(12,190) xoffst(kk)
  190 format(f8.3,', ',$)
  200 continue
#else
      write(12,182)
  182 format("rx= ",$)
      write(12,190) (xoffst(kk),kk=1,nrec)
  190 format(5(2x,f8.3,','))
#endif
      write(12,*)' /'
      close(12)
      return
      end
