!  |------------------------------------------------------------|
!  | $Id: waves.F90,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $        |
!  |                             MODIFICATIONS:                 |
!  |                             changed wavelet (sub moment)   |
!  |                             added Radial component option  |
!  |                             irvsel  0=Vert  1=Radial       |
!  |                                                            |
!  | 24 April 2014:  32bit to 64 bit OS fix memory problems     |
!  |                 dimension of irz converted to integer*2    |
!  |                 Also, edit genwav.f so ry=nrec*0.0,        |
!  |                                        rz=nrec*0.0         |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  |  waves:   main program for computing displacement          |
!  |           seismic traces for rayleigh waves (vert. comp.)  |
!  |                                                            |
!  |     <<<<<<<<<<<double precision version>>>>>>>>>>>>>>>>>   |
!  |     <<<< Note: keep xr,xi,amp,phz,p,g,f,tzero,samin >>>>   |
!  |     <<<<       and sign all single precision to be  >>>>   |
!  |     <<<<       compatable with sublib4              >>>>   |
!  |     <<<<                                            >>>>   |
!  |     >>>>  modified for group velocity plot          >>>>   |
!  |     <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<   |
!  |                                                            |
!  |------------------------------------------------------------|
!  |  Namelist Definitions:                                     |
!  | &waves                                                     |
!  |    deltz=depth increment for integration                   |
!  |    iout = 0 line printer plot of traces                    |
!  |           1 traces output to a file, segy                  |
!  |    iprint = 0 supress line printer output                  |
!  |            -1 print freq,phase vel, group vel              |
!  |          >= 2 print aa1 (p & s mix),test (lagrangian)      |
!  |               rm test, total vectors                       |
!  |          >= 1 amp and phase of displacement                |
!  |      note: trace plots and spectral plots are generated    |
!  |            on the line printer unless iout=1               |
!  |    mapmat = switch to select scilab or octave.m file       |
!  |              for plot output  0=octave 1=scilab            |
!  |                                                            |
!  |    maple1=output file name for maple plot                  |
!  |           (phase velocity or wavenumber dispersion)        |
!  |    curve=name of input file with dispersion curve          |
!  |           (from disper.f)                                  |
!  |    idft = 0 use fft (may modify final sample rate)         |
!  |         = 1 use dft (slower execution) deactivated 30/10/91|
!  |    ksw= 1 plot wavenumber, 0 plot phase velocity           |
!  |    fsamin = sample rate in seconds for output traces       |
!  |    tzero = zero reference time for spectral plot           |
!  |    iswdrv = derivative switch for motion stress vector     |
!  |              0 = colocational polynomial derivatives       |
!  |              1 = autonomous matrix equation                |
!  |3 Oct 1999 Modification:  Add horizontal displacement option|
!  |    irvsel = select vertical or horizontal displacement     |
!  |             for computed traces to be output               |
!  |              0 = vertical displacement (default)           |
!  |              1 = radial (in line horizontal)               |
!  |                                                            |
!  | &model                                                     |
!  |    nlay=number of layers                                   |
!  |    rho=vector of densities (one for each layer)            |
!  |    mu=vector of shear moduli                               |
!  |    lame=vector of lame's moduli                            |
!  |    zi=vector of layer tops                                 |
!  |                                                            |
!  | &source                                                    |
!  |    sz=source depth                                         |
!  |    sx=source x-coord.                                      |
!  |    sy=source y-coord.                                      |
!  |    tm=moment tensor of source                              |
!  |         minimum phase source mo(t)                         |
!  |    bias= factor for minimum spectral level (phase calc)    |
!  |          typical: .01<bias<1.0                             |
!  |    alpha= decay factor for step function                   |
!  |          typical: alpha>0.0 (otherwise spectral notches)   |
!  |                                                            |
!  | &recvr                                                     |
!  |    nrec=number of receivers                                |
!  |    rx=vector of receiver x-coord.                          |
!  |    ry=vector of receiver y-coord.                          |
!  |    rz=vector of receiver depths                            |
!  |------------------------------------------------------------|
!  |  Dimensions:                                               |
!  |    ndim= number of control points in the model             |
!  |    nsmax= maximum length of seismic traces                 |
!  |    nfmax= maximum number of frequencies                    |
!  |    ngrp=  maximum number of receiver channels              |
!  |    maxmod= maximum number of modes that can be used        |
!  |------------------------------------------------------------|
!  |  Parameter Statements:                                     |
!  |  The following subroutines have parameter statements       |
!  |  pltvel....msv....lgrtst....dspspc....ptrace....           |
!  |------------------------------------------------------------|
!  |  Subroutines Required:                                     |
!  |  Inline:                                                   |
!  |           pltvel...matmlt...msv...lgrtst...dspspc...       |
!  |           ptrace...intgr4...propgt...intgr3...dpoly2...    |
!  |           hlfint...pmatr2...propg2...gparms...halfsp...    |
!  |           moment...segy...cubic...amatrx                   |
!  |  Sublib4:                                                  |
!  |           nrad2...czero...flist...nlogn...tlist...         |
!  |------------------------------------------------------------|
!  |  NOTICE:                                                   |
!  |  Copyright (C) 2017 Paul Michaels                          |
!  |  <pm@cgiss.boisestate.edu>                                 |
!  |  This program is free software; you can                    |
!  |  redistribute it and/or modify it under the terms          |
!  |  of the GNU General Public License as published            |
!  |  by the Free Software Foundation; either version           |
!  |  3 of the License, or (at your option) any later           |
!  |  version.  This program is distributed in the              |
!  |  hope that it will be useful, but                          |
!  |  WITHOUT ANY WARRANTY; without even the implied            |
!  |  warranty of MERCHANTABILITY or FITNESS FOR A              |
!  |  PARTICULAR PURPOSE.  See the GNU General Public           |
!  |  License for more details.                                 |
!  |  You should have received a copy of the GNU                |
!  |  General Public License along with this program;           |
!  |  if not, write to the Free Software Foundation,            |
!  |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
!  |                                                            |
!  |------------------------------------------------------------|
!
      program main
      parameter (ndim=100, i5=5, i4=4)
      parameter (nsmax=20000)
      parameter (nfmax=1000,maxmod=10,ngrp=100)
!
      implicit real*8 (a-h,o-z)
!
      real*8 r1atz(ngrp),r2atz(ngrp)
      real*8 cph(ngrp),sph(ngrp),frange(ngrp)
      complex*8  gf(nfmax,ngrp)
!
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
      real*8 pv(maxmod),pk(maxmod)
      real*8 plt(nfmax,maxmod),pltu(nfmax,maxmod)
      character*40 infile
      character*74 card
      character*40 ofile
!      character*40 maple1,maple2
      character*40 matlb1,matlb2
      character*40 scilb1,scilb2
      character*40 curve
!
      REAL*4 xr(nsmax),xi(nsmax),amp(nsmax),phz(nsmax)
      REAL*4 p(nsmax),f(nsmax)
      REAL*4 tzero
      complex*8  x(nsmax)
      complex*8  y(nsmax)
!
      character*4 line,igeoor
      integer*2 iyear,iday
      integer*2 modes(maxmod-1)
!
!...receivers
      real*8 rz(ngrp),rx(ngrp),ry(ngrp)
      real*8 sx,sy,sz
      integer*2 irz(ngrp)
      real*8 tm(3,3)
!
!
      common /array1/ gf,plt,pltu
!
      namelist /waves/ &
      deltz,ofile,scilb1,iprint,iout,modes, &
      fmin,fmax,ksw,curve,tzero,fsamin, &
      iyear,iday,line,scilb2,stepz,mapmat, &
      matlb1,matlb2,iswdrv,irvsel
!
      namelist /model/ rho,mu,lame,nlay,zi
!
      namelist /source/ tm,sz,sx,sy,alpha,bias
!
      namelist /recvr/ rz,rx,ry,nrec
!
!cc      write(infile,'("waves.d")')
      write(ofile,'("waves.tmp")')
!      write(maple1,'("maplec")')
!      write(maple2,'("mapleu")')
      write(matlb1,'("phase.m")')
      write(matlb2,'("group.m")')
      write(scilb1,'("phase.sci")')
      write(scilb2,'("group.sci")')
      write(curve,'("disper.crv")')
!
      write(igeoor,'("vert")')
      write(line,'("v-1 ")')
!
!...set defaults
!.../////////// stepz increments per minimum shear ////
!...wave wavelength for numerical integration
      stepz=20.0
!...//////////////////////////////////////////////////
!
      do 3 j=1,(maxmod-1)
    3 modes(j)=int(j,2)
      mapmat=0
      irvsel=0
      iswdrv=0
      iyear=91
      iday=17
      zero=0.
      fsamin=zero
      tzero=0.0
      iprint=-1
      alpha=1.0
      bias=.1
      iout=1
      idft=1
      ksw=0
      nlay=2
      rho(1)=1500.
      rho(2)=2000.
      mu(1)=135.e+06
      mu(2)=1280.e+06
      lame(1)=465.e+06
      lame(2)=1940.e+06
      zi(1)=0.
      zi(2)=40.
      fmax=10.0
      fmin=1.0
      pi=3.14159265358979311599796346854
      pi2=2.00000000*pi
      deltz=1.
      do 1 j=1,nfmax
      do 1 k=1,maxmod
    1 plt(j,k)=0.0
      do 2 j=1,nfmax
      do 2 k=1,ngrp
    2 gf(j,k)=cmplx(0.0,0.0)
!
!...test if input file in argument list on command line
      nargs=iargc()
      if(nargs.eq.0) then
      write(*,*)'  enter: name of input file'
      read(*,'(a)') infile
      else
      call getarg(1,infile)
      endif
!
!...open files
      open(09, file=infile, access='sequential')
!
!...read input data
      read(09,nml=waves)
      read(09,nml=source)
      read(09,nml=recvr)
!
!...round  off protection
      if(fsamin.ne.0.0E+00) then
      fsamin=fsamin + .1*fsamin
      endif
!
      if (nrec.gt.ngrp) then
      write(10,'("  abort: too many receivers")')
      stop
      endif
!
      open(10, file=ofile, access='sequential')
!
      rewind 9
      write(10,'(//)')
    7 read(09,'(a)',end=8) card
      write(10,'(5x,a)') card
      go to 7
    8 continue
      close(09)
!
!...transpose i/o
      do 16 j=1,3
      do 16 k=j,3
      hold=tm(j,k)
      tm(j,k)=tm(k,j)
      tm(k,j)=hold
   16 continue
!
!...open dispersion curve (freq., wavenumber..fund..mode)
      open(unit=08, file=curve, access='sequential')
      read(08,nml=model)
      write(10,700) nlay
  700 format('  nlay=',i5)
      write(10,'("  rho=")')
      write(10,701) (rho(kk),kk=1,nlay)
  701 format(5(2x,e10.4))
      write(10,'("  lame=")')
      write(10,701) (lame(kk),kk=1,nlay)
      write(10,'("  mu=")')
      write(10,701) (mu(kk),kk=1,nlay)
      write(10,'("  zi=")')
      write(10,701) (zi(kk),kk=1,nlay)
      write(10,'(1x,70(1h+))')
!
!...compare deltz with minimum wavelength
      betamn=999999.
      do 17 kk=1,nlay
      beta=sqrt(mu(kk)/rho(kk))
      if(beta.lt.betamn) betamn=beta
   17 continue
      wavelm=betamn/fmax
      deltw=wavelm/stepz
      if(deltz.gt.deltw) deltz=deltw
!
!...trim deltz
!
      nz=int(zi(nlay)/deltz + 0.50)
      deltz=zi(nlay)/dfloat(nz)
!
!...make nz even
      ievn=(nz/2)*2
      if(nz.ne.ievn) then
      nz=nz-1
      deltz=zi(nlay)/float(nz)
      endif
!
      write(*,'("  nz=",i5)') nz
      write(10,'("  nz=",i5)') nz
      write(*,'("  trimmed deltz=",f10.5)') deltz
      write(10,'("  trimmed deltz=",f10.5)') deltz
!
!...compute geometrical factors
      do 200 jr=1,nrec
!cc      frange(jr)=dsqrt((sx-rx(jr))**2 + (sy-ry(jr))**2)
      frange(jr)=sqrt((sx-rx(jr))**2 + (sy-ry(jr))**2)
      if(frange(jr).eq.0.0D+00) then
      rx(jr)=1.d-10
      ry(jr)=1.d-10
!cc   frange(jr)=dsqrt((sx-rx(jr))**2 + (sy-ry(jr))**2)
      frange(jr)=sqrt((sx-rx(jr))**2 + (sy-ry(jr))**2)
      endif
      cph(jr)=rx(jr)/frange(jr)
      sph(jr)=ry(jr)/frange(jr)
      irz(jr)=int(rz(jr)/deltz,2)
  200 continue
!
      lc=1
      write(10,201)
  201 format(/6x, &
      "| frequency |",  & 
      " phase velocity (fundamental ====> higher modes)    |")
!
!...start frequency loop
      jf=1
  500 continue
!
      if(lc.gt.55) then
      write(10,'(a)') char(12)
      lc=1
      write(10,202)
  202 format(/6x, &
      "| frequency |", &
      " phase velocity (fundamental ====> higher modes)    |")
      endif
!
!...read wavenumbers
      do 11 jv=1,maxmod
   11 pv(jv)=0.0
!
      read(8,*,end=33) fq,(pk(jj),jj=1,9)
      if(fq.lt.fmin) go to 500
      wfreq=fq*(2.000000*pi)
!
!...number of modes found
      nmode=0
      do 501 jm=1,maxmod-1
  501 if(pk(jm).gt.(.00001)) nmode=jm
!
      if(nmode.eq.0) then
      write(*,'(" freq=",f10.4," skipped, no modes given")') fq
      go to 500
      endif
!
      do 12 jv=1,nmode
   12 pv(jv)=wfreq/pk(jv)
!
!...output iteration
      if(lc.gt.55) then
      write(10,'(a)') char(12)
      lc=1
      write(10,502)
  502 format(/6x, &
      "| frequency |", &
      " phase velocity (fundamental ====> higher modes)    |")
      endif
! phase velocity
      if(nmode.ge.6) then
      write(10,800) fq,(pv(jm),jm=1,5)
      write(10,800) fq,(pv(jm),jm=6,nmode)
      lc=lc+2
      else
      write(10,800) fq,(pv(jm),jm=1,nmode)
      lc=lc+1
      endif
  800 format(7x,f10.4,' | ',5(f10.3),' |')
!
!...save data for plot
      if(jf.gt.nfmax) then
!     write(10,'("  abort: more than nfmax frequencies")')
!     go to 999
      endif
      do 483 kf=1,nmode
  483 plt(jf,kf+1)=pv(kf)
      plt(jf,1)=fq
!
      jf=jf+1
      if(fq.lt.fmax) go to 500
   33 nfreq=jf-1
! -----end frequency loop
!
      if(nfreq.gt.nfmax) then
      write(10,'(" abort---too many frequencies")')
      stop
      endif
!
!...create plot phase velocity file
      icorg=0
      call pltvel(ksw,icorg,scilb1,scilb2,nmode,nfreq, &
      mapmat,matlb1,matlb2)
!
!...compute eigenvectors
      call msv(nfreq,iswdrv, &
      lame,mu,rho,zi,nlay,rv1,rv2,iout, &
      deltz,rtotal,ndim,i5,modes,p,f, &
      r1ath,r2ath,r1atz,r2atz,dr1ath,dr2ath,nrec, &
      frange,cph,sph,rz,iprint,tm,nsmax,sz,irz, &
      xr,xi,amp,phz,x,y,tzero,fsamin,idft, &
      rx,ry,sx,sy,iyear,iday,line,igeoor,flgrmx,fqlgrx,irvsel,mapmat)
!
!...create plot group velocity file
      icorg=1
      call pltvel(ksw,icorg,scilb1,scilb2,nmode,nfreq, &
      mapmat,matlb1,matlb2)
!
!
      close(8)
      close(10)
      if(iout.eq.1) close(11)
      open(unit=8,access='sequential',file='waves.his')
      krec=0
  600 continue
      read(8,'(a)',end=601) card
      krec=krec+1
      go to 600
  601 continue
      rewind(8)
      do 602 jr=1,krec
  602 read(8,'(a)') card
      write(8,702) infile(1:12),flgrmx,fqlgrx
  702 format(a,'  maximum lagrangian =',f10.7, &
      '  at frequency=',f10.4)
      close(8)
      stop
      end
!# |------------------------------------------------------------|
!  | waves.f                                 date:  27 apr 93   |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | amatrx:  fills matrix with elastic parameters              |
!  |------------------------------------------------------------|
      subroutine amatrx(a1,grho,gmu,glame,fkn1,wfreq, &
      depth,i5)
      implicit real*8 (a-h,o-z)
      real*8 a1(i5,i5)
!
      f4=4.000000000000000
      f1=1.000000000000000
      zero=0.000000000000000
      x=glame+2.*gmu
      a1(1,1)=zero
      a1(1,2)=fkn1
      a1(1,3)=f1/gmu
      a1(1,4)=zero
      a1(1,5)=zero
      a1(2,1)=-(fkn1*glame)/x
      a1(2,2)=zero
      a1(2,3)=zero
      a1(2,4)=f1/x
      a1(2,5)=zero
      a1(3,1)=f4*gmu*(glame+gmu)*fkn1*fkn1
      a1(3,1)=a1(3,1)/x - wfreq*wfreq*grho
      a1(3,2)=zero
      a1(3,3)=zero
      a1(3,4)=-a1(2,1)
      a1(3,5)=zero
      a1(4,1)=zero
      a1(4,2)=-(wfreq*wfreq*grho)
      a1(4,3)=-fkn1
      a1(4,4)=zero
      a1(4,5)=zero
      a1(5,1)=zero
      a1(5,2)=zero
      a1(5,3)=zero
      a1(5,4)=zero
      a1(5,5)=f1/depth
      return
      end
!# |------------------------------------------------------------|
!  |                                         date:  18 feb 94   |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | pltvel:  matlab plot of group or phase velocities          |
!  |                                                            |
!  |------------------------------------------------------------|
      subroutine pltvel(ksw,icorg,scilb1,scilb2,nmode,nfreq, &
      mapmat,matlb1,matlb2)
!
      parameter (nfmax=1000,maxmod=10,ngrp=100)
      complex*8  gf(nfmax,ngrp)
      real*8 plt(nfmax,maxmod),pltu(nfmax,maxmod)
      character*3 mode(maxmod)
      character*40 scilb1,scilb2
      character*40 matlb1,matlb2
!
      common /array1/ gf,plt,pltu
!
!  no longer supporting maple format
!  if mapmat=1 then scilab, matmat=0 matlab or octave
      write(mode(2),'(" c1")')
      write(mode(3),'(" c2")')
      write(mode(4),'(" c3")')
      write(mode(5),'(" c4")')
      write(mode(6),'(" c5")')
      write(mode(7),'(" c6")')
      write(mode(8),'(" c7")')
      write(mode(9),'(" c8")')
      write(mode(10),'(" c9")')
!
      pi=3.14159265358979311599796346854
      pi2=2.00000000*pi
      zero=0.00d+00
!
      if(mapmat.eq.1) then
!...open scilab plot files
      if(icorg.eq.0) then
      open(file=scilb1,unit=12,access='sequential')
      write(12,*) 'clear'
      write(12,*) '// created by waves.f <pm@cgiss.boisestate.edu>'
      else
      open(file=scilb2,unit=12,access='sequential')
      write(12,*) 'clear'
      write(12,*) '// created by waves.f <pm@cgiss.boisestate.edu>'
      endif
      endif
!
      if(mapmat.ne.1) then
!...open octave or matlab
      if(icorg.eq.0) then
      open(file=matlb1,unit=12,access='sequential')
      write(12,*) 'clear'
      write(12,*) '%  created by waves.f <pm@cgiss.boisestate.edu>'
      else
      open(file=matlb2,unit=12,access='sequential')
      write(12,*) 'clear'
      write(12,*) '%  created by waves.f <pm@cgiss.boisestate.edu>'
      endif
      endif
!
!...output to maple file for plot
      do 484 kf=2,10
      iswp=0
      do 485 jf=1,nfreq
      if(plt(jf,kf).ne.zero.and.iswp.ge.0) iswp=1
!/////////////
      if(iswp.gt.0) then
!
!-------------
      if(ksw.eq.0) then
        if(icorg.eq.0) then
        write(12,900) mode(kf),plt(jf,1),plt(jf,kf)
        else
        write(12,900) mode(kf),pltu(jf,1),pltu(jf,kf)
        endif
      else
!     wavenumber 1/m units
      fk1=real(plt(jf,1)/plt(jf,kf),4)
      fku=real(plt(jf,1)/pltu(jf,kf),4)
        if(icorg.eq.0) then
        write(12,900) mode(kf),plt(jf,1),fk1
        else
        write(12,900) mode(kf),pltu(jf,1),fku
        endif
      endif
!-------------
!
      iswp=-1
      go to 485
      endif
!/////////////
      if(jf.lt.nfreq.and.plt(jf,kf).ne.zero) then
!------------
      if(ksw.eq.0) then
        if(icorg.eq.0) then
        write(12,901) plt(jf,1),plt(jf,kf)
        else
        write(12,901) pltu(jf,1),pltu(jf,kf)
        endif
      else
!    wavenumber 1/m units
      fk1=real(plt(jf,1)/plt(jf,kf),4)
      fku=real(plt(jf,1)/plt(jf,kf),4)
        if(icorg.eq.0) then
        write(12,901) plt(jf,1),fk1
        else
        write(12,901) pltu(jf,1),fku
        endif
      endif
!-----------
!
      endif
!/////////////
!
      if(jf.eq.nfreq.and.plt(jf,kf).ne.zero) then
!----------
      if(ksw.eq.0) then
        if(icorg.eq.0) then
        write(12,902) plt(jf,1),plt(jf,kf)
        else
        write(12,902) pltu(jf,1),pltu(jf,kf)
        endif
      else
!    wavenumber 1/m units
      fk1=real(plt(jf,1)/plt(jf,kf),4)
      fku=real(plt(jf,1)/plt(jf,kf),4)
        if(icorg.eq.0) then
        write(12,902) plt(jf,1),fk1
        else
        write(12,902) pltu(jf,1),fku
        endif
      endif
!-----------
!
      endif
!////////////
  485 continue
  484 continue
!
!c900 format(a,'=[',f10.4,', ',f10.3,',')
  900 format(a,'=[',f10.4,', ',f10.3,';...')
!c901 format(f10.4,', ',f10.3,',')
  901 format(f10.4,', ',f10.3,';...')
  902 format(f10.4,', ',f10.3,'];')
!
!...select matlab or scilab output format
! mapmat not equal 1, matlab
      if(mapmat.ne.1) then
!----------------------------------*
      write(12,*) 'figure(1)'
      write(12,*) 'clf'
      write(12,*) 'set(gca(),''fontsize'',12) '
      if(nmode.eq.1) then
      write(12,903)
      endif
      if(nmode.eq.2) then
      write(12,904)
      endif
      if(nmode.eq.3) then
      write(12,905)
      endif
      if(nmode.eq.4) then
      write(12,906)
      endif
      if(nmode.eq.5) then
      write(12,907)
      endif
      if(nmode.eq.6) then
      write(12,908)
      endif
      if(nmode.eq.7) then
      write(12,909)
      endif
      if(nmode.eq.8) then
      write(12,910)
      endif
      if(nmode.eq.9) then
      write(12,911)
      endif
!
  903 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on')
  904 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'  )
  905 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'/ &
      ' plot(c3(:,1),c3(:,2),''-r'')'  )
  906 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'/ &
      ' plot(c3(:,1),c3(:,2),''-r'')'/ &
      ' plot(c4(:,1),c4(:,2),''-r'')'  )
  907 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'/ &
      ' plot(c3(:,1),c3(:,2),''-r'')'/ &
      ' plot(c4(:,1),c4(:,2),''-r'')'/ &
      ' plot(c5(:,1),c5(:,2),''-r'')'  )
  908 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'/ &
      ' plot(c3(:,1),c3(:,2),''-r'')'/ &
      ' plot(c4(:,1),c4(:,2),''-r'')'/ &
      ' plot(c5(:,1),c5(:,2),''-r'')'/ &
      ' plot(c6(:,1),c6(:,2),''-r'')'  )
  909 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'/ &
      ' plot(c3(:,1),c3(:,2),''-r'')'/ &
      ' plot(c4(:,1),c4(:,2),''-r'')'/ &
      ' plot(c5(:,1),c5(:,2),''-r'')'/ &
      ' plot(c6(:,1),c6(:,2),''-r'')'/ &
      ' plot(c7(:,1),c7(:,2),''-r'')'  )
  910 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'/ &
      ' plot(c3(:,1),c3(:,2),''-r'')'/ &
      ' plot(c4(:,1),c4(:,2),''-r'')'/ &
      ' plot(c5(:,1),c5(:,2),''-r'')'/ &
      ' plot(c6(:,1),c6(:,2),''-r'')'/ &
      ' plot(c7(:,1),c7(:,2),''-r'')'/ &
      ' plot(c8(:,1),c8(:,2),''-r'')'  )
  911 format(' plot(c1(:,1),c1(:,2),''-b'',''linewidth'',2)'/ &
      'grid on'/'hold on'/ &
      ' plot(c2(:,1),c2(:,2),''-r'')'/ &
      ' plot(c3(:,1),c3(:,2),''-r'')'/ &
      ' plot(c4(:,1),c4(:,2),''-r'')'/ &
      ' plot(c5(:,1),c5(:,2),''-r'')'/ &
      ' plot(c6(:,1),c6(:,2),''-r'')'/ &
      ' plot(c7(:,1),c7(:,2),''-r'')'/ &
      ' plot(c8(:,1),c8(:,2),''-r'')'/ &
      ' plot(c9(:,1),c9(:,2),''-r'')'  )
      write(12,*) 'grid on'
      if(icorg.eq.0) then
      if (ksw.eq.0) then
      write(12,912)
  912 format('title(''Dispersion: Phase Velocity'')')
      else
      write(12,*) 'title(''Dispersion: Phase Wavenumber'')'
      endif
      write(12,*) 'xlabel(''Frequency Hz'')'
      if (ksw.eq.0) then
      write(12,*) 'ylabel(''Phase Velocity m/s'')'
      else
      write(12,*) 'ylabel(''Phase Wave Number rad/m '')'
      endif
      write(12,*) '%print(''C.fig'',''-color'',''-dfig'')'
      else
      if (ksw.eq.0) then
      write(12,913)
  913 format('title(''Dispersion: Group Velocity'')')
      else
      write(12,*) 'title(''Dispersion: Group Wavenumber'')'
      endif
      write(12,*) 'xlabel(''Frequency Hz'')'
      if (ksw.eq.0) then
      write(12,*) 'ylabel(''Group Velocity m/s'')'
      else
      write(12,*) 'ylabel(''Group Wavenumber 1/m'')'
      endif
      write(12,*) '%print(''U.fig'',''-color'',''-dfig'')'
      endif
!---------------------------*
      else
      if(nmode.eq.1) write(12,914)
      if(nmode.eq.2) write(12,915)
      if(nmode.eq.3) write(12,916)
      if(nmode.eq.4) write(12,917)
      if(nmode.eq.5) write(12,918)
      if(nmode.eq.6) write(12,919)
      if(nmode.eq.7) write(12,920)
      if(nmode.eq.8) write(12,921)
      if(nmode.eq.9) write(12,922)
      write(12,*)' xgrid(3) '
!
  914 format(' plot2d(c1(:,1),c1(:,2),2)')
  915 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'  )
  916 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'/ &
      ' plot2d(c3(:,1),c3(:,2),5)'  )
  917 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'/ &
      ' plot2d(c3(:,1),c3(:,2),5)'/ &
      ' plot2d(c4(:,1),c4(:,2),5)'  )
  918 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'/ &
      ' plot2d(c3(:,1),c3(:,2),5)'/ &
      ' plot2d(c4(:,1),c4(:,2),5)'/ &
      ' plot2d(c5(:,1),c5(:,2),5)'  )
  919 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'/ &
      ' plot2d(c3(:,1),c3(:,2),5)'/ &
      ' plot2d(c4(:,1),c4(:,2),5)'/ &
      ' plot2d(c5(:,1),c5(:,2),5)'/ &
      ' plot2d(c6(:,1),c6(:,2),5)'  )
  920 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'/ &
      ' plot2d(c3(:,1),c3(:,2),5)'/ &
      ' plot2d(c4(:,1),c4(:,2),5)'/ &
      ' plot2d(c5(:,1),c5(:,2),5)'/ &
      ' plot2d(c6(:,1),c6(:,2),5)'/ &
      ' plot2d(c7(:,1),c7(:,2),5)'  )
  921 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'/ &
      ' plot2d(c3(:,1),c3(:,2),5)'/ &
      ' plot2d(c4(:,1),c4(:,2),5)'/ &
      ' plot2d(c5(:,1),c5(:,2),5)'/ &
      ' plot2d(c6(:,1),c6(:,2),5)'/ &
      ' plot2d(c7(:,1),c7(:,2),5)'/ &
      ' plot2d(c8(:,1),c8(:,2),5)'  )
  922 format(' plot2d(c1(:,1),c1(:,2),2)'/ &
      ' plot2d(c2(:,1),c2(:,2),5)'/ &
      ' plot2d(c3(:,1),c3(:,2),5)'/ &
      ' plot2d(c4(:,1),c4(:,2),5)'/ &
      ' plot2d(c5(:,1),c5(:,2),5)'/ &
      ' plot2d(c6(:,1),c6(:,2),5)'/ &
      ' plot2d(c7(:,1),c7(:,2),5)'/ &
      ' plot2d(c8(:,1),c8(:,2),5)'/ &
      ' plot2d(c9(:,1),c9(:,2),5)'  )
      if(icorg.eq.1) then
      if (ksw.eq.0) then
      write(12,923)
  923 format('xtitle(''Dispersion: Group Velocity'',...'/ &
      ' ''Frequency  Hz'',...'/ &
      ' ''Group Velocity M/S'')')
      else
      write(12,925)
  925 format('xtitle(''Dispersion: Group Wavenumber'',...'/ &
      ' ''Frequency  Hz'',...'/ &
      ' ''Group Wavenumber 1/m'')')
      endif
      else
      if (ksw.eq.0) then
      write(12,924)
  924 format('xtitle(''Dispersion: Phase Velocity'',...'/ &
      ' ''Frequency  Hz'',...'/ &
      ' ''Phase Velocity M/S'')')
      else
      write(12,926)
  926 format('xtitle(''Dispersion: Phase Wavenumber'',...'/ &
      ' ''Frequency  Hz'',...'/ &
      ' ''Phase Wavenumber  1/m'')')
      endif
      endif
      endif
!-----------------------------------*
!
      close(12)
      return
      end
!# |------------------------------------------------------------|
!  |                                         date:  18 feb 94   |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | matmlt:  matrix multiply                                   |
!  |                                                            |
!  |------------------------------------------------------------|
      subroutine matmlt(a,x,y,n,m)
      implicit real*8 (a-h,o-z)
      real*8 a(n,m),x(m),y(n)
      do 5 j=1,n
      y(j)=0.0000000000000
      do 6 k=1,m
    6 y(j)=y(j)+a(j,k)*x(k)
    5 continue
      return
      end
!# |------------------------------------------------------------|
!  |                                        date:  18 feb 94    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | msv: calculates motion stress vector                       |
!  | Modified 3 Oct 1999 added irvsel=select output motion      |
!  |            irvsel=0 Vertical (default)                     |
!  |            irvsel=1 Radial, horizontal inline              |
!  |                                                            |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine msv(nfreq,iswdrv, &
      lame,mu,rho,zi,nlay,rv1,rv2,iout, &
      deltz,rtotal,ndim,i5,modes,p,f, &
      r1ath,r2ath,r1atz,r2atz,dr1ath,dr2ath,nrec, &
      frange,cph,sph,rz,iprint,tm,nsmax,sz,irz, &
      xr,xi,amp,phz,x,y,tzero,fsamin,idft, &
      rx,ry,sx,sy,iyear,iday,line,igeoor,flgrmx,fqlgrx,irvsel,mapmat)
      parameter (nfmax=1000,maxmod=10,ngrp=100)
      implicit real*8 (a-h,o-z)
      real*8 r1atz(nrec),r2atz(nrec)
      real*8 frange(nrec),cph(nrec),sph(nrec),rz(nrec)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 tv1(5),tv2(5)
      real*8 rtotal(i5)
      real*8 s1(5),s2(5)
      real*8 plt(nfmax,maxmod),pltu(nfmax,maxmod)
!
      real*8 rx(nrec),ry(nrec)
      character*4 line,igeoor
      integer*2 iyear,iday
!
      complex*8  gf(nfmax,ngrp)
      complex*8  dczero
      complex*8  fm0
      real*8 tm(3,3)
      integer*2 irz(nrec)
!
      REAL*4 xr(nsmax),xi(nsmax),amp(nsmax),phz(nsmax)
      REAL*4 samin,tzero
      complex*8  x(nsmax),y(nsmax)
      REAL*4 p(nsmax),f(nsmax)
!
      integer*2 modes(maxmod-1)
!
      common /array1/ gf,plt,pltu
!
      dczero=complex(0.0,0.0)
      pi=3.14159265358979311599796346854
      pi2=2.00e+00*pi
      pi4=pi/4.00e+00
      f2=2.00e+00
      f8=8.00e+00
      zero=0.00e+00
      flgrmx=0.0
!
      jf=1
!
!...fill out symetry of spectrum, real trace
      fmin=plt(1,1)
      fmax=plt(nfreq,1)
      deltf=plt(2,1)-plt(1,1)
!...double for symetry
      jf1=int(fmin/deltf +0.5) + 1
!
      if (fsamin.ne.zero) then
      nnew=int(1.0000/(fsamin*deltf)+0.5)
      nstop=nnew/2
      else
      nstop=nfreq + jf1 -1
      endif
!
!/    if(idft.eq.1) then
      dummy=idft
!/    npts=2*nstop
!/    else
!-----radix 2----------
      npts=2*nstop
      call nrad2(npts,npr2)
      npts=npr2
      if(npts.gt.nsmax) then
      write(*,'("  abort--trace length exeeds memory")')
      endif
!----------------------
!/    endif
!
      write(10,'(" npts=",i5)') npts
      write(*,'(" npts=",i5)') npts
      samin=real(1.0/(npts*deltf),4)
      write(10,'("  deltf=",e10.4)') deltf
      write(10,'("  fmin,fmax=",f10.4,2x,f10.4)') fmin,fmax
      write(10,'("  samin=",f10.4)') samin
!
      call moment(npts,y,fmin,fmax,deltf,f,p, &
      xr,mapmat)
!
      if(iprint.eq.-1) then
      write(10,471) char(12)
  471 format(a,8x,"freq.",4x,"phase vel.", &
      4x,"group vel.",5x,"lagrangian test"/) 
      lcnt=2
      endif
!
!.......................start frequency loop..........
      jfi=jf1-1
!
      do 500 jf=1,nfreq
      wfreq=pi2*plt(jf,1)
!
!...store frequency in group velocity plot array
      pltu(jf,1)=plt(jf,1)
!
      rwfreq=1.00e+00/wfreq
!...moment m0 for step function wrt time
!     fm0=-cmplx(0.0,rwfreq)
!...moment m0 for minimum phase earth displacement
      jfi=jfi+1
      fm0=y(jfi)
!
!...number of modes
      nmode=0
      do 501 jm=2,maxmod
  501 if(plt(jf,jm).ne.zero) nmode=jm-1
!
!...zero sumation registers over recievers
      do 502 jrec=1,nrec
  502 gf(jf,jrec)=dczero

!
      nmodep=nmode+1
!........................start mode loop .............
      do 505 imode=2,nmodep
!
!...skip modes not desired
      if(modes(imode-1).eq.0) go to 505
!
      fkn1=(plt(jf,1)*pi2)/plt(jf,imode)
!
!...assign initial motion/stress vectors
      zstart=.00e+00
      zero=0.00e+00
      aa1=1.00e+00
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
      phasev=wfreq/fkn1
!
!...base new model
!...using trial k and initial p/s wave balance, aa1
!
      z1=zi(nlay)
      call halfsp(grho,gmu,glame,wfreq,z1,phasev, &
      r1,r2,r3,r4,aa1,rv1,rv2)
!
!      write(10,'(/" @zr1: ",5(e10.4,1x))') (rv1(jj),jj=1,5)
!      write(10,'(" @zr2: ",5(e10.4,1x))') (rv2(jj),jj=1,5)
!...save eigenvectors at base of model
      do 40 j=1,5
      s1(j)=rv1(j)
   40 s2(j)=rv2(j)
!
      zstart=zi(nlay)
      rv1(5)=zstart
      rv2(5)=zstart
!
!...integrate upwards
      fsign=-1.0
      iprnt=0
      iskip=1
      call intgr4(fkn1,zstart,deltz,nlay,wfreq, &
      rtotal,fsign,rho,mu,lame,zi,rv1,rv2,iskip, &
      ndim,aa1,i5,iprnt)
!      write(10,'(/" @0 r1: ",5(e10.4,1x))') (rv1(jj),jj=1,5)
!      write(10,'(" @0 r2: ",5(e10.4,1x))') (rv2(jj),jj=1,5)
!
!...motion stress vector at surface in rv1,rv2
!...determine total eigenvector and new p/s wave balance, aa1
!
      aa1=-rv2(3)/rv1(3)
      aa2=-rv2(4)/rv1(4)
      if(iprint.ge.2) write(10,'(70(1h=))')
      if(iprint.ge.2) write(10,'("   aa1",f14.7)') aa1
      if(iprint.ge.2) write(10,'("   aa2",f14.7)') aa2
!
!...initialized total motion stress vector at surface
!...for starting intgr3 integration
!
      do 30 j=1,4
   30 rtotal(j)=aa1*rv1(j) + rv2(j)
      rtotal(5)=rv1(5)
!
!     write(*,'(" total vector at surface:",i5/
!    +5(1x,e10.4))') jf,(rtotal(jj),jj=1,5)
      if(iprint.ge.2) write(10,32) (rtotal(jj),jj=1,5)
  32  format(6x," total vector at surface:"/ &
      6x,5(1x,e10.4))
!...............................................
!....analytic eigenfunctions at base of model using
!....new p/s wave balance
!
      z1=zi(nlay)
      call halfsp(grho,gmu,glame,wfreq,z1,phasev, &
      rz1,rz2,rz3,rz4,aa1,tv1,tv2)
!
      am1=aa1*s1(1) + s2(1)
      am2=aa1*s1(2) + s2(2)
      am3=aa1*s1(3) + s2(3)
      am4=aa1*s1(4) + s2(4)
!
      rm1=am1/rz1
      rm2=am2/rz2
      rm3=am3/rz3
      rm4=am4/rz4
      if(iprint.ge.2) then
      write(10,'(/"  rm1,rm2,rm3,rm4=",4(1x,e10.4))') &
      rm1,rm2,rm3,rm4
      endif
!
!...integrate upwards from zi(nlay) with wavenumber fkn1
!...note: upward integration for stability
      fsign=-1.0
      fq=plt(jf,1)
!
      zend=zi(nlay)
!
!...initial values for numerical integration
      call halfsp(grho,gmu,glame,wfreq,zend,phasev,r1,r2,r3,r4, &
      aa1,rv1,rv2)
      rtotal(1)=r1
      rtotal(2)=r2
      rtotal(3)=r3
      rtotal(4)=r4
      rtotal(5)=zend
      if(iprint.ge.2) then
      write(10,34) (rtotal(jj),jj=1,5)
   34 format(6x," ====>total vector at zend:"/" ==>", &
      6x,5(1x,e10.4))   
      endif
!
      call intgr3(fkn1,deltz,nlay,wfreq,iswdrv, &
      rtotal,fsign,rho,mu,lame,zi,rv1,rv2, &
      ndim,aa1,i5,sum1,sum2,sum3,sum4,u,rz, &
      r1ath,r2ath,r1atz,r2atz,dr1ath,dr2ath,nrec,irz,sz)
!
!...store group velocity in plot array
      pltu(jf,imode)=u
!
!...print option and lagrangian test========================|
!
      call lgrtst(iprint,jf,lcnt,wfreq,sum1,sum2,sum3,sum4, &
      fkn1,fq,phasev,u,rz,frange,cph,sph,r1atz,r2atz, &
      sz,r1ath,r2ath,dr1ath,dr2ath,nrec,flgrmx,fqlgrx)
!
!.....end print option and lagrangian test==================|
!
!
!
!...generate displacement spectrum =========================|
!
      call dspspc(jf,frange,phasev,deltf,u,cph,sph,tm,fkn1, &
      r1ath,r2ath,dr1ath,dr2ath,r1atz,r2atz,irvsel,sum1,zero,fm0,nrec, &
      f2,f8,pi,pi4)
!
!...end displacement spectrum===============================|.
!
  505 continue
!....end mode loop.>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!
!...small print block if iprint=1 ........
      if(iprint.ge.1) then
      do 507 jrec=1,nrec
      ampr=real(gf(jf,jrec))
      ampi=aimag(gf(jf,jrec))
      ampp=sqrt(ampr*ampr+ampi*ampi)
      if(ampr.eq.0.0) then
      ampr=1.e-30
      endif
      phzp=atan(ampi/ampr)
      phzp=phzp*(180.0/pi)
      write(10,'(/"  ....receiver=",i5," depth= ",f10.4)') &
      jrec,rz(jrec)
      write(10,902) fq,gf(jf,jrec),ampp,phzp
  507 continue
  902 format(5x,'disp. spectrum: freq,real,imag=',f10.4, &
      2x,2(e14.7,2x)/5x,'amp, phase= ',e14.7,2x,f10.4)
      endif
!...end small print block................
!
  500 continue
!....end frequency loop >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!
!
!....compute time domain traces and output....
      call ptrace(nrec,npts,jf1,x,nfreq,iout,samin,tzero, &
      xr,xi,amp,phz,frange,rx,ry,rz,sx,sy,sz,iyear,iday, &
      line,igeoor,nsmax,irvsel)
!
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | lgrtst: computes lagrangian test                           |
!  |         includes printer options                           |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine lgrtst(iprint,jf,lcnt,wfreq,sum1,sum2,sum3,sum4, &
      fkn1,fq,phasev,u,rz,frange,cph,sph,r1atz,r2atz, &
      sz,r1ath,r2ath,dr1ath,dr2ath,nrec,flgrmx,fqlgrx)
      parameter (nfmax=1000,maxmod=10,ngrp=100)
!
      implicit real*8 (a-h,o-z)
      real*8 r1atz(nrec),r2atz(nrec)
      real*8 frange(nrec),cph(nrec),sph(nrec),rz(nrec)
      real*8 plt(nfmax,maxmod),pltu(nfmax,maxmod)
      complex*8  gf(nfmax,ngrp)
!
      common /array1/ gf,plt,pltu
!
!
      test1=wfreq*wfreq*sum1 - fkn1*sum3
      test2=fkn1*fkn1*sum2 + sum4
      test=test1 - test2
      test=test/abs(test1)
!
      write(*,800) jf,fq,phasev,u,test
  800 format(2x,i5,f10.4,'hz',2x,'c=',f9.3,2x,'u=',f9.3, &
      2x,'test=',f11.8)
!
      if(iprint.eq.-1) then
!
      if(lcnt.gt.55) then
      write(10,802) char(12) 
  802 format(a,8x,"freq.",4x,"phase vel.", &
      4x,"group vel.",5x,"lagrangian test"/)
      lcnt=2
      endif
!
      write(10,'(5x,f10.4,5x,f9.3,5x,f9.3,5x,f11.8)') &
      fq,phasev,u,test
      lcnt=lcnt+1
      endif
!
!
!...flag questionable modes with large lagrangians
      if(abs(test).gt.flgrmx) then
      if(abs(test).gt.flgrmx) flgrmx=abs(test)
      fqlgrx=plt(jf,1)
!cc      write(10,906) phasev,test,plt(jf,1)
!cc      write(*,906) phasev,test,plt(jf,1)
!  906 format(5x,'warning**** lagrangian is large for', &
!      ' phase velocity=',f10.5/5x,'normalized lagrangian=', &
!      f10.5,'  freq=',f10.4)
      endif
!
      if(iprint.ge.2) then
!      write(10,'("  freq and phase vel=",2(f10.4,2x))')
!     +fq,phasev
!      write(10,'("  sum1= ",e14.7)') sum1
!      write(10,'("  sum2= ",e14.7)') sum2
!      write(10,'("  sum3= ",e14.7)') sum3
!      write(10,'("  sum4= ",e14.7)') sum4
!      write(10,'("  test1= ",e14.7)') test1
!      write(10,'("  test2= ",e14.7)') test2
      write(10,'(" .... test= ",e14.7)') test
!
!...list terms of particle displacement for this mode
      do 495 jr=1,nrec
      write(10,900) jr,rz(jr),frange(jr),cph(jr),sph(jr), &
      r1atz(jr),r2atz(jr),fkn1,fq,sum1,u,phasev
  495 continue
      write(10,901) sz,r1ath,r2ath,dr1ath,dr2ath
  900 format(/5x,'rec= ',i5,' depth= ',f10.4/5x, &
      'frange= ',f10.3,' cos(az)= ',e10.4, &
      ' sin(az)= ',e10.4/5x,'eigenfunction: r1(z)= ',e10.4, &
      ' r2(z)= ',e10.4,/5x,'wavenumber= ',e10.4, &
      ' freq= ',f10.4,' sum1= ',e10.4,/5x,'group vel= ',f10.4, &
      ' phase vel= ',f10.4)
  901 format(/5x,'shot: depth= ',f10.4,' r1= ',e10.4, &
      ' r2= ',e10.4,/5x,' dr1dz= ',e10.4,' dr2dz= ',e10.4)
      write(10,'(1x,70(1h-))')
      endif
!
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | dspspc: computes displacement spectrum                     |
!  |         of vertical component rayleigh waves               |
!  |modified to add horizontal displacement option 3 Oct 1999 pm|
!  |  irvsel=1 then horizontal, else vertical (default) computed|
!  |                                                            |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine dspspc(jf,frange,phasev,deltf,u,cph,sph,tm,fkn1, &
      r1ath,r2ath,dr1ath,dr2ath,r1atz,r2atz,irvsel,sum1,zero,fm0,nrec, &
      f2,f8,pi,pi4)
      parameter (nfmax=1000,maxmod=10,ngrp=100)
      implicit real*8 (a-h,o-z)
      real*8 r2atz(nrec)
!Mod. 3 Oct 1999 add r1atz array below
      real*8 r1atz(nrec)
      real*8 frange(nrec),cph(nrec),sph(nrec)
      real*8 plt(nfmax,maxmod),pltu(nfmax,maxmod)
!
      complex*8  gf(nfmax,ngrp)
      complex*8  fm0,ctmp
      real*8 tm(3,3)
!
      common /array1/ gf,plt,pltu
!
!
!
!...generate displacement spectrum =========================|
!...start receiver loop.......
      deltf=plt(2,1)-plt(1,1)
      do 510 jrec=1,nrec
!
!...only include modes with unaliased phase
!...delta phase=(2*pi*frange/phasev)*delta freq  <  pi
!...(ie. event must be in the time window 1/2*deltf)
!
!     testc=(frange(jrec)/phasev)*deltf
!...use group velocity
      testc=(frange(jrec)/u)*deltf
!
      if(testc.lt.0.5e+00) then
      cp=cph(jrec)
      sp=sph(jrec)
      tmp1=tm(1,1)*cp*cp+(tm(1,2)+tm(2,1))*sp*cp+tm(2,2)*sp*sp
      tmp2=tm(1,3)*cp+tm(2,3)*sp
      tmp3=tm(3,1)*cp+tm(3,2)*sp
      tmpr=fkn1*r1ath*tmp1 + dr2ath*tm(3,3)
      tmpi=dr1ath*tmp2 - fkn1*r2ath*tmp3
!...modified to add horizontal displacement option 3 Oct 1999 pm.
!     irvsel=1 then horizontal, else vertical (default) computed
      if(irvsel.eq.1) then
      tmp4=(r1atz(jrec)/(f8*phasev*u*sum1))
      else
      tmp4=(r2atz(jrec)/(f8*phasev*u*sum1))
      endif
      if(frange(jrec).eq.zero) frange(jrec)=1.e-30
      tmp4=tmp4*sqrt(f2/(pi*fkn1*frange(jrec)))
      tmp4=tmp4*1.0D+06
!  (Aki and Richards Eq. 7.149, or 7.150 if irvsel=1)
!...phase angle negated for my sign convention
!...modified to add horizontal displacement option 3 Oct 1999 pm.
!     irvsel=1 then horizontal, else vertical (default) computed
      if(irvsel.eq.1) then
      angle=-(fkn1*frange(jrec)-pi4)
      else
      angle=-(fkn1*frange(jrec)+pi4)
      endif
      tmpr=tmpr*tmp4
! (Aki and Richards Eq. 7.149, or 7.150 if irvsel=1)
!...phase angle negated for my sign convention
      tmpi=-tmpi*tmp4
      ctmp=complex(tmpr,tmpi)*fm0*complex(dcos(angle),dsin(angle))
!
!...sum at this frequency over modes
      gf(jf,jrec)=gf(jf,jrec)+ctmp
!      write(10,'("  frange=",e10.4)') frange(jrec)
!      write(10,'("  r2atz=",e10.4)') r2atz(jrec)
!      write(10,'("  angle=",e10.4)') angle
      else
      write(6,123) plt(jf,1),frange(jrec)
  123 format("WARNING from dspspc: frq=",f10.4," offset=",f10.4)
      write(6,*) 'increase trace length or decrease offset '
      endif
!
  510 continue
!....end receiver loop...............
!
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | ptrace: computes time domain traces and                    |
!  |         outputs in segy or line printer format             |
!  |  Modified 3 Oct 1999 added irvsel  0=vertical 1=radial out |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine ptrace(nrec,npts,jf1,x,nfreq,iout,samin,tzero, &
      xr,xi,amp,phz,frange,rx,ry,rz,sx,sy,sz,iyear,iday, &
      line,igeoor,nsmax,irvsel)
      parameter (nfmax=1000,maxmod=10,ngrp=100)
!
      implicit real*8 (a-h,o-z)
      real*8 frange(nrec),rz(nrec)
      real*8 rx(nrec),ry(nrec)
      character*4 line,igeoor
      integer*2 iyear,iday
      character*5 recvr
!
      complex*8  gf(nfmax,ngrp)
      real*8 plt(nfmax,maxmod),pltu(nfmax,maxmod)
!
      REAL*4 xr(nsmax),xi(nsmax),amp(nsmax),phz(nsmax)
      REAL*4 samin,tzero,sign
      complex*8  x(nsmax)
      character*70 title
!
      common /array1/ gf,plt,pltu
!
!
!....start receiver loop--output traces
      do 430 jrec=1,nrec
      k1=npts +2 - jf1
      kk=1
      call czero(npts,x)
      nstop=nfreq + jf1 -1
!      write(10,'("  jf1,nstop=",i5,2x,i5)') jf1,nstop
      do 405 jf=jf1,nstop
      x(jf)=gf(kk,jrec)
      if(k1.le.npts) then
      x(k1)=conjg(gf(kk,jrec))
      endif
      k1=k1 - 1
      kk=kk + 1
  405 continue
      write(recvr,'(i5)') jrec
      write(title,'(" receiver",a," spectra")') recvr
      ilin=1
!
!c    do 427 jf=1,npts
!c    write(10,'(5x,i5,5x,2(e10.4,2x))') jf,x(jf)
!c427 continue
!
      if(iout.eq.0) then
      write(*,'("  start,flist")')
      call flist(npts,x,title,samin,tzero,xr,xi,amp,phz,ilin)
      write(*,'("  done,flist")')
      endif
      sign=+1.0
!
!/ note: idft deactivated --not an argument
!/    if(idft.eq.1) then
!/    write(*,'("  start dft")')
!/    call dft(npts,x,y,sign)
!/    do 410 jt=1,npts
!/    xr(jt)=real(y(jt))
!      xi(jt)=aimag(y(jt))
!/410 continue
!/    write(*,'("  done, dft")')
!/    else
!     write(*,'("  start fft")')
      call nlogn(npts,x,sign)
      do 411 jt=1,npts
      xr(jt)=real(x(jt))
  411 continue
!     write(*,'("  done, fft")')
!/    endif
!
      write(title,'(" receiver 1 real part")')
!
      if(iout.eq.0) then
      write(*,'("  start, tlist")')
      call tlist(npts,xr,title)
      write(*,'("  done, tlist")')
      else
      xsamin=samin
      call segy(npts,xr,xsamin,irvsel, &
      jrec,frange,rx,ry,rz,sx,sy,sz,iyear,iday,line,igeoor)
!DEBUG=========================================================
!     if (jrec.eq.25) then
!      do 111 jt=1,npts
!     write(11,'(i5,1x,E14.7,1x,f10.4,1x,i5,1x,i5,1x,E14.7/,6(1x,f10.4)/
!    +,1x,i5,1x,i5,1x,a,1x,a)')
!    +npts,xr(jt),xsamin,irvsel,jrec,frange(jrec),
!    +rx(jrec),ry(jrec),rz(jrec),sx,sy,sz,iyear,iday,line,igeoor
! 111  continue
!       endif
!DEBUG=========================================================
      endif
!
!      write(title,'(" receiver 1 imag part")')
!      call tlist(npts,xi,title)
  430 continue
!.......end receiver loop --output traces
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | intgr4: propagator matrix version of intgrt                |
!  |                                                            |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine intgr4(fkn1,zstart,deltz,nlay,wfreq, &
      rtotal,fsign,rho,mu,lame,zi,rv1,rv2,iskip, &
      ndim,aa1,i5,iprnt)
      implicit real*8 (a-h,o-z)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
      real*8 p(4,4)
      character*80 card
      character*2 rvct(4)
!
!
      write(rvct(1),'("r1")')
      write(rvct(2),'("r2")')
      write(rvct(3),'("r3")')
      write(rvct(4),'("r4")')
      zero=0.0000000000000000
      jzsw=1
      itrig=1
!
!...get elastic parameters
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
!
!...use trial k for analytic formula
      phasev=wfreq/fkn1
!
!...upper and lower wavenumbers
!
!...depth integration loop
      if(zstart.ge.zi(nlay)) then
      nz=int(zstart/deltz + 0.50)
      rv1(5)=zstart
      else
      nz=int(zi(nlay)/deltz + 0.50)
      rv1(5)=zi(nlay)
      endif
!
!     write(*,'("  zstart,deltz,nz",2e10.4,i5)') zstart,deltz,nz
      lcount=61
      if(nz.lt.1) go to 21
      h=fsign*deltz
      do 20 jz=1,nz
!
!...integrate above zi(nlay), analytic below
!   for stability (as opposed to accuracy)
      test=zi(nlay)
      if(rv1(5).le.test) then
!
!...synchronize integration on zi(nlay)
      if(itrig.eq.1) then
      zin=zi(nlay)
      call halfsp(grho,gmu,glame,wfreq,zin,phasev, &
      rz1,rz2,rz3,rz4,aa1,rv1,rv2)
      itrig=0
      endif
!
!...propagate set 1 and set 2
      z0=rv1(5)
      z1=rv1(5) + h
      call propgt(zi,mu,lame,rho,wfreq,phasev,nlay, &
      p,rv1,rv2,ndim,i5,z0,z1)
!
!      write(10,'("  rv1",5(e10.4,1x))') (rv1(jjj),jjj=1,5)
!      write(10,'("  rv2",5(e10.4,1x))') (rv2(jjj),jjj=1,5)
!
!...analytic eigenfunctions ................................
      else
      z1=rv1(5)+deltz*fsign
!
      if(mod(jz,iskip).eq.0 .or. jz.eq.1) then
      call halfsp(grho,gmu,glame,wfreq,z1,phasev,r1,r2,r3,r4, &
      aa1,rv1,rv2)
      endif
!
      rv1(5)=z1
      rv2(5)=z1
      endif
!
      if (iprnt.ne.zero) then
      if(mod(jz,iskip).eq.0 .or. jz.eq.1) then
      do 30 jjj=1,4
   30 rtotal(jjj)=aa1*rv1(jjj) + rv2(jjj)
      rtotal(5)=rv1(5)
!
      if (lcount.gt.55) then
      write(10,'(a)') char(12)
      write(10,'(6x,"eigenvector vs depth")')
      write(10,32)
   32 format(12x, &
      "u-horiz|u-vert|stress-horiz|stress-vert|depth")
      lcount=1
      endif
!
!...only print output if depth < zstart
      if(rtotal(5).le.zstart) then
      write(10,'(6x,5(1x,e10.4))') (rtotal(jjj),jjj=1,5)
      lcount=lcount+1
      endif
!
      endif
!
!
!
!
!. . . . . . . . . . . . . . . . . . . . . .
!...only print output if depth < zstart
      if(rtotal(5).le.zstart) then
!
!...maple plot of eigenvector
      if(jzsw.eq.1) then
      rtl1=fman(rtotal(1),itl1)
      rtl2=fman(rtotal(2),itl2)
      rtl3=fman(rtotal(3),itl3)
      rtl4=fman(rtotal(4),itl4)
      write(14,800) rvct(1),rtotal(5),rtl1,itl1
      write(15,800) rvct(2),rtotal(5),rtl2,itl2
      write(16,800) rvct(3),rtotal(5),rtl3,itl3
      write(17,800) rvct(4),rtotal(5),rtl4,itl4
      jzsw=0
      endif
!
      if(mod(jz,iskip).eq.0) then
      if(jz.gt.1.and.jz.lt.nz) then
!
      rtl1=fman(rtotal(1),itl1)
      rtl2=fman(rtotal(2),itl2)
      rtl3=fman(rtotal(3),itl3)
      rtl4=fman(rtotal(4),itl4)
      write(14,801) rtotal(5),rtl1,itl1
      write(15,801) rtotal(5),rtl2,itl2
      write(16,801) rtotal(5),rtl3,itl3
      write(17,801) rtotal(5),rtl4,itl4
!
      endif
      endif
!
      if(jz.eq.nz) then
!-----
      rtl1=fman(rtotal(1),itl1)
      rtl2=fman(rtotal(2),itl2)
      rtl3=fman(rtotal(3),itl3)
      rtl4=fman(rtotal(4),itl4)
      write(14,802) rtotal(5),rtl1,itl1
      write(15,802) rtotal(5),rtl2,itl2
      write(16,802) rtotal(5),rtl3,itl3
      write(17,802) rtotal(5),rtl4,itl4
  800 format(a,':=[',f20.7,',',1x,f10.7,'*10**(',i4,'),')
  801 format(f22.7,',',1x,f10.7,'*10**(',i4,'),')
  802 format(f22.7,',',1x,f10.7,'*10**(',i4,')];')
!-----
      endif
      endif
!
!
!...end of block if, depth<zstart
      endif
!
   20 continue
!===(end of depth loop, jz=1,nz)===
!
!
!...transfer scratch plots
      if(iprnt.ne.zero) then
      rewind(15)
      rewind(16)
      rewind(17)
      do 23 jz=1,nz
      read(15,'(a)',end=33) card
   23 write(14,'(a)') card
   33 continue
      do 24 jz=1,nz
      read(16,'(a)',end=44) card
   24 write(14,'(a)') card
   44 continue
      do 25 jz=1,nz
      read(17,'(a)',end=55) card
   25 write(14,'(a)') card
   55 continue
!
      write(14,803)
  803 format('a:=plot({r1,r2},style=line):'/ &
      'b:=plot({r3,r4},style=line):'/ &
      'a;'/'b;'/ &
      'interface(plotdevice=postscript,plotoutput=plotd);'/ &
      'a;'/ &
      'interface(plotdevice=postscript,plotoutput=plots);'/ &
      'b;')
!
      endif
   21 continue
!
      return
      end
!----------------------------------------------------------------
!...function for scientific notation in maple
      real*8 function fman(x,ilog)
      implicit real*8 (a-h,o-z)
      unity=1.0
      zero=0.0e+00
      if(abs(x).le.zero) then
      ilog=0
      fman=zero
      else
      sg=sign(unity,x)
!cc   flog=dlog10(abs(x))
      flog=log10(abs(x))
      ilog=int(flog)
      xman=flog-dfloat(ilog)
      fman=sg*10**xman
      endif
      end
!# |------------------------------------------------------------|
!  | propgt.f                               date:  18 feb 94    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  |                                                            |
!  | propgt: propagate the motion/stress vector                 |
!  |         from depth z0 to depth z1                          |
!  |                                                            |
!  |------------------------------------------------------------|
      subroutine propgt(zi,mu,lame,rho,wfreq,phasev,nlay, &
      p,rv1,rv2,ndim,i5,z0,z1)
      implicit real*8 (a-h,o-z)
      real*8 mu(ndim),lame(ndim),rho(ndim),zi(ndim)
      real*8 tmp(4),tmp2(4),rv1(i5),rv2(i5)
      real*8 p(4,4)
!
      fkn=wfreq/phasev
!
      depth=(z0 + z1)/2.00e+00
!
!...get elastic parameters
      call gparms(rho,mu,lame,zi,nlay,ndim, &
      depth,grho,gmu,glame)
!
      call pmatr2(p,gmu,glame,grho,wfreq,fkn, &
      z0,z1)
!
!      write(10,'(" p",4(e10.4,1x))') (p(1,j),j=1,4)
!      write(10,'(" p",4(e10.4,1x))') (p(2,j),j=1,4)
!      write(10,'(" p",4(e10.4,1x))') (p(3,j),j=1,4)
!      write(10,'(" p",4(e10.4,1x))') (p(4,j),j=1,4)
      do 5 j=1,4
    5 tmp(j)=rv1(j)
      call matmlt(p,tmp,tmp2,4,4)
      do 10 j=1,4
   10 rv1(j)=tmp2(j)
      rv1(5)=z1
      do 15 j=1,4
   15 tmp(j)=rv2(j)
      call matmlt(p,tmp,tmp2,4,4)
      do 20 j=1,4
   20 rv2(j)=tmp2(j)
      rv2(5)=z1
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | intgr3: pmatrix version of intgrt                          |
!  |                                                            |
!  |   integrates upward   total motion/stress vector           |
!  |   assume: initial motion stress vector in "rtotal"         |
!  |           at zi(nlay).                                     |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine intgr3(fkn1,deltz,nlay,wfreq,iswdrv, &
      rtotal,fsign,rho,mu,lame,zi,rv1,rv2, &
      ndim,aa1,i5,sum1,sum2,sum3,sum4,u,rz, &
      r1ath,r2ath,r1atz,r2atz,dr1ath,dr2ath,nrec,irz,sz)
      implicit real*8 (a-h,o-z)
      real*8 r1atz(nrec),r2atz(nrec)
      integer*2 irz(nrec)
      real*8 rz(nrec)
      real*8 rho(ndim),mu(ndim),lame(ndim),zi(ndim)
      real*8 rv1(i5),rv2(i5)
      real*8 rtotal(i5)
      real*8 p(4,4)
      real*8 a1(5,5), dtotal(5)
!
!...get elastic parameters
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
!
!...use trial k for analytic formula
      phasev=wfreq/fkn1
!
      zero=0.00d+00
      itrig=1
!
!...upper and lower wavenumbers
!
!...depth integration loop limits
      nz=int(zi(nlay)/deltz +0.50) + 1
!
!...make nz even
      ievn=(nz/2)*2
      if(nz.ne.ievn) then
      nz=nz-1
      deltz=zi(nlay)/float(nz)
      endif
!
      lcount=61
      h=fsign*deltz
!
!...find index for source eigenfunction to save
      isz=nz - int(sz/deltz)
      depth=rtotal(5)
!
!...assign initial states for energy integrals
      sum1=0.0
      sum2=0.0
      sum3=0.0
      sum4=0.0
      depth0=rtotal(5)
      rt10=rtotal(1)
      rt20=rtotal(2)
      call gparms(rho,mu,lame,zi,nlay,ndim, &
      depth0,grho0,gmu0,glame0)
!
!...clear eigenfunction values at source depth
      r1ath=0.0e+00
      r2ath=0.0e+00
      dr1ath=0.0e+00
      dr2ath=0.0e+00
!
!...clear eigenfunction values at receiver depth
      do 19 jsv=1,nrec
      r1atz(jsv)=0.0e+00
   19 r2atz(jsv)=0.0e+00
!
!...start eigenvector (integ. aut. equ.)
!
      do 20 jz=1,nz
!
!
!...integrate above zi(nlay), propagation matrix below
!   for stability (as opposed to accuracy)
      test=zi(nlay)
      if(rtotal(5).le.test) then
!
!...synchronize integration on zi(nlay)
      if(itrig.eq.1) then
      zin=zi(nlay)
      call halfsp(grho,gmu,glame,wfreq,zin,phasev, &
      rz1,rz2,rz3,rz4,aa1,rv1,rv2)
      rtotal(1)=rz1
      rtotal(2)=rz2
      rtotal(3)=rz3
      rtotal(4)=rz4
      rtotal(5)=zin
      itrig=0
!...sync shift
      depth0=zin
      rt10=rtotal(1)
      rt20=rtotal(2)
      call gparms(rho,mu,lame,zi,nlay,ndim, &
      depth0,grho0,gmu0,glame0)
      endif
!
!...propagate rtotal
      z0=rtotal(5)
      if (jz.ne.nz) then
      z1=rtotal(5) + h
      else
      z1=0.00D+00
      endif
      call propg2(zi,mu,lame,rho,wfreq,phasev,nlay, &
      p,rtotal,ndim,i5,z0,z1)
!
!...analytic eigenfunctions ................................
      else
      rtotal(5)=rtotal(5)+deltz*fsign
      z1=rtotal(5)
!
!...analytic displacements replace matrix estimates........
      call halfsp(grho,gmu,glame,wfreq,z1,phasev,r1,r2,r3,r4, &
      aa1,rv1,rv2)
      rtotal(1)=r1
      rtotal(2)=r2
      rtotal(3)=r3
      rtotal(4)=r4
      rtotal(5)=z1
!
      endif
!........energy integrals........................
!
!...if an odd jz
      if(mod(jz,2).eq.1) then
      depth1=rtotal(5)
      rt11=rtotal(1)
      rt21=rtotal(2)
      delta=depth1-depth0
      call gparms(rho,mu,lame,zi,nlay,ndim, &
      depth1,grho1,gmu1,glame1)
      endif
!
!
!
!...for displacement spectrum, save eigenfunctions at
!   source and receiver depths
!........receivers....
      do 50 jsv=1,nrec
      itest=nz-irz(jsv)
      if(itest.eq.jz) then
      r1atz(jsv)=rt10
      r2atz(jsv)=rt20
!     write(*,'("  receiver save",i5)') jz
      endif
   50 continue
!
!...if even jz
      if(mod(jz,2).eq.0) then
      depth2=rtotal(5)
      rt12=rtotal(1)
      rt22=rtotal(2)
      call gparms(rho,mu,lame,zi,nlay,ndim, &
      depth2,grho2,gmu2,glame2)
!
      x0=grho0*(rt10*rt10+rt20*rt20)
      x1=grho1*(rt11*rt11+rt21*rt21)
      x2=grho2*(rt12*rt12+rt22*rt22)
      sum1=sum1 + simpsn(x0,x1,x2,delta)
!
      gl2m0=glame0 + 2.0000000*gmu0
      gl2m1=glame1 + 2.0000000*gmu1
      gl2m2=glame2 + 2.0000000*gmu2
      x0=gl2m0*rt10*rt10 + gmu0*rt20*rt20
      x1=gl2m1*rt11*rt11 + gmu1*rt21*rt21
      x2=gl2m2*rt12*rt12 + gmu2*rt22*rt22
      sum2=sum2 + simpsn(x0,x1,x2,delta)
!
!------------------------------
!
!...compute derivatives of motion stress vector wrt. depth
!
      if(iswdrv.eq.0) then
      call dpoly2(rt10,rt11,rt12,delta,drt10,drt11,drt12)
      call dpoly2(rt20,rt21,rt22,delta,drt20,drt21,drt22)
      else
!
!...at depth0
      call amatrx(a1,grho0,gmu0,glame0,fkn1,wfreq, &
      depth0,i5)
      call matmlt(a1,rtotal,dtotal,5,5)
      drt10=dtotal(1)
      drt20=dtotal(2)
!
!...at depth1
      call amatrx(a1,grho1,gmu1,glame1,fkn1,wfreq, &
      depth1,i5)
      call matmlt(a1,rtotal,dtotal,5,5)
      drt11=dtotal(1)
      drt21=dtotal(2)
!
!...at depth2
      call amatrx(a1,grho2,gmu2,glame2,fkn1,wfreq, &
      depth2,i5)
      call matmlt(a1,rtotal,dtotal,5,5)
      drt12=dtotal(1)
      drt22=dtotal(2)
!------------------------------end of derivatives wrt depth
      endif
!
!........source.....
      if(isz.eq.jz-1) then
      r1ath=rt10
      r2ath=rt20
      dr1ath=drt10
      dr2ath=drt20
      endif
      if(isz.eq.jz) then
      r1ath=rt11
      r2ath=rt21
      dr1ath=drt11
      dr2ath=drt21
      endif
!
      x0=(glame0*rt10*drt20 - gmu0*rt20*drt10)
      x1=(glame1*rt11*drt21 - gmu1*rt21*drt11)
      x2=(glame2*rt12*drt22 - gmu2*rt22*drt12)
      sum3=sum3 + simpsn(x0,x1,x2,delta)
!
      x0=(gl2m0*drt20*drt20 + gmu0*drt10*drt10)
      x1=(gl2m1*drt21*drt21 + gmu1*drt11*drt11)
      x2=(gl2m2*drt22*drt22 + gmu2*drt12*drt12)
      sum4=sum4 + simpsn(x0,x1,x2,delta)
!
!...shift
      depth0=depth2
      rt10=rt12
      rt20=rt22
      gmu0=gmu2
      grho0=grho2
      glame0=glame2
      endif
!
!...............end of energy integrals above zi(nlay)....
   20 continue
!
!...flip signs on energy integrals to correct
!...for backwards integration (depth to surface)
      sum1=-sum1
      sum2=-sum2
      sum3=-sum3
      sum4=-sum4
!
!...eigenfunctions below zi(nlay) if any
      do 60 jsv=1,nrec
      itest=nz-irz(jsv)
      if(itest.lt.0.0) then
      z1=rz(jsv)
      call halfsp(grho,gmu,glame,wfreq,z1,phasev,r1,r2,r3,r4, &
      aa1,rv1,rv2)
      r1atz(jsv)=r1
      r2atz(jsv)=r2
      endif
   60 continue
      if(isz.lt.0.0) then
      z1=sz
      call halfsp(grho,gmu,glame,wfreq,z1,phasev,r1,r2,r3,r4, &
      aa1,rv1,rv2)
      r1ath=r1
      r2ath=r2
      t1=(phasev*phasev)/(gmu/grho)
      g1=sqrt(1.0-t1)*fkn1
      g2=sqrt(1.0-(phasev*phasev)/((glame+(2.0e+00)*gmu)/grho))*fkn1
      t2=t1-2.0e+00
      e1=exp(-g1*z1)
      e2=exp(-g2*z1)
      dr1ath=(-g2)*aa1*fkn1*e2 + (-g1)*fkn1*(t2/2.0e+00)*e1
      dr2ath=(-g2)*aa1*g2*e2 + (-g1)*((fkn1*fkn1)/(2.0e+00*g1))*t2*e1
      endif
!
!...add halfspace contribution
      z1=zi(nlay)
      call hlfint(grho,gmu,glame,z1,wfreq,phasev,aa1, &
      sumi1,sumi2,sumi3,sumi4)
!     write(10,'(" sum 1,2,3,4=",4(e10.4,1x))') sum1,sum2,sum3,sum4
!     write(10,'(" sumi1,2,3,4=",4(e10.4,1x))')
!    +sumi1,sumi2,sumi3,sumi4
!
!...and normalize integrals
      half=0.5e+00
      sum1=half*(sum1+sumi1)
      sum2=half*(sum2+sumi2)
      sum3=sum3+sumi3
      sum4=half*(sum4+sumi4)
!
!...group velocity (Eq 7.76 Aki and Richards)
      fkn2=2.00d+00*fkn1
      phasev=wfreq/fkn1
      u=(sum2 + (sum3/fkn2) )/(phasev*sum1)
!
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | dpoly2: computes derivatives for collocational polynomial  |
!  |         y0,y1,y2 are 3 consecutive samples                 |
!  |         sample interval = h                                |
!  |         dy0,dy1,dy2 are 3 consecutive derivatives          |
!  |                                                            |
!  |                                                            |
!  |------------------------------------------------------------|
      subroutine dpoly2(y0,y1,y2,h,dy0,dy1,dy2)
      implicit real*8 (a-h,o-z)
!
      h2=2.00D+00*h
      dy0=(4.00D+00*y1-3.00D+00*y0-y2)/h2
      dy1=(y2-y0)/h2
      dy2=(3.00D+00*y2+y0-4.00D+00*y1)/h2
      return
      end
!.................simpson's rule for integration.................
      real*8 function simpsn(y0,y1,y2,h)
      real*8 y0,y1,y2,h
      simpsn=(y0+4.00D+00*y1+y2)*(h/3.00D+00)
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | hlfint: computes energy integrals from z=zi(nlay) to       |
!  |         infinity based on analytic formulas                |
!  |                                                            |
!  |  grho=density of half space                                |
!  |  gmu= shear modulus for half space                         |
!  |  glame= lame's constant for half space                     |
!  |  z=zi(nlay)                                                |
!  |------------------------------------------------------------|
!
      subroutine hlfint(grho,gmu,glame,z,wfreq,phasev,aa1, &
      sumi1,sumi2,sumi3,sumi4)
      implicit real*8 (a-h,o-z)
      real*8 lm,mu
!
      lm=glame
      mu=gmu
      rho=grho
!
      fkn1=wfreq/phasev
      d=glame/gmu
      t1=(phasev*phasev)/(gmu/grho)
      g1=sqrt(1.0-t1)*fkn1
      g2=sqrt(1.0-t1/(d+2.0e+00))*fkn1
      t2=t1-2.0e+00
      e1=exp(-g1*z)
      e2=exp(-g2*z)
!
      a2=aa1*fkn1
      a1=fkn1*(t2/2.0e+00)
!
      b2=aa1*g2
      b1=((fkn1*fkn1)/(2.0e+00*g1))*t2
!
!...integral 1
      f2 = 2.00e+00
      t8 = exp(-g1*z)
      t11 = exp(-g2*z)
      t17 = a2**2
      t18 = b2**2
      t21 = t11**2
      t26 = a1**2
      t27 = b1**2
      t30 = t8**2
      t35 = (-f2)*rho*(a1*a2+b1*b2)/(g2+g1)*t8*t11 - &
      rho*(t17+t18)/g2*t21/(f2) - rho*(t26+t27)/g1*t30/(f2)
      sumi1=-t35
!
!...integral 2
      t3 = a1*a2
      t12 = exp(-g1*z)
      t15 = exp(-g2*z)
      t20 = a1**2
      t24 = b1**2
      t28 = t12**2
      t32 = a2**2
      t36 = b2**2
      t40 = t15**2
      t44 = -f2*t15*t12/(g2+g1)*(mu*b1*b2+t3*lm+f2*t3*mu) - &
      t28/g1*(t20*lm+f2*t20*mu+t24*mu)/f2 - &
      t40/g2*(t32*lm+f2*t32*mu+t36*mu)/f2
      sumi2=-t44
!
!...integral 3
      t2 = -lm+mu
      t5 = exp(-g1*z)
      t6 = t5**2
      t13 = exp(-g2*z)
      t14 = t13**2
      t40 = -t6*t2*b1*a1/f2-t14*t2*b2*a2/f2 - &
      t13*t5/(g2+g1)*(-lm*a1*b2*g2 - &
      lm*a2*b1*g1+mu*b1*a2*g2+mu*b2*a1*g1)
      sumi3=-t40
!
!...integral 4
      t1 = b2**2
      t5 = a2**2
      t10 = exp(-g2*z)
      t11 = t10**2
      t15 = b1**2
      t19 = a1**2
      t24 = exp(-g1*z)
      t25 = t24**2
      t45 = -t11*(lm*t1+f2*t1*mu+t5*mu)*g2/f2 - &
      t25*(lm*t15+f2*t15*mu+t19*mu)*g1/f2 - &
      f2*t10*t24/(g2+g1)*(mu*a1*a2+b2*b1*lm+f2*mu*b1*b2)*g2*g1
      sumi4=-t45
!
      return
      end
!# |------------------------------------------------------------|
!  | pmatr2.f                               date:  18 feb 94    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  |                                                            |
!  | pmatrx: compute propagation matrix for a constant layer    |
!  |                                                            |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine pmatr2(p,gmu,glame,grho,wfreq,fkn, &
      z0,z1)
      implicit real*8 (a-h,o-z)
!
      real*8 p(4,4),m,lm,k,z1,z0,r,w
      zero=0.00e+00
      f2=2.00e+00
!
!...get elastic parameters
      m=gmu
      lm=glame
      r=grho
      w=wfreq
      k=fkn
      dz=z1-z0
      a2=(lm+f2*m)/r
      b2=m/r
!
!...test for harmonic solution
      arg1=k*k-(w*w)/(b2)
      arg2=k*k-(w*w)/(a2)
      if(arg1.lt.zero .or. arg2.lt.zero) then
!
!========start g1 complex, g2 real block
      if(arg1.lt.zero .and. arg2.ge.zero) then
      g1=sqrt(abs(arg1))
      if(arg2.eq.zero) then
      g2=zero
      else
      g2=sqrt(arg2)
      endif
      sg1=sin(g1*dz)
      cg1=cos(g1*dz)
      sg2=sinh(g2*dz)
      cg2=cosh(g2*dz)
!
      a=(-k/g1)
      c=(m*(-g1*g1 + k*k))/g1
      e=-f2*k*m
      b=-g2/k
      d=f2*m*g2
      f=(k*k*lm - (lm+f2*m)*g2*g2)/k
      fme=f-e
      adcb=a*d-c*b
!
      p(1,1)=(f*cg1-e*cg2)/fme
      p(3,3)=p(1,1)
!
      p(1,2)=-(c*sg2+d*sg1)/adcb
      p(4,3)=-p(1,2)
!
      p(1,3)=(a*sg2+b*sg1)/adcb
!
      p(1,4)=-(cg1-cg2)/fme
      p(2,3)=-p(1,4)
!
      p(2,1)=(a*f*sg1-b*e*sg2)/fme
      p(3,4)=-p(2,1)
!
      p(2,2)=(a*d*cg1-b*c*cg2)/adcb
      p(4,4)=p(2,2)
!
      p(2,4)=-(a*sg1-b*sg2)/fme
!
      p(3,1)=(c*f*sg1-e*d*sg2)/fme
!
      p(3,2)=(c*d)*((cg1-cg2)/adcb)
      p(4,1)=-p(3,2)
!
      p(4,2)=-(c*f*sg2+e*d*sg1)/adcb
!
      endif
!...============>end complex  g1, real g2 block
!
!
!...============>totaly harmonic block
!....both g1 and g2 complex
      if(arg1.lt.zero .and. arg2.lt.zero) then
      g2=sqrt(abs(arg2))
      g1=sqrt(abs(arg1))
      sg2=sin(g2*dz)
      sg1=sin(g1*dz)
      cg2=cos(g2*dz)
      cg1=cos(g1*dz)
!
      a=(-k/g1)
      c=(m*(-g1*g1 + k*k))/g1
      e=-f2*k*m
      b=-g2/k
      d=f2*m*g2
      f=(k*k*lm + (lm+f2*m)*g2*g2)/k
      fme=f-e
      adcb=a*d-c*b
!
      p(1,1)=(f*cg1-e*cg2)/fme
      p(3,3)=p(1,1)
!
      p(1,2)=-(c*sg2+d*sg1)/adcb
      p(4,3)=-p(1,2)
!
      p(1,3)=(a*sg2+b*sg1)/adcb
!
      p(1,4)=-(cg1-cg2)/fme
      p(2,3)=-p(1,4)
!
      p(2,1)=(a*f*sg1+b*e*sg2)/fme
      p(3,4)=-p(2,1)
!
      p(2,2)=(a*d*cg1-b*c*cg2)/adcb
      p(4,4)=p(2,2)
!
      p(2,4)=-(a*sg1+b*sg2)/fme
!
      p(3,1)=(c*f*sg1+e*d*sg2)/fme
!
      p(3,2)=(c*d)*((cg1-cg2)/adcb)
      p(4,1)=-p(3,2)
!
      p(4,2)=-(c*f*sg2+e*d*sg1)/adcb
!
      endif
!...==============>end totaly harmonic (g1 and g2 complex)

!
      else
!
!...============>totaly non harmonic block
      if(arg2.eq.zero) then
      g2=zero
      else
      g2=sqrt(arg2)
      endif
      if(arg1.eq.zero) then
      g1=zero
      else
      g1=sqrt(arg1)
      endif
      sg2=sinh(g2*dz)
      sg1=sinh(g1*dz)
      cg2=cosh(g2*dz)
      cg1=cosh(g1*dz)
!
!      write(10,'(/"  alpha,beta,w,g1,g2 ",5(e10.4,1x))')
!     +a2,b2,w,g1,g2
!
      a=(-k/g1)
      c=(m*(g1*g1 + k*k))/g1
      e=-f2*k*m
      b=-g2/k
      d=f2*m*g2
      f=(k*k*lm - (lm+f2*m)*g2*g2)/k
      fme=f-e
      adcb=a*d-c*b
!
      p(1,1)=(f*cg1-e*cg2)/fme
      p(3,3)=p(1,1)
!
      p(1,2)=-(c*sg2-d*sg1)/adcb
      p(4,3)=-p(1,2)
!
      p(1,3)=(a*sg2-b*sg1)/adcb
!
      p(1,4)=-(cg1-cg2)/fme
      p(2,3)=-p(1,4)
!
      p(2,1)=(a*f*sg1-b*e*sg2)/fme
      p(3,4)=-p(2,1)
!
      p(2,2)=(a*d*cg1-b*c*cg2)/adcb
      p(4,4)=p(2,2)
!
      p(2,4)=-(a*sg1-b*sg2)/fme
!
      p(3,1)=(c*f*sg1-e*d*sg2)/fme
!
      p(3,2)=(c*d)*((cg1-cg2)/adcb)
      p(4,1)=-p(3,2)
!
      p(4,2)=-(c*f*sg2-e*d*sg1)/adcb
!
      endif
!...==============>end totaly non harmonic block
      return
      end
!# |------------------------------------------------------------|
!  | propg2.f                               date:  18 feb 94    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  |                                                            |
!  | propgt: propagate the motion/stress vector                 |
!  |         from depth z0 to depth z1                          |
!  |       modified to one set                                  |
!  |------------------------------------------------------------|
      subroutine propg2(zi,mu,lame,rho,wfreq,phasev,nlay, &
      p,rv1,ndim,i5,z0,z1)
      implicit real*8 (a-h,o-z)
      real*8 mu(ndim),lame(ndim),rho(ndim),zi(ndim)
      real*8 tmp(4),tmp2(4),rv1(i5)
      real*8 p(4,4)
!
      fkn=wfreq/phasev
!
      depth=(z0 + z1)/2.00d+00
!
!...get elastic parameters
      if(depth.lt.0.00d+00) depth=0.00d+00
      call gparms(rho,mu,lame,zi,nlay,ndim, &
      depth,grho,gmu,glame)
!
      call pmatr2(p,gmu,glame,grho,wfreq,fkn, &
      z0,z1)
!
!      write(10,'(" p",4(e10.4,1x))') (p(1,j),j=1,4)
!      write(10,'(" p",4(e10.4,1x))') (p(2,j),j=1,4)
!      write(10,'(" p",4(e10.4,1x))') (p(3,j),j=1,4)
!      write(10,'(" p",4(e10.4,1x))') (p(4,j),j=1,4)
      do 5 j=1,4
    5 tmp(j)=rv1(j)
      call matmlt(p,tmp,tmp2,4,4)
      do 10 j=1,4
   10 rv1(j)=tmp2(j)
      rv1(5)=z1
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 01 may 93    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | gparms: gets elastic parameters                            |
!  |                                                            |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine gparms(rho,mu,lame,zi,nlay,ndim, &
      depth,grho,gmu,glame)
      implicit real*8 (a-h,o-z)
      real*8 mu(ndim),lame(ndim),rho(ndim),zi(ndim)
!
!...determine current layer
      l1=1
      do 10 j=1,nlay
   10 if(depth.gt.zi(j)) l1=j
!
      if(l1.lt.nlay) then
      l2=l1+1
      dz=zi(l2) - zi(l1)
      sloper=(rho(l2)-rho(l1))/dz
      slopel=(lame(l2)-lame(l1))/dz
      slopem=(mu(l2)-mu(l1))/dz
      z=depth-zi(l1)
      grho=sloper*z + rho(l1)
      gmu=slopem*z + mu(l1)
      glame=slopel*z + lame(l1)
      else
      grho=rho(nlay)
      gmu=mu(nlay)
      glame=lame(nlay)
      endif
      return
      end
!# |------------------------------------------------------------|
!  | halfsp.f                               date:  18 feb 94    |
!  | author: p. michaels                                        |
!  |------------------------------------------------------------|
!  | halfsp: calculates the theoretical phase velocity and      |
!  |         displacement,stress eigenfunctions at depth, z.    |
!  |                                                            |
!  | r1=horizontal displacement      (aki & richards sign       |
!  | r2=vertical displacement         convention)               |
!  | r3=horizontal stress                                       |
!  | r4=vertical stress                                         |
!  | c=phase velocity for rayleigh waves                        |
!  |                                                            |
!  | rho=half space density                                     |
!  | fmu=half space shear modulus                               |
!  | flame= half space lame's constant                          |
!  |------------------------------------------------------------|
!
      subroutine halfsp(rho,fmu,flame,wfreq,z,c,r1,r2,r3,r4, &
      aa1,rv1,rv2)
      implicit real*8 (a-h,o-z)
      real*8 rv1(5),rv2(5)
      unity=1.0
!
!...compute shear velocity
      b=sqrt(fmu/rho)
      d=flame/fmu
      if(c.eq.0.0e+00) then
!
!   compute phase velocity
!...set up coef. for cubic equation
      a1=-8.0e+00
      a2=((8.0e+00)*(d*3.0e+00+4.0e+00))/(d+2.0e+00)
      a3=((-16.0e+00)*(d+1.0e+00))/(d+2.0e+00)
!
!...solve cubic equation
      pi=3.14159265358979311599796346854
      q=(a1*a1-3*a2)/9.0e+00
      r=(2.0e+00*a1*a1*a1-9.0e+00*a1*a2+27.0e+00*a3)/54.0e+00
      test=q*q*q - r*r
      if(test.ge.0.0e+00) then
      t1=acos(r/sqrt(q*q*q))
      t2=-2.0e+00*sqrt(q)
      t3=3.0e+00
      x1=t2*cos(t1/t3)-a1/t3
      x2=t2*cos((t1+2.0e+00*pi)/t3)-a1/t3
      x3=t2*cos((t1+4.0e+00*pi)/t3)-a1/t3
      else
      t1=(sqrt(r*r-q*q*q)+abs(r))**.333333333
      t3=3.0e+00
      x1=-sign(unity,r)*(t1+q/t1)-a1/t3
      endif
      c=sqrt(b*b*x1)
      endif
!
!...compute eigenfunctions
      fk=wfreq/c
      t1=(c*c)/(b*b)
      g1=sqrt(1.0-t1)*fk
      g2=sqrt(1.0-t1/(d+2.0e+00))*fk
      t2=t1-2.0e+00
      e1=exp(-g1*z)
      e2=exp(-g2*z)
!
!...p-wave set
      rv1(1)=fk*e2
      rv1(2)=g2*e2
      rv1(3)=fmu*fk*(-2.0e+00)*g2*e2
      rv1(4)=(fk*fk*fmu*t2)*e2
      rv1(5)=z
!
!...s-wave set
      rv2(1)=fk*(t2/2.0e+00)*e1
      rv2(2)=((fk*fk)/(2.0e+00*g1))*t2*e1
      rv2(3)=fmu*fk*((t2*t2*fk*fk)/(g1*2.0e+00))*e1
      rv2(4)=-(fk*fk*fmu*t2)*e1
      rv2(5)=z
!
      r1=aa1*fk*e2 + fk*(t2/2.0e+00)*e1
!
      r2=aa1*g2*e2 + ((fk*fk)/(2.0e+00*g1))*t2*e1
!
      r3=fmu*fk*(aa1*(-2.0e+00)*g2*e2  + &
      ((t2*t2*fk*fk)/(g1*2.0e+00))*e1)
!
      r4=(fk*fk*fmu*t2)*(aa1*e2 - e1)
!
!
      return
      end
!# |------------------------------------------------------------|
!  |                                         date: 02 mar 2006  |
!  | author: p. michaels           new version with ARMA wavelet|
!  |------------------------------------------------------------|
!  | moment: computes wavelet in frequency domain               |
!  |         corresponding to ground motion of source           |
!  |                                                            |
!  |        (minimum phase band limited by frange of phase vel.)|
!  |                                                            |
!  |------------------------------------------------------------|
      subroutine moment(npts,y,fmin,fmax,deltf,f,p, &
      xr,mapmat)
!
      implicit real*8 (a-h,o-z)
      REAL*4 sign
      REAL*4 xr(npts)
!cc      REAL*4 xi(npts),amp(npts),phz(npts)
      REAL*4 p(npts),f(npts)
      REAL*4 samin
      complex*8  y(npts)
      character*70 title
      integer  hpass,lpass,bpass
      real*4 f1,f2
      hpass=2
      lpass=0
      bpass=1
!
      write(*,*) npts
      samin=real(1.e+00/(dfloat(npts)*deltf),4)
      pi=3.14159265358979311599796346854
      pi2=2.0*pi
      np2=npts/2
      np2p1=np2+1

!
!...clear trace
      do 10 j=1,npts
      f(j)=0.0
      p(j)=0.0
      xr(j)=0.0
   10 y(j)=cmplx(0.0,0.0)
!
!...load time domain function, impulse in f(t)
!...filter with high pass minimum phase filter
!...output in p(t)
      f(1)=1.0
      ifaz=1
      npoles=10
      bwidth=fmin
      f1=real(fmin*4.,4)
!      write(6,'("debug: fmin,fmax=",f10.4,2x,f10.4)') fmin,fmax
! bandwidth not used if high or low pass
! CAUTION: float arguments must be real*4
      call ARMA(npts,f,p,hpass,f1,bwidth,npoles,samin,ifaz)
!
!...filter with low pass minimum phase filter
!...input in p(t), output in g(t)
      ifaz=1
      f2=real(fmax/2,4)
!      write(6,'("debug: f1,f2=",f10.4,2x,f10.4)') f1,f2
! CAUTION: float arguments must be real*4
      call ARMA(npts,p,xr,lpass,f2,bwidth,npoles,samin,ifaz)
!
!...xr(t) now has the time domain wavelet
!...truncate at npts/2, zero out 2nd half to avoid circular convolution
      do 340 j=np2p1,npts
      xr(j)=0.0
  340 continue
      do 20, j=1,npts
      y(j)=cmplx(xr(j),0.0)
   20 continue
!
!...forward transform compute spectrum of wavelet
      sign=-1.0
      call nlogn(npts,y,sign)
!
      write(title,900) npts,fmin,fmax,samin
  900 format(' npts=',i5,' fmin= ',f8.4,' fmax=',f8.4,' samin= ',f7.4)
      nlst=100
      nlst=npts/2
      call tlist(nlst,xr,title)
!
!...open file and write out wavelet in reflectivity format or scilab
      if (mapmat.eq.1) then
      open(unit=7,file='m0.sci',access='sequential')
      else
      open(unit=7,file='m0.m',access='sequential')
      endif
!cc      write(7,901) (xr(j),j=1,npts)
!cc 901 format(8f10.4)
      if (mapmat.eq.1) then
      write(7,*) 'clear '
      write(7,*) '// generated by waves.f <pm@cgiss.boisestate.edu> '
      write(7,*) '// scalar source moment, plot wavelet '
      write(7,*) ' '
      else
      write(7,*) 'clear '
      write(7,*) '%  generated by waves.f <pm@cgiss.boisestate.edu> '
      write(7,*) '%  scalar source moment, plot wavelet '
      write(7,*) ' '
      endif
      write(7,*) 'data=[...'
      if (npts.ge.2048) then
      nplt=2048
      else
      nplt=npts
      endif
      nplt=nplt/2
      do 7 j=1,nplt
      tim=float(j-1)*samin
      write(7,901) tim,xr(j)
    7 continue
  901 format(1x,f10.5,2x,e14.7,' ;...')
      if (mapmat.eq.1) then
      write(7,*) ']; '
      write(7,*) 'tm=data(:,1); '
      write(7,*) 'mo=data(:,2); '
      write(7,*) 'Mo=fft(mo,-1); '
      write(7,*) 'npts=length(mo); '
      write(7,*) 'k=npts/2; '
      write(7,*) 'dt=tm(2)-tm(1); '
      write(7,*) 'frq=0:npts-1; '
      write(7,*) 'frq=frq/(npts*dt); '
      write(7,*) 'subplot(211) '
      write(7,*) 'plot2d(tm,mo,2); '
      write(7,*) 'xtitle("Rayleigh Wave source wavelet","Time (s)"); '
      write(7,*) 'xgrid(3); '
      write(7,*) 'subplot(212)'
      write(7,*) 'plot2d(frq(1:k),abs(Mo(1:k)),5) '
      write(7,*) 'xtitle("Amplitude Spectrum","Frequency (Hz)"); '
      write(7,*) 'xgrid(3); '
      else
      write(7,*) ']; '
      write(7,*) 'tm=data(:,1); '
      write(7,*) 'mo=data(:,2); '
      write(7,*) 'Mo=fft(mo); '
      write(7,*) 'npts=length(mo); '
      write(7,*) 'k=npts/2; '
      write(7,*) 'dt=tm(2)-tm(1); '
      write(7,*) 'frq=0:npts-1; '
      write(7,*) 'frq=frq/(npts*dt); '
      write(7,*) 'subplot(211) '
      write(7,*) 'plot(tm,mo,''-b'',''linewidth'',2); '
      write(7,*) 'title(''Rayleigh Wave source wavelet''); '
      write(7,*) 'xlabel(''Time (s)'');'
      write(7,*) 'grid on; '
      write(7,*) 'subplot(212)'
      write(7,*) 'plot(frq(1:k),abs(Mo(1:k)),''-r'',''linewidth'',2) '
      write(7,*) 'title(''Amplitude Spectrum''); '
      write(7,*) 'xlabel(''Frequency Hz'');'
      write(7,*) 'grid on; '
      write(7,*) '%print(''m0.fig'',''-color'',''-dfig'')'
      endif
!
!      tzero=0.0
!      ilin=1
!      call flist(npts,y,title,samin,tzero,xr,xi,amp,phz,ilin)
!
      close(7)
      return
      end
!# |------------------------------------------------------------|
!  | author: p. michaels                date: 01 may 93         |
!  |------------------------------------------------------------|
!  | segy: sets up header and outputs a trace in segy format    |
!  |                                                            |
!  | Modified: 3 Oct 1999 added irvsel                          |
!  |  irvsel  0=vertical component traces                       |
!  |          1=horizontal component traces                     |
!  |                                                            |
!  |------------------------------------------------------------|
!
      subroutine segy(npts,trace,fsamin,irvsel, &
      jrec,frange,rx,ry,rz,sx,sy,sz,iyear,iday,line,igeoor) 
#ifdef  WIN
      include '../include/bsegy.f90'
#else
      include '../Fort/include/bsegy.f90' 
#endif
      real*8 rx(*),ry(*),rz(*),frange(*)
      real*8 fsamin,sx,sy,sz
      real*4 fsamin4
      REAL*4 trace(*)
      character*4 line,igeoor
      character*40 outfil
      integer*2 iyear,iday
      data icall11 /0/

      common /header/ hd
!
      fsamin4=real(fsamin,4)
      mtimee=0
      ttrace=0
      write(spname,'("0001")')
      write(rstnam,'(i4.4)') jrec
      syear=92
      geover=0
      daca=0
      errlt=0
      tsnt=jrec
      gc=1
!
!...set scalar multipliers
      deltx=real(frange(2)-frange(1),4)
      smul1=1
      smul2=1
      if(ABS(deltx).lt.100.0) then
      smul1=-10
      smul2=-10
      endif
      if(ABS(deltx).lt.10.0) then
      smul1=-100
      smul2=-100
      endif
      if(ABS(deltx).lt.1.0) then
      smul1=-1000
      smul2=-1000
      endif
!

      tsnl=jrec
      idist=int2(frange(jrec))
      irel=int2(rz(jrec)*float(abs(smul2)))
      ishd=int2(sz*float(abs(smul2)))
      ishlo=int2(sx*float(abs(smul2)))
      ishla=int2(sy*float(abs(smul2)))
      irlo=int2(rx(jrec)*float(abs(smul2)))
      irla=int2(ry(jrec)*float(abs(smul2)))
!
!...output only first half of the trace, since second
!...half is a buffer to prevent wraparound and may not
!...include all modes (some of which could be aliase
!...in phase spectrum)
      length=int(npts/2,2)
!
      isi=int(fsamin*1.0e+06 + 0.5,2 )
      geoor=igeoor
      tnofr=jrec
      lineid=line
!
      ofrn=1
      espn=1
      tic=1
      ishe=0
      delr=0
      dels=0
      cunits=1
      gaint=3
      tcorr=1
      tyear=iyear
      tday=iday
!
!
! open file with the correct record length
! if is first trace
        nlen=240+4*length
        if(jrec.eq.1) then
        if(irvsel.eq.1) then
        write(outfil,'("wavR.seg")')
        open (11,file=outfil,access='direct',recl=240)
        else
        write(outfil,'("wavV.seg")')
        open (11,file=outfil,access='direct',recl=240)
        endif
        close(11,status='delete')
!       if(irvsel.eq.1) then
!       open (11,file='wavesR.segy',access='direct',recl=nlen)
!       else
!       open (11,file='waves.segy',access='direct',recl=nlen)
!       endif
        endif
!
      iunit11=11
      call BSEGOUT(iunit11, &
      icall11,length,trace,fsamin4,outfil,jrec)
        return
        end
!# |------------------------------------------------------------|
!  | cubic.f                                date: 01 may 93     |
!  | author: p. michaels (after press et.al. numerical recipies)|
!  |------------------------------------------------------------|
!  | cubic: solves a cubic equation of the form:                |
!  |         x^3 + (a1)*x^2 + (a2)*x + a3 = 0                   |
!  |       for real roots: x1,x2,x3                             |
!  |       nroots=number of real roots                          |
!  |------------------------------------------------------------|
!
      subroutine cubic(a1,a2,a3,x1,x2,x3,nroots)
      implicit real*8 (a-h,o-z)
      unity=1.0
!
!...solve cubic equation
      pi=3.14159265358979311599796346854
      q=(a1*a1-3*a2)/9.0e+00
      r=(2.0e+00*a1*a1*a1-9.0e+00*a1*a2+27.0e+00*a3)/54.0e+00
      test=q*q*q - r*r
      if(test.ge.0.0e+00) then
      t1=acos(r/sqrt(q*q*q))
      t2=-2.0e+00*sqrt(q)
      t3=3.0e+00
      x1=t2*cos(t1/t3)-a1/t3
      x2=t2*cos((t1+2.0e+00*pi)/t3)-a1/t3
      x3=t2*cos((t1+4.0e+00*pi)/t3)-a1/t3
      nroots=3
      else
      t1=(sqrt(r*r-q*q*q)+abs(r))**.333333333
      t3=3.0e+00
      x1=-sign(unity,r)*(t1+q/t1)-a1/t3
      nroots=1
      endif
      return
      end
